@@ -1,3 +1,23 @@
+/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
+ *
+ * This file is part of slrn.
+ *
+ * Slrn is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ * 
+ * Slrn is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with Slrn; see the file COPYING.  If not, write to the Free
+ * Software Foundation, 59 Temple Place - Suite 330, 
+ * Boston, MA  02111-1307, USA.
+ */
+
 /* It is too bad that this cannot be done at the preprocessor level.
  * Unfortunately, C is not completely portable yet.  Basically the #error
  * directive is the problem.
@@ -6,9 +26,23 @@
 
 #include <stdio.h>
 #ifdef VMS
-#include <ssdef.h>
+# include <ssdef.h>
 #endif
+
+#ifdef HAVE_STDLIB_H
+# include <stdlib.h>
+#endif
+
 #include <slang.h>
+#include "jdmacros.h"
+
+#ifdef VMS
+# define SUCCESS	1
+# define FAILURE	2
+#else
+# define SUCCESS	0
+# define FAILURE	1
+#endif
 
 static char *make_version (unsigned int v)
 {
@@ -33,7 +67,7 @@ int main (int argc, char **argv)
    if ((argc < 3) || (argc > 4))
      {
 	fprintf (stderr, "Usage: %s <PGM> <SLANG-VERSION> <SUGG VERSION>\n", argv[0]);
-	return 1;
+	return FAILURE;
      }
 #ifndef SLANG_VERSION
    sl_version = 0;
@@ -46,16 +80,12 @@ int main (int argc, char **argv)
    else sug_version = sl_version;
    
    
-   ret = 0;
+   ret = SUCCESS;
    if (sl_version < min_version)
      {
 	fprintf (stderr, "This version of %s requires slang version %s.\n",
 		 argv[1], make_version(min_version));
-#ifdef VMS
-	ret = SS$_ABORT;
-#else
-	ret = 1;
-#endif
+	ret = FAILURE;
      }
    
    if (sl_version < sug_version)
