@@ -1,3 +1,23 @@
+/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
+ *
+ * This file is part of slrn.
+ *
+ * Slrn is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ * 
+ * Slrn is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with Slrn; see the file COPYING.  If not, write to the Free
+ * Software Foundation, 59 Temple Place - Suite 330, 
+ * Boston, MA  02111-1307, USA.
+ */
+
 #include "config.h"
 #include "slrnfeat.h"
 
@@ -43,7 +63,7 @@ int slrn_edit_score (Slrn_Header_Type *h, char *newsgroup)
    
    if (Slrn_Prefix_Arg_Ptr == NULL)
      {
-	ch = slrn_get_response ("SsFfRrEeCc\007", "Pick Score type: S-ubject, F-rom, R-eferences, E-dit, C-ancel");
+	ch = slrn_get_response ("SsFfRrEeCc\007", "Pick Score type: \001Subject, \001From, \001References, \001Edit, \001Cancel");
 	if (ch == 7) return -1;
 	ch |= 0x20;
 	if (ch == 'c') return -1;
@@ -53,13 +73,13 @@ int slrn_edit_score (Slrn_Header_Type *h, char *newsgroup)
 	  return -1;
 	if (score == 0) return -1;
 	
-	if ('a' == (0x20 | slrn_get_response ("TtaA", "Which newsgroups: T-his group, A-ll groups")))
+	if ('a' == (0x20 | slrn_get_response ("TtaA", "Which newsgroups: \001This group, \001All groups")))
 	  ng = "*";
 
 	while (1)
 	  {
 	     *qregexp = 0;
-	     if (-1 == slrn_read_input ("Expires (MM/DD/YYYY or DD-MM-YYYY or leave blank)", NULL, qregexp, 1))
+	     if (-1 == slrn_read_input ("Expires (MM/DD/YYYY or DD-MM-YYYY or leave blank)", NULL, qregexp, 1, 0))
 	       return -1;
 	     if (*qregexp)
 	       {
