@@ -1,8 +1,25 @@
 /* -*- mode: C; mode: fold -*- */
 
-/*  Copyright (c) 1995, 1996 John E. Davis (davis@space.mit.edu)
- *  All rights reserved.
+/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
+ *
+ * This file is part of slrn.
+ *
+ * Slrn is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ * 
+ * Slrn is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with Slrn; see the file COPYING.  If not, write to the Free
+ * Software Foundation, 59 Temple Place - Suite 330, 
+ * Boston, MA  02111-1307, USA.
  */
+
 #include "config.h"
 #include "slrnfeat.h"
 
@@ -43,6 +60,7 @@
 #include "menu.h"
 #include "util.h"
 #include "startup.h"
+#include "slrndir.h"
 
 /*}}}*/
 
@@ -56,11 +74,12 @@ int Slrn_Write_Newsrc_Flags = 0;       /* if 1, do not save unsubscribed
 					* if 2, do not save new unsubscribed.
 					*/
 
+int Slrn_Display_Cursor_Bar;
 int Slrn_Group_Description_Column = 40;/* column where group descr start */
 char *Slrn_Group_Help_Line;
 
-SLscroll_Window_Type Slrn_Group_Window;
 Slrn_Group_Type *Slrn_Group_Current_Group;
+static SLscroll_Window_Type Group_Window;
 static Slrn_Group_Type *Groups;
 
 int Slrn_Group_Display_Descriptions = 1;
@@ -547,24 +566,24 @@ static int add_group (char *name, unsigned int len, /*{{{*/
 
 static void find_line_num (void) /*{{{*/
 {
-   Slrn_Group_Window.lines = (SLscroll_Type *) Groups;
-   Slrn_Group_Window.current_line = (SLscroll_Type *) Slrn_Group_Current_Group;
-   (void) SLscroll_find_line_num (&Slrn_Group_Window);
+   Group_Window.lines = (SLscroll_Type *) Groups;
+   Group_Window.current_line = (SLscroll_Type *) Slrn_Group_Current_Group;
+   (void) SLscroll_find_line_num (&Group_Window);
 }
 
 /*}}}*/
 static void init_group_win_struct (void) /*{{{*/
 {
-   Slrn_Group_Window.nrows = SLtt_Screen_Rows - 3;
-   Slrn_Group_Window.hidden_mask = GROUP_HIDDEN;
-   Slrn_Group_Window.current_line = (SLscroll_Type *) Slrn_Group_Current_Group;
-   Slrn_Group_Window.cannot_scroll = SLtt_Term_Cannot_Scroll;
-   Slrn_Group_Window.lines = (SLscroll_Type *) Groups;
-   Slrn_Group_Window.border = 1;
+   Group_Window.nrows = SLtt_Screen_Rows - 3;
+   Group_Window.hidden_mask = GROUP_HIDDEN;
+   Group_Window.current_line = (SLscroll_Type *) Slrn_Group_Current_Group;
+   Group_Window.cannot_scroll = SLtt_Term_Cannot_Scroll;
+   Group_Window.lines = (SLscroll_Type *) Groups;
+   Group_Window.border = 1;
    if (Slrn_Scroll_By_Page)
      {
 	/* Slrn_Group_Window.border = 0; */
-	Slrn_Group_Window.cannot_scroll = 2;
+	Group_Window.cannot_scroll = 2;
      }
    find_line_num ();
 }
@@ -586,7 +605,7 @@ static void free_all_groups (void) /*{{{*/
      }
    Groups = NULL;
    Slrn_Group_Current_Group = NULL;
-   SLMEMSET((char *) &Slrn_Group_Window, 0, sizeof (SLscroll_Window_Type));
+   SLMEMSET((char *) &Group_Window, 0, sizeof (SLscroll_Window_Type));
    
    for (i = 0; i < GROUP_HASH_TABLE_SIZE; i++) Group_Hash_Table[i] = NULL;
 }
@@ -610,7 +629,7 @@ static SLRegexp_Type *read_group_regexp (char *prompt, char *origpat) /*{{{*/
 {
    static char pattern[256];
    
-   if (slrn_read_input (prompt, NULL, pattern, 1) <= 0) return NULL;
+   if (slrn_read_input (prompt, NULL, pattern, 1, 0) <= 0) return NULL;
    
    if (origpat != NULL)
      strcpy (origpat, pattern);
@@ -730,16 +749,16 @@ int slrn_group_search (char *str) /*{{{*/
 /*}}}*/
 unsigned int slrn_group_up_n (unsigned int n) /*{{{*/
 {
-   n = SLscroll_prev_n (&Slrn_Group_Window, n);
-   Slrn_Group_Current_Group = (Slrn_Group_Type *) Slrn_Group_Window.current_line;
+   n = SLscroll_prev_n (&Group_Window, n);
+   Slrn_Group_Current_Group = (Slrn_Group_Type *) Group_Window.current_line;
    return n;
 }
 
 /*}}}*/
 unsigned int slrn_group_down_n (unsigned int n) /*{{{*/
 {
-   n = SLscroll_next_n (&Slrn_Group_Window, n);
-   Slrn_Group_Current_Group = (Slrn_Group_Type *) Slrn_Group_Window.current_line;
+   n = SLscroll_next_n (&Group_Window, n);
+   Slrn_Group_Current_Group = (Slrn_Group_Type *) Group_Window.current_line;
    return n;
 }
 
@@ -747,7 +766,7 @@ unsigned int slrn_group_down_n (unsigned int n) /*{{{*/
 
 int slrn_group_select_group (void) /*{{{*/
 {
-   int min, max, n, max_available;
+   int min, max, n, max_available, last_n;
    int ret;
    Slrn_Range_Type *r;
    int prefix;
@@ -759,10 +778,20 @@ int slrn_group_select_group (void) /*{{{*/
      }
    else prefix = 0;
    
+   if (Slrn_Group_Current_Group == NULL) return -1;
+
+   last_n = Slrn_Group_Current_Group->unread;
+
    if (-1 == group_sync_group_with_server (Slrn_Group_Current_Group, &min, &max))
      return -1;
    
    n = Slrn_Group_Current_Group->unread;
+
+#if 1
+   if ((prefix == 0) && (n == 0) && (n != last_n))
+     return -1;
+#endif
+
    max_available = Slrn_Group_Current_Group->range.max - Slrn_Group_Current_Group->range.min + 1;
 
    if ((prefix & 1) || (n == 0))
@@ -862,9 +891,9 @@ static void group_pagedown (void) /*{{{*/
 {
    Slrn_Full_Screen_Update = 1;
    
-   if (-1 == SLscroll_pagedown (&Slrn_Group_Window))
+   if (-1 == SLscroll_pagedown (&Group_Window))
      slrn_error ("End of Buffer.");
-   Slrn_Group_Current_Group = (Slrn_Group_Type *) Slrn_Group_Window.current_line;
+   Slrn_Group_Current_Group = (Slrn_Group_Type *) Group_Window.current_line;
 }
 
 /*}}}*/
@@ -873,10 +902,10 @@ static void group_pageup (void) /*{{{*/
 {
    Slrn_Full_Screen_Update = 1;
    
-   if (-1 == SLscroll_pageup (&Slrn_Group_Window))
+   if (-1 == SLscroll_pageup (&Group_Window))
      slrn_error ("Top of Buffer.");
    
-   Slrn_Group_Current_Group = (Slrn_Group_Type *) Slrn_Group_Window.current_line;
+   Slrn_Group_Current_Group = (Slrn_Group_Type *) Group_Window.current_line;
 }
 
 /*}}}*/
@@ -931,16 +960,16 @@ static void group_search_cmd (void) /*{{{*/
    g = Slrn_Group_Current_Group;
    if (g == NULL) return;
    
-   if (slrn_read_input ("Search", search_str, NULL, 1) <= 0) return;
+   if (slrn_read_input ("Search", search_str, NULL, 1, 0) <= 0) return;
    
-   n = Slrn_Group_Window.line_num;
+   n = Group_Window.line_num;
    if (0 == slrn_group_search (search_str))
      {
         slrn_error ("Not found.");
 	return;
      }
    
-   if (n > Slrn_Group_Window.line_num)
+   if (n > Group_Window.line_num)
      slrn_message ("Search wrapped.");
 }
 
@@ -951,7 +980,7 @@ static void add_group_cmd (void) /*{{{*/
    char group[256];
    
    *group = 0;
-   if (slrn_read_input ("Add group", NULL, group, 1) > 0)
+   if (slrn_read_input ("Add group", NULL, group, 1, 0) > 0)
      {
 	if (!find_group (group)
 	    && (Slrn_List_Active_File == 0))
@@ -1059,10 +1088,10 @@ static void move_group_cmd (void) /*{{{*/
 	  }
 
 	/* For a recenter if possible. */
-	/* if (Slrn_Group_Window.top_window_line == Slrn_Group_Window.current_line) */
-	Slrn_Group_Window.top_window_line = NULL;
+	/* if (Group_Window.top_window_line == Group_Window.current_line) */
+	Group_Window.top_window_line = NULL;
 	
-	slrn_update_screen (1);
+	slrn_update_screen ();
      }
 }
 
@@ -1362,14 +1391,21 @@ void slrn_post_cmd (void) /*{{{*/
 	slrn_error ("Posting not allowed.");
 	return;
      }
+
+#if SLRN_HAS_SLANG
+   SLang_run_hooks ("post_hook", 0);
+   if (SLang_Error)
+     return;
+#endif
+
    if (Slrn_User_Wants_Confirmation && (Slrn_Batch == 0) &&
        (slrn_get_yesno (1, "Are you sure that you want to post") <= 0))
      return;
    
    if (Slrn_Group_Current_Group == NULL) name = ""; else name = Slrn_Group_Current_Group->name;
    strcpy (group, name);
-   if (slrn_read_input ("Newsgroup", NULL, group, 1) <= 0) return;
-   *subj = 0; if (slrn_read_input ("Subject", NULL, subj, 1) <= 0) return;
+   if (slrn_read_input ("Newsgroup", NULL, group, 1, 1) <= 0) return;
+   *subj = 0; if (slrn_read_input ("Subject", NULL, subj, 1, 0) <= 0) return;
    
    ret = slrn_post (group, subj, dist);
    if (ret < 0)
@@ -1386,7 +1422,7 @@ static void toggle_scoring (void) /*{{{*/
      return;
 
    switch (slrn_get_response ("FfSsNnCc\007",
-			      "Select scoring mode: F-ull, S-imple, N-one, C-ancel"))
+			      "Select scoring mode: \001Full, \001Simple, \001None, \001Cancel"))
      {
       case 'F':
       case 'f':
@@ -1442,16 +1478,26 @@ static void slrn_group_hup (int sig)
 static void enter_group_mode_hook (void)
 {
 #if SLRN_HAS_SLANG
-   SLang_run_hooks ("group_mode_hook", NULL, NULL);
+   SLang_run_hooks ("group_mode_hook", 0);
 #endif
 }
 
+static void group_winch_sig (int old_r, int old_c)
+{
+   (void) old_r; (void) old_c;
+
+   if (SLtt_Screen_Rows > 3)
+     Group_Window.nrows = SLtt_Screen_Rows - 3;
+   else
+     Group_Window.nrows = 1;
+}
+
 
 static Slrn_Mode_Type Group_Mode_Cap =
 {
    NULL,
    group_update_screen,		       /* redraw */
-   NULL,			       /* sig winch hook */
+   group_winch_sig,		       /* sig winch hook */
    slrn_group_hup,		       /* hangup hook */
    enter_group_mode_hook,	       /* enter_mode_hook */
    SLRN_GROUP_MODE
@@ -1493,6 +1539,7 @@ static SLKeymap_Function_Type Group_Functions [] = /*{{{*/
    A_KEY("up", group_up),
    A_KEY("repeat_last_key", slrn_repeat_last_key),
    A_KEY("transpose_groups", transpose_groups),
+   A_KEY("post_postponed", slrn_post_postponed),
    A_KEY(NULL, NULL)
 };
 
@@ -1576,6 +1623,20 @@ static void group_mouse_right (void)
 
 /*}}}*/
 
+#define USE_TEST_FUNCTION 0
+#if USE_TEST_FUNCTION
+static void test_function (void)
+{
+   char *file;
+   file = slrn_browse_dir (".");
+   if (file != NULL) 
+     {
+	slrn_message (file);
+	slrn_free (file);
+     }
+}
+#endif
+     
 void slrn_init_group_mode (void) /*{{{*/
 {
    char  *err = "Unable to create group keymap!";
@@ -1606,7 +1667,7 @@ void slrn_init_group_mode (void) /*{{{*/
    SLkm_define_key  ("\033<", (FVOID_STAR) group_bob, Slrn_Group_Keymap);
    SLkm_define_key  ("^D", (FVOID_STAR) group_pagedown, Slrn_Group_Keymap);
    SLkm_define_key  ("^V", (FVOID_STAR) group_pagedown, Slrn_Group_Keymap);
-#ifdef __os2__
+#if defined(IBMPC_SYSTEM)
    SLkm_define_key  ("^@Q", (FVOID_STAR) group_pagedown, Slrn_Group_Keymap);
    SLkm_define_key  ("\xE0Q", (FVOID_STAR) group_pagedown, Slrn_Group_Keymap);
    SLkm_define_key  ("^@I", (FVOID_STAR) group_pageup, Slrn_Group_Keymap);
@@ -1630,6 +1691,7 @@ void slrn_init_group_mode (void) /*{{{*/
    SLkm_define_key  (" ", (FVOID_STAR) select_group_cmd, Slrn_Group_Keymap);
    SLkm_define_key  (".", (FVOID_STAR) slrn_repeat_last_key, Slrn_Group_Keymap);
    SLkm_define_key  ("P", (FVOID_STAR) slrn_post_cmd, Slrn_Group_Keymap);
+   SLkm_define_key  ("\033P", (FVOID_STAR) slrn_post_postponed, Slrn_Group_Keymap);
    SLkm_define_key  ("?", (FVOID_STAR) slrn_group_help, Slrn_Group_Keymap);
    SLkm_define_key  ("\r", (FVOID_STAR) select_group_cmd, Slrn_Group_Keymap);
    SLkm_define_key  ("q", (FVOID_STAR) slrn_group_quit, Slrn_Group_Keymap);
@@ -1638,7 +1700,7 @@ void slrn_init_group_mode (void) /*{{{*/
    SLkm_define_key  ("^R", (FVOID_STAR) slrn_redraw, Slrn_Group_Keymap);
    SLkm_define_key  ("^L", (FVOID_STAR) slrn_redraw, Slrn_Group_Keymap);
    SLkm_define_key  ("^P", (FVOID_STAR) group_up, Slrn_Group_Keymap);
-#ifdef __os2__
+#if defined(IBMPC_SYSTEM)
    SLkm_define_key  ("^@H", (FVOID_STAR) group_up, Slrn_Group_Keymap);
    SLkm_define_key  ("\xE0H", (FVOID_STAR) group_up, Slrn_Group_Keymap);
    SLkm_define_key  ("^@P", (FVOID_STAR) group_down, Slrn_Group_Keymap);
@@ -1659,7 +1721,9 @@ void slrn_init_group_mode (void) /*{{{*/
    SLkm_define_key  ("\033[M\040", (FVOID_STAR) group_mouse_left, Slrn_Group_Keymap);
    SLkm_define_key  ("\033[M\041", (FVOID_STAR) group_mouse_middle, Slrn_Group_Keymap);
    SLkm_define_key  ("\033[M\042", (FVOID_STAR) group_mouse_right, Slrn_Group_Keymap);
-   
+#if USE_TEST_FUNCTION
+   SLkm_define_key  ("y", (FVOID_STAR) test_function, Slrn_Group_Keymap);
+#endif
    if (SLang_Error) slrn_exit_error (err);
 }
 
@@ -1669,16 +1733,13 @@ void slrn_init_group_mode (void) /*{{{*/
 
 int slrn_select_group_mode (void) /*{{{*/
 {
-   Slrn_Current_Mode = &Group_Mode_Cap;
-   
    init_group_win_struct ();
    
    Last_Cursor_Row = 0;
    group_quick_help ();
    Slrn_Full_Screen_Update = 1;
-   
-   enter_group_mode_hook ();
-
+ 
+   slrn_push_mode (&Group_Mode_Cap);
    return 0;
 }
 
@@ -1720,7 +1781,7 @@ void slrn_get_group_descriptions (void) /*{{{*/
 #ifdef VMS
    sprintf (file, "%s-dsc", Slrn_Newsrc_File);
 #else
-# ifdef __os2__
+# if defined(IBMPC_SYSTEM)
    sprintf (file, "ds-%s", Slrn_Newsrc_File);
 # else
    sprintf (file, "%s.dsc", Slrn_Newsrc_File);
@@ -1796,7 +1857,7 @@ int slrn_read_group_descriptions (void) /*{{{*/
 #ifdef VMS
    sprintf (file, "%s-dsc", Slrn_Newsrc_File);
 #else
-# ifdef __os2__
+# if defined(IBMPC_SYSTEM)
    sprintf (file, "ds-%s", Slrn_Newsrc_File);
 # else
    sprintf (file, "%s.dsc", Slrn_Newsrc_File);
@@ -1855,11 +1916,11 @@ void slrn_check_new_groups (int create_flag) /*{{{*/
 #ifdef VMS
    sprintf (file, "%s-time", Slrn_Newsrc_File);
 #else
-#ifdef __os2__
-   sprintf (file, "tm-%s", Slrn_Newsrc_File);
-#else
+# ifdef SLRN_USE_OS2_FAT
+   slrn_os2_make_fat (file, Slrn_Newsrc_File, ".tim");
+# else
    sprintf (file, "%s.time", Slrn_Newsrc_File);
-#endif
+# endif
 #endif
    
    
@@ -2198,7 +2259,7 @@ int slrn_read_newsrc (int create_flag) /*{{{*/
    
    if (create_flag)
      {
-	if (Slrn_Group_Window.num_lines == 0)
+	if (Group_Window.num_lines == 0)
 	  {
 	     Slrn_Group_Current_Group = Groups;
 	     while (Slrn_Group_Current_Group != NULL)
@@ -2255,7 +2316,11 @@ int slrn_write_newsrc (void) /*{{{*/
 #ifdef VMS
    sprintf (backup_file, "%s-bak", file);
 #else
+# ifdef SLRN_USE_OS2_FAT
+   slrn_os2_make_fat (backup_file, file, ".bak");
+# else
    sprintf (backup_file, "%s~", file);
+# endif
 #endif
    
    /* Create a temp backup file.  Delete it later if user 
@@ -2279,7 +2344,7 @@ int slrn_write_newsrc (void) /*{{{*/
      }
    
 #ifdef __unix__
-# ifndef __os2__
+# if !defined(IBMPC_SYSTEM)
    /* Try to preserve .newsrc permissions and owner/group */
 #  ifndef S_IRUSR
 #   define S_IRUSR 0400
@@ -2434,7 +2499,7 @@ static void group_quick_help (void) /*{{{*/
 static void group_update_screen (void) /*{{{*/
 {
    Slrn_Group_Type *g;
-   int height = (int) Slrn_Group_Window.nrows;
+   int height = (int) Group_Window.nrows;
    int row;
    
    /* erase last cursor */
@@ -2444,13 +2509,13 @@ static void group_update_screen (void) /*{{{*/
 	SLsmg_write_string ("  ");
      }
 
-   g = (Slrn_Group_Type *) Slrn_Group_Window.top_window_line;
-   (void) SLscroll_find_top (&Slrn_Group_Window);
+   g = (Slrn_Group_Type *) Group_Window.top_window_line;
+   (void) SLscroll_find_top (&Group_Window);
    
-   if (g != (Slrn_Group_Type *) Slrn_Group_Window.top_window_line)
+   if (g != (Slrn_Group_Type *) Group_Window.top_window_line)
      {
 	Slrn_Full_Screen_Update = 1;
-	g = (Slrn_Group_Type *) Slrn_Group_Window.top_window_line;
+	g = (Slrn_Group_Type *) Group_Window.top_window_line;
      }
 
    for (row = 0; row < height; row++)
@@ -2513,18 +2578,25 @@ static void group_update_screen (void) /*{{{*/
    if (Slrn_Server_Obj->sv_name != NULL)
      SLsmg_write_string (Slrn_Server_Obj->sv_name);
 
-   slrn_print_percent (SLtt_Screen_Rows - 2, 60, &Slrn_Group_Window);
+   slrn_print_percent (SLtt_Screen_Rows - 2, 60, &Group_Window);
    
    if (Slrn_Use_Mouse) slrn_update_group_menu ();
    else slrn_update_top_status_line ();
    
    if (Slrn_Message_Present == 0) group_quick_help ();
    
-   Last_Cursor_Row = 1 + Slrn_Group_Window.window_row;   
+   Last_Cursor_Row = 1 + Group_Window.window_row;   
    SLsmg_gotorc (Last_Cursor_Row, 0);
    
    slrn_set_color (CURSOR_COLOR);
-   SLsmg_write_string ("->");
+
+#if SLANG_VERSION > 10003
+   if (Slrn_Display_Cursor_Bar)
+     SLsmg_set_color_in_region (CURSOR_COLOR, Last_Cursor_Row, 0, 1, SLtt_Screen_Cols);
+   else
+#endif
+     SLsmg_write_string ("->");
+
    slrn_set_color (0);
    Slrn_Full_Screen_Update = 0;
 }
