@@ -1,3 +1,23 @@
+/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
+ *
+ * This file is part of slrn.
+ *
+ * Slrn is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ * 
+ * Slrn is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with Slrn; see the file COPYING.  If not, write to the Free
+ * Software Foundation, 59 Temple Place - Suite 330, 
+ * Boston, MA  02111-1307, USA.
+ */
+
 extern int slrn_select_group_mode (void);
 extern void slrn_group_quit (void);
 extern void slrn_select_next_group (void);
@@ -68,5 +88,4 @@ extern int Slrn_Query_Group_Cutoff;
 extern int Slrn_Prompt_Next_Group;
 extern int Slrn_Use_Xgtitle;
 extern int Slrn_Write_Newsrc_Flags;
-
-extern SLscroll_Window_Type Slrn_Group_Window;
+extern int Slrn_Display_Cursor_Bar;
