@@ -1,4 +1,24 @@
 /* -*- mode: C; mode: fold -*- */
+/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
+ *
+ * This file is part of slrn.
+ *
+ * Slrn is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ * 
+ * Slrn is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with Slrn; see the file COPYING.  If not, write to the Free
+ * Software Foundation, 59 Temple Place - Suite 330, 
+ * Boston, MA  02111-1307, USA.
+ */
+
 
 #include "config.h"
 #include "slrnfeat.h"
@@ -27,6 +47,7 @@
 #include "menu.h"
 #include "interp.h"
 #include "util.h"
+#include "print.h"
 
 /*}}}*/
 
@@ -37,6 +58,7 @@ char *Slrn_Macro_Dir;
 
 /*}}}*/
 
+
 /*{{{ Screen update and message functions */
 
 static void error (char *msg) /*{{{*/
@@ -45,104 +67,143 @@ static void error (char *msg) /*{{{*/
 }
 
 /*}}}*/
-static void message (char *msg) /*{{{*/
-{
-   slrn_message ("%s", msg);
-}
 
-/*}}}*/
 static void update (void) /*{{{*/
 {
-   slrn_update_screen (1);
+   slrn_update_screen ();
+}
+/*}}}*/
+
+static void free_argv_list (char **argv, unsigned int argc)
+{
+   unsigned int i;
+   
+   for (i = 0; i < argc; i++)
+     slrn_free (argv[i]);
+   slrn_free ((char *) argv);
 }
 
-/*}}}*/
-static int interp_select_box (int *np) /*{{{*/
+static char **pop_argv_list (unsigned int *argcp)
 {
-   int ret = 0;
-   int n, i;
-   char **selections;
-   Slrn_Select_Box_Type box;
-   
-   n = *np;
-   if (n <= 0) return ret;
-   
-   n++;				       /* for title */
-   
-   if (NULL == (selections = (char **) SLMALLOC (sizeof (char *) * (n + 1))))
+   int n;
+   char **argv;
+   unsigned int i, argc;
+
+   if (-1 == SLang_pop_integer (&n))
+     return NULL;
+
+   if (n < 0)
+     {
+	slrn_error ("positive integer expected");
+	return NULL;
+     }
+
+   argc = (unsigned int) n;
+      
+   if (NULL == (argv = (char **) slrn_malloc (sizeof (char *) * (argc + 1), 1, 1)))
      {
 	SLang_Error = SL_MALLOC_ERROR;
-	return ret;
+	return NULL;
      }
    
-   selections [n] = NULL;
-   i = n;
+   argv [n] = NULL;
+   i = argc;
    while (i != 0)
      {
 	i--;
-	if (SLpop_string (selections + i))
+	if (-1 == SLpop_string (argv + i))
 	  {
-	     i++;
-	     goto return_error;
+	     free_argv_list (argv, argc);
+	     return NULL;
 	  }
      }
+   *argcp = argc;
+
+   return argv;
+}
    
-   box.title = selections[0];;
-   box.lines = selections + 1;	       /* +1 to skip title */
-   
+static int interp_select_box (void) /*{{{*/
+{
+   unsigned int n;
+   Slrn_Select_Box_Type box;
+   int ret;
+
    if (Slrn_Batch)
      {
-	slrn_error ("select box function now available in batch mode.");
-	ret = -1;
+	slrn_error ("select box function not available in batch mode.");
+	return -1;
      }
-   else ret = slrn_select_box (&box);
 
-   i = 0;
-   
-   /* drop */
-   
-   return_error:
-   while (i < n)
+   if (NULL == (box.lines = pop_argv_list (&n)))
+     return -1;
+
+   if (-1 == SLpop_string (&box.title))
      {
-	SLFREE (selections[i]);
-	i++;
+	free_argv_list (box.lines, n);
+	return -1;
      }
-   SLFREE (selections);
+   
+   ret = slrn_select_box (&box);
+   free_argv_list (box.lines, n);
+   slrn_free (box.title);
+   
    return ret;
 }
 
 /*}}}*/
-static int get_yesno_cancel (char *prompt)
+
+static void select_list_box (void)
 {
-   char *p = prompt;
+   char *title;
+   unsigned int argc;
+   char **argv;
+   int active;
+   int ret;
+
+   if (-1 == SLang_pop_integer (&active))
+     return;
    
-   /* The slrn_get_response function is sprintf.  For that reason,
-    * % cannot be used unless it is doubled up.
-    */
-   while (NULL != (p = slrn_strchr (p, '%')))
+   argv = pop_argv_list (&argc);
+   if (argv == NULL)
+     return;
+   
+   if (-1 == SLpop_string (&title))
      {
-	p++;
-	if (*p != '%') return -1;
-	p++;
+	free_argv_list (argv, argc);
+	return;
      }
    
-   return slrn_get_yesno_cancel (prompt);
+   ret = slrn_select_list_mode (title, argc, argv, active - 1, NULL);
+   if (ret == -1)
+     {
+	SLang_push_string ("");
+	return;
+     }
+   
+   SLang_push_string (argv[ret]);
+
+   slrn_free (title);
+   free_argv_list (argv, argc);
+}
+
+static int get_yesno_cancel (char *prompt)
+{
+   return slrn_get_yesno_cancel ("%s", prompt);
+}
+
+static int get_response (char *choices, char *prompt)
+{
+   return slrn_get_response (choices, "%s", prompt);
 }
 
    
-static void tt_send (void)
+static void tt_send (char *s)
 {
-   char *s;
-   
-   if (SLpop_string (&s))
-     return;
-   
    if (Slrn_Batch == 0)
      {
 	SLtt_write_string (s);
 	SLtt_flush_output ();
      }
-   SLFREE (s);
 }
 
 /*}}}*/
@@ -160,11 +221,10 @@ static void make_home_filename (char *name) /*{{{*/
 
 static int evalfile (char *file)
 {
-   slrn_message_now ("loading %s", file);
-   if (0 == SLang_load_file (file))
-     return -1;
+   if (-1 == slrn_eval_slang_file (file))
+     return 0;
 
-   return 0;
+   return 1;
 }
 
 
@@ -175,91 +235,69 @@ int slrn_eval_slang_file (char *name) /*{{{*/
    if (Slrn_Macro_Dir != NULL)
      {
 	int n = 0;
-	int comma = ',';
-	char *dirp;
 	char dir[SLRN_MAX_PATH_LEN];
 	
 	while (1)
 	  {
-	     dirp = SLang_extract_list_element (Slrn_Macro_Dir, &n, &comma);
-	     
-	     if (0 == strlen (dirp))
+	     if (-1 == SLextract_list_element (Slrn_Macro_Dir, n, ',',
+						   file, sizeof (file)))
 	       break;
 	     
-	     slrn_make_home_dirname (dirp, dir);
+	     slrn_make_home_dirname (file, dir);
 	     if ((-1 != slrn_dircat (dir, name, file))
 		 && (1 == slrn_file_exists (file)))
 	       {
-		  return evalfile (file);
+		  slrn_message_now ("loading %s", file);
+		  if (-1 == SLang_load_file (file))
+		    return -1;
+		  return 0;
 	       }
 	     n++;
 	  }
      }
    
    slrn_make_home_filename (name, file);
-   return evalfile (file);
+   slrn_message_now ("loading %s", file);
+   if (0 == SLang_load_file (file))
+     return -1;
+   return 0;
 }
 
 /*}}}*/
 
 
+
 /*}}}*/
 
 /*{{{ Set/Get Variable Functions */
 
-static void set_string_variable (void) /*{{{*/
+static void set_string_variable (char *s1, char *s2) /*{{{*/
 {
-   char *s1 = NULL, *s2 = NULL;
-   
-   if ((0 == SLpop_string (&s2))
-       && (0 == SLpop_string (&s1)))
-     {
-	if (-1 == slrn_set_string_variable (s1, s2))
-	  slrn_error ("%s is not a valid variable name.", s1);
-     }
-   
-   slrn_free (s1);
-   slrn_free (s2);
-}
-
-/*}}}*/
-
-static void set_integer_variable (void) /*{{{*/
-{
-   char *s1;
-   int val;
-   
-   if (SLang_pop_integer (&val))
-     return;
-   
-   if (SLpop_string (&s1))
-     return;
-
-   if (-1 == slrn_set_integer_variable (s1, val))
+   if (-1 == slrn_set_string_variable (s1, s2))
      slrn_error ("%s is not a valid variable name.", s1);
-   
-   SLFREE (s1);
 }
 
 /*}}}*/
 
-static void get_variable_value (void) /*{{{*/
+static void set_integer_variable (char *s1, int *val) /*{{{*/
+{
+   if (-1 == slrn_set_integer_variable (s1, *val))
+     slrn_error ("%s is not a valid variable name.", s1);
+}
+
+/*}}}*/
+
+static void get_variable_value (char *name) /*{{{*/
 {
-   char *name;
    char **s;
    int *ip;
    int type;
    
-   if (SLpop_string (&name))
-     return;
-   
    if (-1 == slrn_get_variable_value (name, &type, &s, &ip))
      {
 	slrn_error ("%s is not a valid variable name.", name);
-	SLFREE (name);
 	return;
      }
-   SLFREE (name);
    
    if (type == STRING_TYPE)
      {
@@ -331,34 +369,19 @@ static void undefinekey (char *key, char *map) /*{{{*/
 
 /*}}}*/
 
-static void generic_read_mini (int no_echo) /*{{{*/
+static void generic_read_mini (int no_echo, char *prompt, char *dfl, char *init) /*{{{*/
 {
    char str[256];
    int ret;
-   char *prompt = NULL, *dfl = NULL, *init = NULL;
-   
-   if (SLpop_string (&init)
-       || SLpop_string (&dfl)
-       || SLpop_string (&prompt))
-     {
-	slrn_free (init);
-	slrn_free (dfl);
-	slrn_free (prompt);
-	return;
-     }
-   
+
    strncpy (str, init, sizeof (str));
    str[sizeof(str) - 1] = 0;
    
    if (no_echo) 
-     ret = slrn_read_input_no_echo (prompt, dfl, str, 0);
+     ret = slrn_read_input_no_echo (prompt, dfl, str, 0, 0);
    else 
-     ret = slrn_read_input (prompt, dfl, str, 0);
+     ret = slrn_read_input (prompt, dfl, str, 0, 0);
    
-   SLFREE (init);
-   SLFREE (dfl);
-   SLFREE (prompt);
-
    if (-1 == ret)
      {
 	error ("Quit!");
@@ -368,15 +391,15 @@ static void generic_read_mini (int no_echo) /*{{{*/
 }
 
 /*}}}*/
-static void read_mini (void) /*{{{*/
+static void read_mini (char *prompt, char *dfl, char *init) /*{{{*/
 {
-   generic_read_mini (0);
+   generic_read_mini (0, prompt, dfl, init);
 }
 
 /*}}}*/
-static void read_mini_no_echo (void) /*{{{*/
+static void read_mini_no_echo (char *prompt, char *dfl, char *init) /*{{{*/
 {
-   generic_read_mini (1);
+   generic_read_mini (1, prompt, dfl, init);
 }
 
 /*}}}*/
@@ -388,6 +411,23 @@ static void set_prefix_arg (int *arg) /*{{{*/
 
 /*}}}*/
 
+static int get_prefix_arg (void) /*{{{*/
+{
+#define NO_PREFIX_ARGUMENT -1
+
+   return Slrn_Prefix_Arg_Ptr ? *Slrn_Prefix_Arg_Ptr : NO_PREFIX_ARGUMENT;
+} 
+
+/*}}}*/
+
+static void reset_prefix_arg (void) /*{{{*/
+{
+   Slrn_Prefix_Arg_Ptr = NULL;
+}
+  
+/*}}}*/
+
+
 static int check_tty (void)
 {
    if (Slrn_TT_Initialized & SLRN_TTY_INIT)
@@ -424,10 +464,17 @@ static void ungetkey (int *c)
 static void set_input_string (void)
 {
    char *s;
-   if (SLpop_string (&s)) s = NULL;
+   if (-1 == SLpop_string (&s)) s = NULL;
    slrn_set_input_string (s);
 }
 
+static void set_input_chars (void)
+{
+   char *s;
+   if (-1 == SLpop_string (&s)) s = NULL;
+   slrn_set_input_chars (s);
+}
+
 /*}}}*/
 
 /*{{{ Article Mode Functions */
@@ -716,6 +763,19 @@ static int get_header_score (void)
 #endif
 }
 
+static int get_grouplens_score (void)
+{
+   if ((-1 == check_article_mode ())
+       || (Slrn_Current_Header == NULL))
+     return 0;
+
+#if SLRN_HAS_GROUPLENS
+   return Slrn_Current_Header->gl_pred;
+#else
+   return 0;
+#endif
+}
+
 /*{{{ Header Searching */
 
 static int re_header_search (char *pat, unsigned int offset, int dir) /*{{{*/
@@ -858,82 +918,146 @@ static void set_group_flags (int *flagsp)
 
 /*}}}*/
 
-static SLang_Name_Type Slrn_Intrinsics [] = /*{{{*/
-{   
-   MAKE_INTRINSIC(".set_header_score", set_header_score, VOID_TYPE, 1),
-   MAKE_INTRINSIC(".get_header_score", get_header_score, INT_TYPE, 0),
-   MAKE_INTRINSIC(".article_as_string", article_as_string, STRING_TYPE, 0),
-   MAKE_INTRINSIC(".call", slrn_call_command, VOID_TYPE, 1),
-   MAKE_INTRINSIC(".collapse_thread", collapse_thread, VOID_TYPE, 0),
-   MAKE_INTRINSIC(".collapse_threads", collapse_threads, VOID_TYPE, 0),
-   MAKE_INTRINSIC(".current_newsgroup", current_group_name, STRING_TYPE, 0),
-   MAKE_INTRINSIC(".definekey", definekey, VOID_TYPE, 3),
-   MAKE_INTRINSIC(".error", error, VOID_TYPE, 1),
-   MAKE_INTRINSIC(".extract_article_header", extract_article_header, STRING_TYPE, 1),
-   MAKE_INTRINSIC(".get_group_flags", get_group_flags, INT_TYPE, 0),
-   MAKE_INTRINSIC(".get_header_flags", get_header_flags, INT_TYPE, 0),
-   MAKE_INTRINSIC(".get_header_tag_number", get_header_tag_number, INT_TYPE, 0),
-   MAKE_INTRINSIC(".get_select_box_response", interp_select_box, INT_TYPE, 1),
-   MAKE_INTRINSIC(".get_variable_value", get_variable_value, VOID_TYPE, 0),
-   MAKE_INTRINSIC(".get_yes_no_cancel", get_yesno_cancel, INT_TYPE, 1),
-   MAKE_INTRINSIC(".getkey", getkey, INT_TYPE, 0),
-   MAKE_INTRINSIC(".goto_num_tagged_header", slrn_goto_num_tagged_header, INT_TYPE, 1),
-   MAKE_INTRINSIC(".group_down_n", group_down_n, INT_TYPE, 1),
-   MAKE_INTRINSIC(".group_search", slrn_group_search, INT_TYPE, 1),
-   MAKE_INTRINSIC(".group_unread", get_group_unread_count, INT_TYPE, 0),
-   MAKE_INTRINSIC(".group_up_n", group_up_n, INT_TYPE, 1),
-   MAKE_INTRINSIC(".header_down", header_down, INT_TYPE, 1),
-   MAKE_INTRINSIC(".header_next_unread", header_next_unread, INT_TYPE, 0),
-   MAKE_INTRINSIC(".header_up", header_up, INT_TYPE, 1),
-   MAKE_INTRINSIC(".input_pending", input_pending, INT_TYPE, 1),
-   MAKE_INTRINSIC(".is_article_visible", slrn_is_article_visible, INT_TYPE, 0),
-   MAKE_INTRINSIC(".is_group_mode", is_group_mode, INT_TYPE, 0),
-   MAKE_INTRINSIC(".is_thread_collapsed", is_thread_collapsed, INT_TYPE, 0),
-   MAKE_INTRINSIC(".make_home_filename", make_home_filename, VOID_TYPE, 1),
-   MAKE_INTRINSIC(".message", message, VOID_TYPE, 1),
-   MAKE_INTRINSIC(".next_tagged_header", slrn_next_tagged_header, INT_TYPE, 0),
-   MAKE_INTRINSIC(".pipe_article", pipe_article_cmd, VOID_TYPE, 1),
-   MAKE_INTRINSIC(".prev_tagged_header", slrn_prev_tagged_header, INT_TYPE, 0),
-   MAKE_INTRINSIC(".quit", quit, VOID_TYPE, 1),
-   MAKE_INTRINSIC(".re_bsearch_author", re_author_search_backward, INT_TYPE, 1),
-   MAKE_INTRINSIC(".re_bsearch_subject", re_subject_search_backward, INT_TYPE, 1),
-   MAKE_INTRINSIC(".re_fsearch_author", re_author_search_forward, INT_TYPE, 1),
-   MAKE_INTRINSIC(".re_fsearch_subject", re_subject_search_forward, INT_TYPE, 1),
-   MAKE_INTRINSIC(".re_search_article", re_search_article, INT_TYPE, 1),
-   MAKE_INTRINSIC(".read_mini", read_mini, VOID_TYPE, 0),
-   MAKE_INTRINSIC(".read_mini_no_echo", read_mini_no_echo, VOID_TYPE, 0),
-   MAKE_INTRINSIC(".save_current_article", save_current_article, INT_TYPE, 1),
-   MAKE_INTRINSIC(".search_article", search_article, INT_TYPE, 1),
-   MAKE_INTRINSIC(".select_group", select_group, INT_TYPE, 0),
-   MAKE_INTRINSIC(".server_name", get_server_name, STRING_TYPE, 0),
-   MAKE_INTRINSIC(".set_article_window_size", set_article_window_size, VOID_TYPE, 1),
-   MAKE_INTRINSIC(".set_group_flags", set_group_flags, VOID_TYPE, 1),
-   MAKE_INTRINSIC(".set_header_flags", set_header_flags, VOID_TYPE, 1),
-   MAKE_INTRINSIC(".set_input_string", set_input_string, VOID_TYPE, 0),
-   MAKE_INTRINSIC(".set_integer_variable", set_integer_variable, VOID_TYPE, 0),
-   MAKE_INTRINSIC(".set_prefix_argument", set_prefix_arg, VOID_TYPE, 1),
-   MAKE_INTRINSIC(".set_string_variable", set_string_variable, VOID_TYPE, 0),
-   MAKE_INTRINSIC(".thread_size", thread_size, INT_TYPE, 0),
-   MAKE_INTRINSIC(".tt_send", tt_send, VOID_TYPE, 0),
-   MAKE_INTRINSIC(".uncollapse_thread", uncollapse_thread, VOID_TYPE, 0),
-   MAKE_INTRINSIC(".uncollapse_threads", uncollapse_threads, VOID_TYPE, 0),
-   MAKE_INTRINSIC(".undefinekey", undefinekey, VOID_TYPE, 2),
-   MAKE_INTRINSIC(".ungetkey", ungetkey, VOID_TYPE, 1),
-   MAKE_INTRINSIC(".update", update, VOID_TYPE, 0),
-   MAKE_VARIABLE(".GROUPS_DIRTY", &Slrn_Groups_Dirty, INT_TYPE, 1),
-   MAKE_VARIABLE(".GROUP_NEW_GROUP_FLAG", &Interp_Group_New_Group_Flag, INT_TYPE, 1),
-   MAKE_VARIABLE(".GROUP_UNSUBSCRIBED", &Interp_Group_Unsubscribed, INT_TYPE, 1),
-   MAKE_VARIABLE(".HEADER_HIGH_SCORE", &Interp_Header_High_Score, INT_TYPE, 1),
-   MAKE_VARIABLE(".HEADER_LOW_SCORE", &Interp_Header_Low_Score, INT_TYPE, 1),
-   MAKE_VARIABLE(".HEADER_READ", &Interp_Header_Read, INT_TYPE, 1),
-   MAKE_VARIABLE(".HEADER_TAGGED", &Interp_Header_Tagged, INT_TYPE, 1),
-   MAKE_VARIABLE(".SCREEN_HEIGHT", &SLtt_Screen_Rows, INT_TYPE, 1),
-   MAKE_VARIABLE(".SCREEN_WIDTH", &SLtt_Screen_Cols, INT_TYPE, 1),
+static void set_color (char *obj, char *fg, char *bg)
+{
+   slrn_set_object_color (obj, fg, bg);
+}
+
+static FILE *Log_File_Ptr;
+static void close_log_file (void)
+{	
+   if (Log_File_Ptr != NULL)
+     {
+	(void) fclose (Log_File_Ptr);
+	Log_File_Ptr = NULL;
+     }
+}
+
+static void open_log_file (char *file)
+{
+   close_log_file ();
+
+   Log_File_Ptr = fopen (file, "w");
+   if (Log_File_Ptr == NULL)
+     slrn_error ("Unable to open log file %s", file);
+}
+
+static void log_message (char *buf)
+{
+   FILE *fp;
+   
+   fp = Log_File_Ptr;
+   if (fp == NULL) fp = stderr;
+   
+   fputs (buf, fp);
+   fflush (fp);
+}
+
+static void set_header_display_format (int *num, char *fmt)
+{
+   (void) slrn_set_header_format (*num, fmt);
+}
+
+static void print_file (char *file)
+{
+   (void) slrn_print_file (file);
+}
+
+static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
+{
+   MAKE_INTRINSIC_S("print_file", print_file, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_S("open_log_file", open_log_file, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("close_log_file", close_log_file, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_S("log_message", log_message, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_IS("set_header_display_format", set_header_display_format, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("get_prefix_arg", get_prefix_arg, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("reset_prefix_arg", reset_prefix_arg, SLANG_VOID_TYPE),
+
+   MAKE_INTRINSIC_0("article_as_string", article_as_string, SLANG_STRING_TYPE),
+   MAKE_INTRINSIC_S("call", slrn_call_command, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("collapse_thread", collapse_thread, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("collapse_threads", collapse_threads, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("current_newsgroup", current_group_name, SLANG_STRING_TYPE),
+   MAKE_INTRINSIC_SSS("definekey", definekey, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_S("extract_article_header", extract_article_header, SLANG_STRING_TYPE),
+   MAKE_INTRINSIC_0("get_group_flags", get_group_flags, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("get_grouplens_score", get_grouplens_score, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("get_header_flags", get_header_flags, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("get_header_score", get_header_score, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("get_header_tag_number", get_header_tag_number, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("get_next_art_pgdn_action", slrn_get_next_pagedn_action, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_SS("get_response", get_response, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("get_select_box_response", interp_select_box, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_S("get_variable_value", get_variable_value, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_S("get_yes_no_cancel", get_yesno_cancel, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("getkey", getkey, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_I("goto_num_tagged_header", slrn_goto_num_tagged_header, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_I("group_down_n", group_down_n, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_S("group_search", slrn_group_search, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("group_unread", get_group_unread_count, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_I("group_up_n", group_up_n, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_I("header_down", header_down, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("header_next_unread", header_next_unread, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_I("header_up", header_up, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_I("input_pending", input_pending, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("is_article_visible", slrn_is_article_visible, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("is_article_window_zoomed", slrn_is_article_win_zoomed, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("is_group_mode", is_group_mode, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("is_thread_collapsed", is_thread_collapsed, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_S("make_home_filename", make_home_filename, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("next_tagged_header", slrn_next_tagged_header, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_S("pipe_article", pipe_article_cmd, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("prev_tagged_header", slrn_prev_tagged_header, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_I("quit", quit, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_S("re_bsearch_author", re_author_search_backward, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_S("re_bsearch_subject", re_subject_search_backward, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_S("re_fsearch_author", re_author_search_forward, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_S("re_fsearch_subject", re_subject_search_forward, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_S("re_search_article", re_search_article, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_SSS("read_mini", read_mini, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_SSS("read_mini_no_echo", read_mini_no_echo, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_S("save_current_article", save_current_article, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_S("search_article", search_article, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("select_group", select_group, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("select_list_box", select_list_box, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("server_name", get_server_name, SLANG_STRING_TYPE),
+   MAKE_INTRINSIC_I("set_article_window_size", set_article_window_size, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_SSS("set_color", set_color, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_I("set_group_flags", set_group_flags, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_I("set_header_flags", set_header_flags, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_I("set_header_score", set_header_score, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("set_input_chars", set_input_chars, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("set_input_string", set_input_string, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_SI("set_integer_variable", set_integer_variable, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_I("set_prefix_argument", set_prefix_arg, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_SS("set_string_variable", set_string_variable, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("thread_size", thread_size, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_S("tt_send", tt_send, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("uncollapse_thread", uncollapse_thread, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("uncollapse_threads", uncollapse_threads, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_SS("undefinekey", undefinekey, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_I("ungetkey", ungetkey, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("update", update, SLANG_VOID_TYPE),
    SLANG_END_TABLE
 };
 
 /*}}}*/
 
+static SLang_Intrin_Var_Type Intrin_Vars [] =
+{
+   MAKE_VARIABLE("GROUPS_DIRTY", &Slrn_Groups_Dirty, SLANG_INT_TYPE, 1),
+   MAKE_VARIABLE("GROUP_NEW_GROUP_FLAG", &Interp_Group_New_Group_Flag, SLANG_INT_TYPE, 1),
+   MAKE_VARIABLE("GROUP_UNSUBSCRIBED", &Interp_Group_Unsubscribed, SLANG_INT_TYPE, 1),
+   MAKE_VARIABLE("HEADER_HIGH_SCORE", &Interp_Header_High_Score, SLANG_INT_TYPE, 1),
+   MAKE_VARIABLE("HEADER_LOW_SCORE", &Interp_Header_Low_Score, SLANG_INT_TYPE, 1),
+   MAKE_VARIABLE("HEADER_READ", &Interp_Header_Read, SLANG_INT_TYPE, 1),
+   MAKE_VARIABLE("HEADER_TAGGED", &Interp_Header_Tagged, SLANG_INT_TYPE, 1),
+   MAKE_VARIABLE("SCREEN_HEIGHT", &SLtt_Screen_Rows, SLANG_INT_TYPE, 1),
+   MAKE_VARIABLE("SCREEN_WIDTH", &SLtt_Screen_Cols, SLANG_INT_TYPE, 1),
+   SLANG_END_TABLE
+};
+
+
 static int interp_system (char *s) /*{{{*/
 {
    return slrn_posix_system (s, 1);
@@ -944,26 +1068,38 @@ static int interp_system (char *s) /*{{{*/
 int slrn_init_slang (void) /*{{{*/
 {
    Slrn_Use_Slang = 0;
-   if (!init_SLang()		       /* basic interpreter functions */
-       || !init_SLmath() 	       /* sin, cos, etc... */
+   if ((-1 == SLang_init_slang ())
+       || (-1 == SLang_init_slmath ())
 #ifdef __unix__
-       || !init_SLunix()	       /* unix system calls */
+       || (-1 == SLang_init_slunix ())
 #endif
-       || !init_SLfiles()	       /* file i/o */
+       || (-1 == SLang_init_slfile ())
        
        /* Now add intrinsics for this application */
-       || !SLang_add_table(Slrn_Intrinsics, "slrn"))
+       || (-1 == SLadd_intrin_fun_table(Slrn_Intrinsics, NULL))
+       || (-1 == SLadd_intrin_var_table(Intrin_Vars, NULL)))
      return -1;
    
-   SLadd_name ("system", (long) interp_system, SLANG_INTRINSIC, SLANG_MAKE_ARGS(INT_TYPE, 1));
-   
-   SLang_Error_Routine = error;
+   SLadd_intrinsic_function ("system", (FVOID_STAR) interp_system, SLANG_INT_TYPE, 1, SLANG_STRING_TYPE);
+   SLadd_intrinsic_function ("evalfile", (FVOID_STAR) evalfile, SLANG_INT_TYPE, 1, SLANG_STRING_TYPE);
+
+   SLang_Error_Hook = error;
 
    Slrn_Use_Slang = 1;
    SLang_User_Clear_Error = slrn_clear_message;
+   SLang_Exit_Error_Hook = slrn_va_exit_error;
+   SLang_Dump_Routine = log_message;
+   SLang_VMessage_Hook = slrn_va_message;
    return 0;
 }
 
 /*}}}*/
 
+int slrn_reset_slang (void)
+{
+   close_log_file ();
+   return 0;
+}
+
 #endif
+
