@@ -1,7 +1,33 @@
+/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
+ *
+ * This file is part of slrn.
+ *
+ * Slrn is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ * 
+ * Slrn is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with Slrn; see the file COPYING.  If not, write to the Free
+ * Software Foundation, 59 Temple Place - Suite 330, 
+ * Boston, MA  02111-1307, USA.
+ */
+
 /* MIME handling routines.
  *
  * Author: Michael Elkins <elkins@aero.org>
  * Modified by John E. Davis <davis@space.mit.edu>
+ * 
+ * Change Log:
+ * Aug 20, 1997 patch from "Byrial Jensen" <byrial@post3.tele.dk>
+ *   added.  Apparantly RFC2047 requires the whitespace separating
+ *   multiple encoded words in headers to be ignored.
+ *   Status: unchecked
  */
 
 #include "config.h"
@@ -21,7 +47,7 @@
 
 #include <ctype.h>
 
-#if defined (__os2__)
+#if defined(__os2__) || defined(__NT__)
 # include <process.h>
 #endif
 
@@ -330,9 +356,15 @@ int slrn_rfc1522_decode_string (char *s)
 {
    char *s1, *s2, ch;
    char *charset, method, *txt;
-   unsigned int count = 0;
+   char *after_last_encoded_word;
+   char *after_whitespace;
+   unsigned int count;
    unsigned int len;
-   
+
+   count = 0;
+   after_whitespace = NULL;
+   after_last_encoded_word = NULL;
+
    while (1)
      {
 	while ((NULL != (s = slrn_strchr (s, '=')))
@@ -386,6 +418,9 @@ int slrn_rfc1522_decode_string (char *s)
 	     continue;
 	  }
 	
+	if (s1 == after_whitespace)
+	  s1 = after_last_encoded_word;
+
 	/* Note: these functions return a pointer to the END of the decoded
 	 * text.
 	 */
@@ -400,6 +435,10 @@ int slrn_rfc1522_decode_string (char *s)
 	*s1 = 0;
 	
 	count++;
+	
+	after_last_encoded_word = s;
+	s = slrn_skip_whitespace (s);
+	after_whitespace = s;
      }
    return count;
 }
@@ -575,10 +614,10 @@ int slrn_mime_call_metamail (void)
        || (slrn_get_yesno (1, "Process this MIME article with metamail") <= 0))
      return 0;
 
-# ifdef __os2__
+# if defined(__os2__) || defined(__NT__)
    if (NULL == (tmp = getenv ("TMP")))
      tmp = ".";
-#else
+# else
    tmp = "/tmp";
 # endif
    
@@ -786,7 +825,11 @@ static int rfc1522_encode (unsigned char *d, unsigned int len,
 	if (ch == 0)
 	  break;
 	
-	if (d == dmax) return -1;
+	if (d == dmax) 
+	  {
+	     *d = 0;
+	     return -1;
+	  }
 	
 	/* walk back until we find a word boundary or begin-of-line */
 	do
@@ -818,7 +861,8 @@ static int rfc1522_encode (unsigned char *d, unsigned int len,
 	 * That limit is ignored here.  Perhaps someday I will recode this to 
 	 * take the limit into account.
 	 */
-	while ((d < dmax) && ((ch = *p) != 0)
+	while ((d < dmax) 
+	       && ((ch = *p) != 0)
 	       && ((ignore_specials != 0) || (0 == IS_RFC850_SPECIAL(ch)))
 	       && ((hibit != 0) 
 		   || ((ch != ' ') && (ch != '\t'))))
@@ -826,21 +870,38 @@ static int rfc1522_encode (unsigned char *d, unsigned int len,
 	     if (ch == ' ') *d++ = '_';
 	     else if ((ch & 0x80) 
 		      || ((ch < 32) && (ch != '\n'))
-		      || IS_RFC850_SPECIAL(ch))
+		      || IS_RFC850_SPECIAL(ch)
+		      || (ch == '=')  /* ? cannot occur as part of encoded word */
+		      || (ch == '?')  /* ? cannot occur as part of encoded word */
+		      || (ch == '_'))  /* encode this since SPACE gets mapped to it */
 	       {
 		  /* We need 3 characters to encode this. */
 		  if (d + 3 >= dmax)
-		    return -1;
-		  sprintf ((char *)d, "=%2X", (int) ch);
+		    {
+		       *d = 0;
+		       return -1;
+		    }
+		  sprintf ((char *)d, "=%02X", (int) ch);
 		  d += 3;
-		  hibit--;
+		  if (ch & 0x80) hibit++;
 	       }
 	     else *d++ = ch;
 	     p++;
 	  }
 	
+	/* Remove encoded spaces at the end (' '--> '_') */
+	while ((p > s) && (*(p-1) == ' '))
+	  {
+	     d--; 
+	     p--;
+	  }
+
 	/* Now turn off encoding.  We need two characters */
-	if (d + 1 >= dmax) return -1;
+	if (d + 1 >= dmax) 
+	  {
+	     *d = 0;
+	     return -1;
+	  }
 	
 	d[0] = '?';
 	d[1] = '=';
