@@ -1,7 +1,24 @@
 /* -*- mode: C; mode: fold; -*- */
-/*  Copyright (c) 1995, 1996 John E. Davis (davis@space.mit.edu)
- *  All rights reserved.
+/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
+ *
+ * This file is part of slrn.
+ *
+ * Slrn is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ * 
+ * Slrn is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with Slrn; see the file COPYING.  If not, write to the Free
+ * Software Foundation, 59 Temple Place - Suite 330, 
+ * Boston, MA  02111-1307, USA.
  */
+
 #include "config.h"
 #include "slrnfeat.h"
 
@@ -13,6 +30,7 @@
 #include <signal.h>
 #include <ctype.h>
 #include <errno.h>
+#include <time.h>
 
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
@@ -22,21 +40,34 @@
 #include <unistd.h>
 #endif
 
-#ifndef VMS
-#  include <pwd.h>
-#  include <sys/types.h>
-#  include <sys/stat.h>
-#else
+#if !defined(VMS) && !defined(__WIN32__) && !defined(__NT__)
+# define HAS_PASSWORD_CODE	1
+# include <pwd.h>
+#endif
+
+#ifdef VMS
 # include "vms.h"
+#else
+# include <sys/types.h>
+# include <sys/stat.h>
 #endif
 
 #if defined(VMS) && defined(MULTINET)
 # include "multinet_root:[multinet.include]netdb.h"
 #else
-# include <netdb.h>
-# ifndef h_errno
+# if defined(__NT__)
+#  include <winsock.h>
+# else
+#  if defined(__WIN32__)
+#   define Win32_Winsock
+#   include <windows.h>
+#  else
+#   include <netdb.h>
+#   ifndef h_errno
 extern int h_errno;
-# endif
+#   endif
+#  endif 
+# endif 
 #endif 
 
 #ifdef HAVE_SYS_WAIT_H
@@ -102,6 +133,8 @@ static int Error_Present;
 static SLang_RLine_Info_Type *Slrn_Keymap_RLI;
 static char *Input_String;
 static char *Input_String_Ptr;
+static char *Input_Chars_Ptr;
+static char *Input_Chars;
 static int Beep_Pending;
 /*}}}*/
 
@@ -123,19 +156,6 @@ void slrn_smg_refresh (void)
      }
 }
 
-void slrn_update_screen (int rf)
-{
-   if (Slrn_Batch) return;
-
-   if ((Slrn_Current_Mode != NULL)
-       && (Slrn_Current_Mode->redraw_fun != NULL))
-     {
-	slrn_push_suspension (0);
-	(*Slrn_Current_Mode->redraw_fun) ();
-	if (rf) slrn_smg_refresh ();
-	slrn_pop_suspension ();
-     }
-}
 
 void slrn_set_color (int color) /*{{{*/
 {
@@ -154,11 +174,7 @@ void slrn_redraw (void) /*{{{*/
    Slrn_Full_Screen_Update = 1;
    
    redraw_message ();
-
-   if ((Slrn_Current_Mode != NULL)
-       && (Slrn_Current_Mode->redraw_fun != NULL))
-     (*Slrn_Current_Mode->redraw_fun) ();
-   
+   slrn_update_screen ();
    redraw_mini_buffer ();
 
    slrn_smg_refresh ();
@@ -215,16 +231,40 @@ static char Message_Buffer[1024];
 
 static void redraw_message (void)
 {
-   if (Slrn_Batch) return;
+   int color;
+   char *m, *mmax;
 
+   if (Slrn_Batch) return;
+   
    if (Slrn_Message_Present == 0)
      return;
    
    slrn_push_suspension (0);
 
    SLsmg_gotorc (SLtt_Screen_Rows - 1, 0);
-   slrn_set_color (Message_Buffer[0]);
-   SLsmg_write_string (Message_Buffer + 1);
+   
+   color = Message_Buffer [0];
+   m = Message_Buffer + 1;
+   
+   while (1)
+     {
+	mmax = slrn_strchr (m, 1);
+	if (mmax == NULL)
+	  mmax = m + strlen(m);
+	
+	slrn_set_color (color);
+	SLsmg_write_nchars (m, (unsigned int) (mmax - m));
+	if (*mmax == 0)
+	  break;
+	mmax++;
+	if (*mmax == 0)
+	  break;
+	
+	slrn_set_color (RESPONSE_CHAR_COLOR);
+	SLsmg_write_nchars (mmax, 1);
+	m = mmax + 1;
+     }
+   
    SLsmg_erase_eol ();
    slrn_set_color (0);
    
@@ -287,6 +327,13 @@ void slrn_clear_message (void) /*{{{*/
 }
 
 /*}}}*/
+
+void slrn_va_message (char *fmt, va_list ap)
+{
+   if (Error_Present == 0)
+     vmessage (stderr, fmt, ap);
+}
+
 int slrn_message (char *fmt, ...) /*{{{*/
 {
    va_list ap;
@@ -326,15 +373,19 @@ void slrn_error (char *fmt, ...) /*{{{*/
 
 /*}}}*/
 
-void slrn_error_now (char *fmt, ...) /*{{{*/
+void slrn_error_now (unsigned secs, char *fmt, ...) /*{{{*/
 {
    va_list ap;
 
-   va_start(ap, fmt);
-   verror (fmt, ap);
-   va_end (ap);
+   if (fmt != NULL)
+     {
+	va_start(ap, fmt);
+	verror (fmt, ap);
+	va_end (ap);
+     }
    slrn_smg_refresh ();
    Slrn_Message_Present = 0;
+   if (secs) sleep (secs);
 }
 
 /*}}}*/
@@ -367,7 +418,7 @@ static void vms_fix_name(char *name)
        name[idx] = '-';
 }
 
-static char Copystr[256];
+static char Copystr[SLRN_MAX_PATH_LEN];
 static int vms_copyname1 (char *name)
 {
    strcpy(Copystr, name);
@@ -387,17 +438,17 @@ void slrn_make_home_filename (char *name, char *file) /*{{{*/
 {
    char *home;
 #ifndef VMS
-   if (
-#ifdef __os2__
-       (*name == '/') || (*name == '\\') || (*(name + 1) == ':') 
-       || ((*name == '.') && (*(name + 1) == '/')) 
-       || ((*name == '.') && (*(name + 1) == '\\'))
-#else
-       (*name == '/') || ((*name == '.') && (*(name + 1) == '/'))
+   if (slrn_is_absolute_path (name)
+       || ((name[0] == '.') && (name[1] == SLRN_PATH_SLASH_CHAR))
+#if defined(IBMPC_SYSTEM)
+       || ((name[0] == '.') && (name[1] == '/'))
 #endif
        )
      {
 	strcpy (file, name);
+#if defined(IBMPC_SYSTEM)
+	slrn_os2_convert_path (file);
+#endif
 	return;
      }
    
@@ -408,8 +459,8 @@ void slrn_make_home_filename (char *name, char *file) /*{{{*/
    slrn_dircat (home, name, file);
 #else
    char *cp, *cp1;
-   static char fname[256];
-   char fn[256], fn1[256];
+   static char fname[SLRN_MAX_PATH_LEN];
+   char fn[SLRN_MAX_PATH_LEN], fn1[SLRN_MAX_PATH_LEN];
    int rc, idx;
    
    strcpy (fn1, name);
@@ -487,8 +538,8 @@ int slrn_make_home_dirname (char *name, char *dir) /*{{{*/
    slrn_make_home_filename (name, dir);
 #else
    char *home, *cp;
-   char fn[256];
-   static char fname[256];
+   char fn[SLRN_MAX_PATH_LEN];
+   static char fname[SLRN_MAX_PATH_LEN];
    int rc, idx, len;
    
    if (NULL != slrn_strchr (name, ':'))
@@ -556,6 +607,26 @@ int slrn_make_home_dirname (char *name, char *dir) /*{{{*/
 
 /*}}}*/
 
+static unsigned int make_random (void)
+{
+   static unsigned long s;
+   static int init;
+   
+   if (init == 0)
+     {
+	s = (unsigned long) time (NULL) + (unsigned long) getpid ();
+	init = 1;
+     }
+
+   s = s * 69069UL + 1013904243UL;
+
+   /* The time has been added to make this number somewhat unpredictable
+    * based on the previous number.  The whole point of this is to foil
+    * any attempt of a hacker to determine the name of the _next_ temp
+    * file that slrn will create.
+    */
+   return (unsigned int) (s + (unsigned long) time (NULL));
+}
    
 /* Note: This function should not create a file that is deleted when it
  * is closed.
@@ -568,19 +639,21 @@ FILE *slrn_open_tmpfile_in_dir (char *dir, char *file, char *mode)
    unsigned int i;
    char buf[80];
    
+#ifndef VMS
    if (2 != slrn_file_exists (dir))
      return NULL;
-   
-#ifdef __os2__
-   sprintf (buf, "SLRN%04u", (unsigned int) getpid ());
+#endif
+
+#if defined(IBMPC_SYSTEM)
+   sprintf (buf, "SLRN%04u", make_random ());
 #else
-   sprintf (buf, "SLRN%uX", (unsigned int) getpid ());
+   sprintf (buf, "SLRN%X", make_random ());
 #endif
    
    if (-1 == slrn_dircat (dir, buf, file))
      return NULL;
    
-#ifdef __os2__
+#if defined(IBMPC_SYSTEM)
 # define MAX_TMP_FILE_NUMBER 999
 #else
 # define MAX_TMP_FILE_NUMBER 1024
@@ -590,6 +663,10 @@ FILE *slrn_open_tmpfile_in_dir (char *dir, char *file, char *mode)
    for (i = 0; i < MAX_TMP_FILE_NUMBER; i++)
      {
 	sprintf (file + len, ".%u", i);
+	
+	/* I wish this were an atomic operation but I do not know of a
+	 * portable way to achieve that using standard C.
+	 */
 	if ((0 == slrn_file_exists (file))	    
 	    && (NULL != (fp = fopen (file, mode))))
 	  {
@@ -614,7 +691,7 @@ FILE *slrn_open_tmpfile (char *file, char *mode) /*{{{*/
 #ifdef VMS
 	dir = "SYS$LOGIN:";
 #else
-# ifdef __os2__
+# if defined(IBMPC_SYSTEM)
 	dir = ".";
 # else
 	dir = "/tmp";
@@ -640,9 +717,9 @@ FILE *slrn_open_home_file (char *name, char *mode, char *file, int create_flag)
      }
 #else
    (void) create_flag;
+#endif	
 
    return fopen (file, mode);
-#endif	
 }
 
 /*}}}*/
@@ -650,7 +727,7 @@ FILE *slrn_open_home_file (char *name, char *mode, char *file, int create_flag)
 int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char *subject) /*{{{*/
 {
    char buf [256 + 2 * SLRN_MAX_PATH_LEN];
-#ifdef __os2__
+#if defined(IBMPC_SYSTEM)
    char outfile [SLRN_MAX_PATH_LEN];
 #endif
    
@@ -662,7 +739,7 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
 	  {
 	     char rsp;
 	     
-	     rsp = slrn_get_response ("yYnNeE", "Mail the message? y/n/e(dit)");
+	     rsp = slrn_get_response ("yYnNeE", "Mail the message? \001Yes, \001No, \001Edit");
 	     rsp |= 0x20;
 	     if (rsp == 'n') return -1;
 	     if (rsp == 'y') break;
@@ -670,7 +747,10 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
 	  }
      }
    slrn_message_now ("Sending ...");
-   
+
+   if (-1 == slrn_save_file_to_mail_file (file, Slrn_Save_Replies_File, NULL))
+     return -1;
+
    slrn_chmap_fix_file (file);
 
 #ifdef VMS
@@ -692,7 +772,7 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
 	if (fp == NULL) return (-1);
 	
 	slrn_mime_scan_file (fp);
-# ifdef __os2__
+# if defined(IBMPC_SYSTEM)
 	pp = slrn_open_tmpfile (outfile, "w");
 # else
 	pp = slrn_popen (Slrn_SendMail_Command, "w");
@@ -727,7 +807,7 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
 	     putc('\n', pp);
 	  }
 	slrn_fclose (fp);
-#ifdef __os2__
+#if defined(IBMPC_SYSTEM)
 	slrn_fclose (pp);
 	sprintf (buf, "%s %s", Slrn_SendMail_Command, outfile);
 	slrn_posix_system (buf, 0);
@@ -737,7 +817,7 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
      }
    else
      {
-#ifdef __os2__
+#if defined(IBMPC_SYSTEM)
 	sprintf (buf, "%s %s", Slrn_SendMail_Command, file);
 #else
 	sprintf (buf, "%s < %s", Slrn_SendMail_Command, file);
@@ -751,28 +831,38 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
 
 /*}}}*/
 
+#if SLRN_HAS_PIPING   
+int _slrn_pclose (FILE *fp)
+{
+   int ret;
+
+   ret = pclose (fp);
+   if (ret == 0) return ret;
+
+#if defined(WIFEXITED) && defined(WEXITSTATUS)
+   if ((ret != -1) && WIFEXITED(ret))
+     {
+	ret = WEXITSTATUS(ret);
+     }
+#endif
+   return ret;
+}
+#endif				       /* SLRN_HAS_PIPING */
+
 int slrn_pclose (FILE *fp) /*{{{*/
 {
 #if SLRN_HAS_PIPING
    int ret;
    if (fp == NULL) return -1;
-   ret = pclose (fp);
+   
+   ret = _slrn_pclose (fp);
    if (ret)
      {
-	char buf[256];
-#if defined(WIFEXITED) && defined(WEXITSTATUS)
-	if ((ret != -1) && WIFEXITED(ret))
-	  {
-	     ret = WEXITSTATUS(ret);
-	  }
-#endif
-	if (ret)
-	  {
-	     fprintf (stderr, "Command returned exit status %d.  Press RETURN.\n", ret);
-	     fgets (buf, 255, stdin);
-	  }
+	char buf[SLRN_MAX_PATH_LEN];
+	fprintf (stderr, "Command returned exit status %d.  Press RETURN.\n", ret);
+	fgets (buf, sizeof(buf), stdin);
      }
-   
+
    slrn_set_display_state (SLRN_TTY_INIT | SLRN_SMG_INIT);
    return 0;
 #else
@@ -794,7 +884,7 @@ FILE *slrn_popen (char *cmd, char *mode) /*{{{*/
      {
 	char buf[256];
 	fprintf (stderr, "Command %s failed to run.  Press RETURN.\n", cmd);
-	fgets (buf, 255, stdin);
+	fgets (buf, sizeof(buf), stdin);
 	slrn_set_display_state (SLRN_TTY_INIT | SLRN_SMG_INIT);
      }
    return fp;
@@ -808,33 +898,6 @@ FILE *slrn_popen (char *cmd, char *mode) /*{{{*/
 
 /*}}}*/
 
-int slrn_posix_system (char *cmd, int reset) /*{{{*/
-{
-   int init_mode = Slrn_TT_Initialized;
-#ifndef VMS
-   void (*sint)(int);
-   void (*squit)(int);
-   int ret;
-   
-   sint = signal (SIGINT, SIG_IGN);
-   squit = signal (SIGQUIT, SIG_IGN);
-   if (reset) slrn_set_display_state (0);
-   ret = system (cmd);
-   signal (SIGINT, sint);
-# ifdef SIGQUIT
-   signal (SIGQUIT, squit);
-# endif
-#else
-   int ret;
-   if (reset) slrn_set_display_state (0);
-   ret = system (cmd);
-#endif
-   if (reset) slrn_set_display_state (init_mode);
-   Slrn_Full_Screen_Update = 1;
-   return ret;
-}
-
-/*}}}*/
 static int create_edit_command (char *edit, char *cmd, char *file, unsigned int line) /*{{{*/
 {
    int d, s;
@@ -906,10 +969,10 @@ int slrn_edit_file (char *editor, char *file, unsigned int line) /*{{{*/
        && (NULL == (editor = getenv("EDITOR")))
        && (NULL == (editor = getenv("VISUAL"))))
      {
-#ifdef VMS
+#if defined(VMS) || defined(__NT__) || defined(__MINGW32__)
 	editor = "edit";
 #else
-# ifdef __os2__
+# if defined(__os2__)
 	editor = "e";
 # else
 #  ifdef __unix__
@@ -987,7 +1050,10 @@ static SLang_RLine_Info_Type *init_readline (void) /*{{{*/
    rli->getkey = SLang_getkey;
    rli->tt_goto_column = NULL;
    rli->update_hook = rline_update;
-   
+#ifdef SL_RLINE_BLINK_MATCH
+   rli->flags |= SL_RLINE_BLINK_MATCH;
+   rli->input_pending = SLang_input_pending;
+#endif
    if (SLang_init_readline (rli) < 0)
      {
 	SLFREE (rli);
@@ -1039,17 +1105,34 @@ static int read_from_input_string (char *str)
 /* s could be NULL.  If so, input string is cleared. */
 void slrn_set_input_string (char *s)
 {
-   if (Input_String != NULL)
-     {
-	SLFREE (Input_String);
-	Input_String = NULL;
-     }
-   
+   slrn_free (Input_String);
    Input_String = s;
    Input_String_Ptr = s;
 }
 
-static int generic_read_input (char *prompt, char *dfl, char *str, int trim_flag, int no_echo) /*{{{*/
+static char read_from_input_char (void)
+{
+   if (Input_Chars_Ptr == NULL)
+     return 0;
+   
+   if (*Input_Chars_Ptr == 0)
+     {
+	slrn_set_input_chars (NULL);
+	return 0;
+     }
+   
+   return *Input_Chars_Ptr++;
+}
+
+void slrn_set_input_chars (char *s)
+{
+   slrn_free (Input_Chars);
+   Input_Chars = s;
+   Input_Chars_Ptr = s;
+}
+
+static int generic_read_input (char *prompt, char *dfl, char *str, int trim_flag, 
+			       int no_echo, int point) /*{{{*/
 {
    int i;
    int tt_init_state;
@@ -1057,6 +1140,8 @@ static int generic_read_input (char *prompt, char *dfl, char *str, int trim_flag
    unsigned int len;
    int save_slang_error;
    
+   Slrn_Full_Screen_Update = 1;
+
    strncpy (prompt_buf, prompt, sizeof (prompt_buf));
    prompt_buf[sizeof(prompt_buf) - 1] = 0;
    len = strlen (prompt);
@@ -1096,7 +1181,11 @@ static int generic_read_input (char *prompt, char *dfl, char *str, int trim_flag
    if ((str != NULL) && *str)
      {
 	strcpy ((char *) Slrn_Keymap_RLI->buf, str);
-	Slrn_Keymap_RLI->point = 0;   /* strlen (str); */
+	if (point == 0)
+	  Slrn_Keymap_RLI->point = 0;
+	else
+	  Slrn_Keymap_RLI->point = strlen (str);
+
 	*str = 0;
      }
    if (str == NULL) str = dfl;
@@ -1144,7 +1233,7 @@ static int generic_read_input (char *prompt, char *dfl, char *str, int trim_flag
 	
 	if (*b) 
 	  {
-	     SLang_rline_save_line (Slrn_Keymap_RLI);
+	     if (no_echo == 0) SLang_rline_save_line (Slrn_Keymap_RLI);
 	     if (trim_flag) 
 	       {
 		  slrn_trim_string (b);
@@ -1183,14 +1272,14 @@ static int generic_read_input (char *prompt, char *dfl, char *str, int trim_flag
 
 /*}}}*/
 
-int slrn_read_input (char *prompt, char *dfl, char *str, int trim_flag)
+int slrn_read_input (char *prompt, char *dfl, char *str, int trim_flag, int point)
 {
-   return generic_read_input (prompt, dfl, str, trim_flag, 0);
+   return generic_read_input (prompt, dfl, str, trim_flag, 0, point);
 }
 
-int slrn_read_input_no_echo (char *prompt, char *dfl, char *str, int trim_flag)
+int slrn_read_input_no_echo (char *prompt, char *dfl, char *str, int trim_flag, int point)
 {
-   return generic_read_input (prompt, dfl, str, trim_flag, 1);
+   return generic_read_input (prompt, dfl, str, trim_flag, 1, point);
 }
 
 int slrn_read_integer (char *prompt, int *dflt, int *np) /*{{{*/
@@ -1207,7 +1296,7 @@ int slrn_read_integer (char *prompt, int *dflt, int *np) /*{{{*/
      }
    
    *str = 0;
-   if (-1 == (n = slrn_read_input (prompt, sdfl, str, 1)))
+   if (-1 == (n = slrn_read_input (prompt, sdfl, str, 1, 0)))
      {
 	slrn_error ("Abort!");
 	return -1;
@@ -1231,41 +1320,49 @@ char slrn_get_response (char *valid_chars, char *str, ...) /*{{{*/
    char *v;
    
    /* if (SLang_Error) return -1; */
-   
+   if (Error_Present)
+     slrn_error_now (2, NULL);
+
    while (1)
      {
-	if (Slrn_TT_Initialized == 0)
-	  {
-	     char buf[256];
-	     
-	     va_start(ap, str);
-	     slrn_tty_vmessage (stdout, str, ap);
-	     va_end(ap);
+	Slrn_Full_Screen_Update = 1;
 
-	     *buf = 0;
-	     (void) fgets (buf, sizeof(buf), stdin);
-	     ch = *buf;
-	  }
-	else
-	  {
-	     SLang_flush_input ();
-	     slrn_clear_message ();
-
-	     va_start(ap, str);
-	     vmessage_1 (0, str, ap);
-	     va_end(ap);
-	     
-	     slrn_smg_refresh ();
-	     
-	     Reading_Input = 2;
-	     ch = SLang_getkey ();
-	     Reading_Input = 0;
-	     
-	     slrn_clear_message ();
-	     SLang_Error = SLKeyBoard_Quit = 0;
-	     
-	  }
+	ch = read_from_input_char ();
 	
+	if (ch == 0)
+	  {
+	     if (Slrn_TT_Initialized == 0)
+	       {
+		  char buf[256];
+	  
+		  va_start(ap, str);
+		  slrn_tty_vmessage (stdout, str, ap);
+		  va_end(ap);
+		  
+		  *buf = 0;
+		  (void) fgets (buf, sizeof(buf), stdin);
+		  ch = *buf;
+	       }
+	     else 
+	       {
+		  SLang_flush_input ();
+		  slrn_clear_message ();
+		  
+		  va_start(ap, str);
+		  vmessage_1 (0, str, ap);
+		  va_end(ap);
+		  
+		  slrn_smg_refresh ();
+		  
+		  Reading_Input = 2;
+		  ch = SLang_getkey ();
+		  Reading_Input = 0;
+	     
+		  slrn_clear_message ();
+		  SLang_Error = SLKeyBoard_Quit = 0;
+	       }
+	  }
+
 	v = valid_chars;
 	while (*v)
 	  {
@@ -1273,7 +1370,7 @@ char slrn_get_response (char *valid_chars, char *str, ...) /*{{{*/
 	     v++;
 	  }
 	
-	slrn_error_now ("Invalid response! Try again.");
+	slrn_error_now (0, "Invalid response! Try again.");
 	if (Slrn_TT_Initialized & SLRN_TTY_INIT)
 	  {
 	     (void) SLang_input_pending (15);
@@ -1298,12 +1395,12 @@ int slrn_get_yesno (int dflt, char *str, ...) /*{{{*/
    if (dflt)
      {
 	ch = 'y';
-	fmt = "? ([y]/n)";
+	fmt = "? [\001Y]es, \001No";
      }
    else
      {
 	ch = 'n';
-	fmt = "? (y/[n])";
+	fmt = "? \001Yes, [\001N]o";
      }
    
    strcat (buf, fmt);
@@ -1330,7 +1427,7 @@ int slrn_get_yesno_cancel (char *str, ...) /*{{{*/
    (void) vsprintf(buf, str, ap);
    va_end(ap);
    
-   strcat (buf, "? [Y]-es, N-o, C-ancel");
+   strcat (buf, "? [\001Y]es, \001No, \001Cancel");
    
    switch (slrn_get_response ("\007yYnNcC\r", buf))
      {
@@ -1423,14 +1520,14 @@ void slrn_get_user_info (void) /*{{{*/
 {
    struct hostent *host_entry;
    char *name, *host;
-#ifndef VMS
+#ifdef HAS_PASSWORD_CODE
    struct passwd *pw;
-   
-   pw = getpwuid (getuid ());
 #endif
    
    /* Fill in what is assumed to be non-NULL by rest of program. */
    Slrn_User_Info.followup_string = slrn_safe_strmalloc ("In article %m, %r wrote:");
+   Slrn_User_Info.reply_string = slrn_safe_strmalloc ("In %n, you wrote:");
+
    Slrn_Courtesy_CC_Message = slrn_safe_strmalloc ("[This message has also been posted.]");
    
    /* Now get default values for rest. */
@@ -1501,7 +1598,7 @@ void slrn_get_user_info (void) /*{{{*/
 #ifdef OUR_HOSTNAME
 # ifdef __unix__
    /* If the value is a file, read the file for the FQDN */
-   if (OUR_HOSTNAME[0] == '/')
+   if (slrn_is_absolute_path (OUR_HOSTNAME))
      {
 	FILE *fp;
 	
@@ -1539,23 +1636,26 @@ void slrn_get_user_info (void) /*{{{*/
 	Slrn_User_Info.host[MAX_HOST_NAME_LEN - 1] = 0;
      }
    
-   if (
 #ifdef VMS
-       ((name = slrn_vms_getlogin()) == NULL)
-#else 
-       /* I cannot use getlogin under Unix because some implementations 
-	* truncate the username to 8 characters.  Besides, I suspect that
-	* it is equivalent to the following line.
-	*/
-       ((pw == NULL) 
-	|| (NULL == (name = pw->pw_name))
-	|| (*name == 0))
+   name = slrn_vms_getlogin();
+#else
+   name = NULL;
+# ifdef HAS_PASSWORD_CODE
+   /* I cannot use getlogin under Unix because some implementations 
+    * truncate the username to 8 characters.  Besides, I suspect that
+    * it is equivalent to the following line.
+    */
+   pw = getpwuid (getuid ());
+   if (pw != NULL)
+     name = pw->pw_name;
+# endif
 #endif
+   
+   if (((name == NULL) || (*name == 0))
        && ((name = getenv("USER")) == NULL)
        && ((name = getenv("LOGNAME")) == NULL))
      name = "";
 
-
    Slrn_User_Info.username = slrn_safe_strmalloc (name);
    Slrn_User_Info.login_name = slrn_safe_strmalloc (name);
 
@@ -1567,7 +1667,7 @@ void slrn_get_user_info (void) /*{{{*/
    Slrn_User_Info.realname = slrn_vms_fix_fullname(slrn_vms_get_uaf_fullname());
 #else
    if (((Slrn_User_Info.realname = getenv ("NAME")) == NULL)
-# ifndef __BEOS__
+# ifdef HAS_PASSWORD_CODE
        && ((pw == NULL) || ((Slrn_User_Info.realname = pw->pw_gecos) == NULL))
 # endif
        )
@@ -1591,7 +1691,7 @@ void slrn_get_user_info (void) /*{{{*/
      {
 	/* Check to see if this is an organization file. */
 	char orgbuf[512];
-	if (*Slrn_User_Info.org == '/')
+	if (slrn_is_absolute_path (Slrn_User_Info.org))
 	  {
 	     FILE *fporg;
 	     if (NULL != (fporg = fopen (Slrn_User_Info.org, "r")))
