@@ -1,6 +1,23 @@
-/*  Copyright (c) 1995 John E. Davis (davis@space.mit.edu)
- *  All rights reserved.
+/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
+ *
+ * This file is part of slrn.
+ *
+ * Slrn is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ * 
+ * Slrn is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with Slrn; see the file COPYING.  If not, write to the Free
+ * Software Foundation, 59 Temple Place - Suite 330, 
+ * Boston, MA  02111-1307, USA.
  */
+
 #include "config.h"
 #include "slrnfeat.h"
 
@@ -49,7 +66,6 @@ static void _nntp_connection_lost_hook (NNTP_Type *s)
      }
 }
 
-
 static int _nntp_interrupt_handler (void)
 {
    (void) slrn_handle_interrupts ();
@@ -60,6 +76,15 @@ static int _nntp_interrupt_handler (void)
    return 0;
 }
 
+static void _nntp_reset (void)
+{
+   NNTP_Type *s = NNTP_Server;
+   
+   nntp_disconnect_server (s);
+   /* The next NNTP command will actually perform the reset. */
+}
+
+   
 static void _nntp_close_server (void)
 {
    NNTP_Type *s = NNTP_Server;
@@ -300,6 +325,7 @@ static int nntp_init_objects (void)
    NNTP_Server_Obj.sv_nntp_xover = _nntp_xover_cmd;
    NNTP_Server_Obj.sv_nntp_head = _nntp_head_cmd;
    NNTP_Server_Obj.sv_nntp_next = _nntp_next_cmd;
+   NNTP_Server_Obj.sv_reset = _nntp_reset;
    return 0;
 }
 
