@@ -1,7 +1,24 @@
-/*  Copyright (c) 1995 John E. Davis (davis@space.mit.edu)
- *  All rights reserved.
- */
 /* post an article */
+/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
+ *
+ * This file is part of slrn.
+ *
+ * Slrn is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ * 
+ * Slrn is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with Slrn; see the file COPYING.  If not, write to the Free
+ * Software Foundation, 59 Temple Place - Suite 330, 
+ * Boston, MA  02111-1307, USA.
+ */
+
 #include "config.h"
 #include "slrnfeat.h"
 
@@ -36,6 +53,7 @@
 #include "uudecode.h"
 #include "util.h"
 #include "chmap.h"
+#include "menu.h"
 
 #if SLRN_HAS_MIME
 # include "mime.h"
@@ -43,9 +61,12 @@
 
 char *Slrn_Courtesy_CC_Message = NULL;
 char *Slrn_Save_Posts_File;
+char *Slrn_Save_Replies_File;
 char *Slrn_Last_Message_Id;
 char *Slrn_Post_Custom_Headers;
 int Slrn_Reject_Long_Lines = 1;
+char *Slrn_Postpone_Dir;
+int Slrn_Generate_Message_Id = 1;
 
 #ifdef VMS
 # define SYSTEM_OS_NAME		"VMS"
@@ -53,14 +74,23 @@ int Slrn_Reject_Long_Lines = 1;
 # ifdef __os2__
 #  define SYSTEM_OS_NAME	"OS/2"
 # else
-#  ifdef __unix__
-#   define SYSTEM_OS_NAME       "UNIX"
+#  ifdef AMIGA
+#   define SYSTEM_OS_NAME	"Amiga"
 #  else
-#   define SYSTEM_OS_NAME	"UNKNOWN"
+#   ifdef __unix__
+#    define SYSTEM_OS_NAME       "UNIX"
+#   else
+#    if defined(__WIN32__) || defined(__NT__)
+#     define SYSTEM_OS_NAME	"Windows"
+#    else
+#     define SYSTEM_OS_NAME	"UNKNOWN"
+#    endif
+#   endif
 #  endif
 # endif
 #endif
 
+static int postpone_file (char *);
 
 #if SLRN_HAS_GEN_MSGID
 static char *slrn_create_message_id (void)
@@ -71,6 +101,9 @@ static char *slrn_create_message_id (void)
    char *chars32 = "0123456789abcdefghijklmnopqrstuv";
    static unsigned long last_now;
    
+   if (Slrn_Generate_Message_Id == 0)
+     return NULL;
+
    while (1)
      {
 	if ((Slrn_User_Info.posting_host == NULL)
@@ -123,9 +156,10 @@ static char *slrn_create_message_id (void)
 void slrn_add_signature (FILE *fp)
 {
    FILE *sfp;
-   char buf[256];
+   char file[SLRN_MAX_PATH_LEN];
+   char buf [256];
    
-   if ((sfp = slrn_open_home_file (Slrn_User_Info.signature, "r", buf, 0)) != NULL)
+   if ((sfp = slrn_open_home_file (Slrn_User_Info.signature, "r", file, 0)) != NULL)
      {
 	/* Apparantly some RFC suggests the -- \n. */
         fputs ("\n\n-- \n", fp);
@@ -162,13 +196,17 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
 #if defined(VMS) || !SLRN_HAS_PIPING
    return -1;
 #else
+#if defined(IBMPC_SYSTEM)
+   char outfile [SLRN_MAX_PATH_LEN];
+#endif
    FILE *pp, *fp;
    char line[1024];
-   unsigned char *b;
    unsigned int cc_line = 0;
    unsigned int linenum;
-   int num_ccd;
-   
+   char buf [1024];
+   unsigned int nth;
+   char *l;
+
    if (NULL == (fp = fopen (file, "r")))
      {
 	slrn_error ("Unable to open %s.");
@@ -183,8 +221,8 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
 	if (0 == slrn_case_strncmp ((unsigned char *)line,
 				    (unsigned char *) "Cc: ", 4))
 	  {
-	     b = (unsigned char *) slrn_skip_whitespace (line + 4);
-	     if (*b && (*b != ',')) cc_line = linenum;
+	     l = slrn_skip_whitespace (line + 4);
+	     if (*l && (*l != ',')) cc_line = linenum;
 	     break;
 	  }
      }
@@ -197,7 +235,11 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
 	return -1;
      }
    
+#if defined(IBMPC_SYSTEM)
+   pp = slrn_open_tmpfile (outfile, "w");
+#else
    pp = slrn_popen (Slrn_SendMail_Command, "w");
+#endif
    if (pp == NULL)
      {
 	slrn_fclose (fp);
@@ -205,49 +247,40 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
      }
    
    fputs ("To: ", pp);
-   num_ccd = 0;
-   b = (unsigned char *) line + 4;
-   while (*b != 0)
-     {
-	unsigned char *bmin, ch;
-	/* skip to first non-whitespace, non-comma char */
-        if ((*b <= ' ') || (*b == ','))
-	  {
-	     b++;
-	     continue;
-	  }
-	
-	bmin = b;
-	/* Now skip past to white space or comma */
-	while (((ch = *b) != ',') && (ch > ' ')) b++;
-	
-	if ((b == bmin + 6)
-	    && (0 == slrn_case_strncmp (bmin, (unsigned char *) "poster", 6)))
-	  {
-	     if (to != NULL)
-	       {
-		  if (num_ccd) putc (',', pp);
-		  fputs (to, pp);
-		  num_ccd++;
-		  to = NULL;
-	       }
-	  }
-	else
-	  {
-	     if (num_ccd) putc (',', pp);
-	     fwrite ((char *)bmin, 1, (unsigned int) (b - bmin), pp);
-	     num_ccd++;
-	  }
-     }
-   putc ('\n', pp);
    
+   /* This line consists of a comma separated list of addresses.  In
+    * particular, "poster" will be replaced by 'to'.
+    */
+   l = line + 4;
+   nth = 0;
+   while (0 == SLextract_list_element (l, nth, ',', buf, sizeof (buf)))
+     {
+	char *b;
+
+	/* Knock whitespace from ends of string */
+	b = slrn_skip_whitespace (buf);
+	slrn_trim_string (b);
+
+	if (nth != 0)
+	  putc (',', pp);
+	
+	if ((0 == slrn_case_strcmp ((unsigned char *)b, (unsigned char *)"poster"))
+	    && (to != NULL))
+	  b = to;
+	
+	fputs (b, pp);
+	nth++;
+     }
+
+   putc ('\n', pp);
    rewind (fp);
+
    linenum = 0;
    
-#ifdef __os2__
+#if SLRN_GEN_FROM_EMAIL_HEADER
    fprintf (pp, "From: %s@%s (%s)\n",
             Slrn_User_Info.username, Slrn_User_Info.host,
-            Slrn_User_Info.realname);
+            ((Slrn_User_Info.realname == NULL) ? "" : Slrn_User_Info.realname));
 #endif
    
    if (msgid != NULL)
@@ -277,13 +310,6 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
 	  fputs (line, pp);
      }
 
-#ifdef __os2__
-   /* Add the correct From: line under OS/2 Warp */
-   fprintf (pp, "From: %s@%s (%s)\n",
-	    Slrn_User_Info.username, Slrn_User_Info.host,
-	    ((Slrn_User_Info.realname == NULL) ? "" : Slrn_User_Info.realname));
-#endif
-   
 # if SLRN_HAS_MIME
    if (Slrn_Use_Mime) slrn_mime_add_headers (pp);
 # endif
@@ -291,7 +317,10 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
    fputs ("\n", pp);
    
    if ((NULL != Slrn_Courtesy_CC_Message) && (*Slrn_Courtesy_CC_Message))
-     fprintf (pp, "%s\n", Slrn_Courtesy_CC_Message);
+     {
+	slrn_insert_followup_format (Slrn_Courtesy_CC_Message, pp);
+	fputs ("\n", pp);
+     }
    
 # if SLRN_HAS_MIME
    if (Slrn_Use_Mime) fp = slrn_mime_encode (fp);
@@ -302,24 +331,29 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
 	fputs (line, pp);
      }
    slrn_fclose (fp);
+# if defined(IBMPC_SYSTEM)
+   slrn_fclose (pp);
+   sprintf (buf, "%s %s", Slrn_SendMail_Command, outfile);
+   slrn_posix_system (buf, 0);
+# else
    slrn_pclose (pp);
+# endif
    return 0;
-#endif
+#endif				       /* NOT VMS */
 }
 
+/* Returns -1 upon error, 0, if ok, 1 if needs repair, 2 is warning is issued */
 static int check_file_for_posting (char *file)
 {
    char line[1024], *the_line;
    FILE *fp;
    unsigned int num;
    char *err;
-   int rsp;
-   int newsgroups_found, subject_found;
+   int newsgroups_found, subject_found, only_warning;
    char ch;
    char *colon;
    
-   try_again:
-   newsgroups_found = subject_found = 0;
+   only_warning = newsgroups_found = subject_found = 0;
    err = NULL;
    fp = fopen (file, "r");
    
@@ -435,6 +469,8 @@ static int check_file_for_posting (char *file)
 	       {
 		  err = "Line contains more than 80 characters.  You need to wrap it.";
 		  the_line = line;
+		  if (Slrn_Reject_Long_Lines == 2)
+		    only_warning = 1;
 		  break;
 	       }
 	  }
@@ -468,7 +504,7 @@ static int check_file_for_posting (char *file)
 	SLsmg_printf ("This message was generated while looking at line %d:", num);
 	SLsmg_gotorc (8,0);
 	slrn_set_color (QUOTE_COLOR);
-	SLsmg_printf (the_line);
+	SLsmg_write_string (the_line);
 	SLsmg_gotorc (12, 0);
 	slrn_set_color (ARTICLE_COLOR);
 	SLsmg_write_string ("Perhaps this error was generated because you did not separate the header");
@@ -476,17 +512,123 @@ static int check_file_for_posting (char *file)
 	SLsmg_write_string ("section from the body by a BLANK line.");
      }
    
-   rsp = slrn_get_response ("yYnNcC\007", "Press 'Y' to re-edit or 'C' to cancel");
-   rsp |= 0x20;
-   if (rsp == 'y')
-     {
-	if (slrn_edit_file (Slrn_Editor_Post, file, num) < 0) return -1;
-	goto try_again;
-     }
-   return -1;
+   if (only_warning) return 2;
+   return 1;
 }
 
-int slrn_post_file (char *file, char *to)
+
+int slrn_save_file_to_mail_file (char *file, char *save_file, char *msgid)
+{
+   FILE *infp, *outfp;
+   time_t now;
+   char save_post_file[SLRN_MAX_PATH_LEN];
+   char line [1024];
+
+   if ((save_file == NULL) || (*save_file == 0))
+     return 0;
+	
+   if (NULL == (infp = fopen (file, "r")))
+     {
+	slrn_error ("File not found: %s--- message not posted.", file);
+	return -1;
+     }
+	
+   if (NULL == (outfp = slrn_open_home_file (save_file, "a",
+					     save_post_file, 1)))
+     {
+	slrn_error ("Error saving to %s", save_post_file);
+	slrn_fclose (infp);
+	return -1;
+     }
+	
+   time (&now);
+   fprintf (outfp, "From %s@%s %s", Slrn_User_Info.username, Slrn_User_Info.host, ctime(&now));
+   fprintf (outfp, "From: %s@%s (%s)\n", Slrn_User_Info.username, Slrn_User_Info.host, Slrn_User_Info.realname);
+   if (msgid != NULL) fprintf (outfp, "Message-Id: <slrn%s@%s>\n", msgid, Slrn_User_Info.posting_host);
+
+   while (NULL != fgets (line, sizeof(line) - 1, infp))
+     {
+	if ((*line == 'F')
+	    && !strncmp ("From", line, 4)
+	    && ((unsigned char)line[4] <= ' '))
+	  {
+	     putc ('>', outfp);
+	  }
+	
+	fputs (line, outfp);
+     }
+   fputs ("\n\n", outfp);	       /* separator */
+   slrn_fclose (infp);
+   return slrn_fclose (outfp);
+}
+
+static int post_references_header (char *line)
+{
+#define GNKSA_LENGTH 998
+   char buf[GNKSA_LENGTH + 1];
+   char *p;
+   unsigned int total_len, len;
+
+   /* Make sure line does not end in whitespace */
+   (void) slrn_trim_string (line);
+   
+   len = strlen (line);
+   if (len <= GNKSA_LENGTH)
+     {
+	Slrn_Post_Obj->po_puts (line);
+	Slrn_Post_Obj->po_puts ("\n");
+	return 0;
+     }
+
+   /* The reference header is too long so start truncating */
+   /* We have to preserve the first message-id */
+   p = slrn_strchr (line, '>');
+   if (p == NULL) return -1;
+   p++;
+   len = p - line;
+   if (len >= GNKSA_LENGTH)
+     {
+	/* Must have been generated by Pine */
+	Slrn_Post_Obj->po_puts (line);
+	Slrn_Post_Obj->po_puts ("\n");
+	return 0;
+     }
+   strncpy (buf, line, len);
+   buf[len] = 0;
+   Slrn_Post_Obj->po_puts (buf);
+   Slrn_Post_Obj->po_puts (" ");
+   total_len = len + 1;
+   
+   line += len;
+   if ((*line == '\t') || (*line == ' ')) 
+     line++;	       /* already handled space */
+
+   /* Now use message-ids at end of the line */
+   p = line + strlen (line);
+
+   while (1)
+     {
+	if (*p == '<')
+	  {
+	     if (strlen (p) + total_len > GNKSA_LENGTH)
+	       {
+		  p = slrn_strchr (p + 1, '<');
+		  if (p == NULL) p = line;
+		  break;
+	       }
+	  }
+	if (p == line) break;
+	p--;
+     }
+   
+   Slrn_Post_Obj->po_puts (p);
+   Slrn_Post_Obj->po_puts ("\n");
+   return 0;
+}
+
+
+/* This function returns 1 if postponed, 0 upon sucess, -1 upon error */
+int slrn_post_file (char *file, char *to, int is_postponed)
 {
    char line[1024], *linep;
    int len, header;
@@ -498,68 +640,135 @@ int slrn_post_file (char *file, char *to)
 #if SLRN_HAS_GEN_MSGID
    int has_messageid = 0;
 #endif
+#if SLRN_HAS_SLANG
+   SLang_Name_Type *filter_hook;
+#endif
+   int once_more;
    
+#if SLRN_HAS_SLANG
+   filter_hook = SLang_get_function ("post_filter_hook");
+#endif
+
    try_again:
+
    perform_cc = 0;
-   if (Slrn_Batch == 0) while (1)
+   once_more = 1;
+
+   if (Slrn_Batch == 0) while (once_more)
      {
-	rsp = slrn_get_response ("yYnNeE", "Post the message? Y-es, N-o, E-dit");
-	rsp |= 0x20;
-	if (rsp == 'n') return -1;
-	if (rsp == 'y') break;
-	if (slrn_edit_file (Slrn_Editor_Post, file, 1) < 0) 
+#if SLRN_HAS_SLANG
+	if (filter_hook != NULL)
 	  {
-	     slrn_error ("User abort.");
+	     if (is_postponed)
+	       rsp = slrn_get_response ("yYnNeEsSDdFf", "Post the message? \001Yes, \001No, \001Edit, po\001Stpone, \001Delete, \001Filter");
+	     else
+	       rsp = slrn_get_response ("yYnNeEsSFf", "Post the message? \001Yes, \001No, \001Edit, po\001Stpone, \001Filter");
+	  }
+	else
+	  {
+#endif
+	     if (is_postponed)
+	       rsp = slrn_get_response ("yYnNeEsSDd", "Post the message? \001Yes, \001No, \001Edit, po\001Stpone, \001Delete");
+	     else
+	       rsp = slrn_get_response ("yYnNeEsS", "Post the message? \001Yes, \001No, \001Edit, po\001Stpone");
+#if SLRN_HAS_SLANG
+	  }
+#endif
+	switch (rsp)
+	  {
+	   case 'n':
+	   case 'N':
 	     return -1;
+	     
+	   case 's':
+	   case 'S':
+	     
+	     if (is_postponed
+		 || (0 == postpone_file (file)))
+	       return 1;
+
+	     /* Instead of returning, let user have another go at it */
+	     /* return -1; */
+	     break;
+	     
+	   case 'd':
+	   case 'D':
+	     rsp = slrn_get_yesno_cancel ("Sure you want to delete it");
+	     if (rsp == 0)
+	       continue;
+	     if (rsp == -1)
+	       return 1;
+	     /* Calling routine will delete it. */
+	     return 0;
+	  
+	   case 'y': case 'Y':
+	     if (0 == (rsp = check_file_for_posting (file)))
+	       {
+		  once_more = 0;
+		  break;
+	       }
+	     if (rsp == -1)
+	       return -1;
+	     
+	     SLtt_beep ();
+	     if (rsp == 1)
+	       rsp = slrn_get_response ("yYEenNcC\007", "re-\001Edit,  or \001Cancel");
+	     else
+	       rsp = slrn_get_response ("EeyYnNcC\007Ff", "re-\001Edit, \001Cancel, or \001Force the posting (not recommended)");
+	     
+	     switch (rsp)
+	       {
+		case 'y': case 'Y': case 'E': case 'e':
+		  rsp = 'y';
+		  break;
+		  
+		case 'c': case 'C': case 7:
+		  return -1;
+		  
+		case 'F': case 'f':
+		  once_more = 0;
+	       }
+
+	     if (rsp != 'y')
+	       break;
+	     
+	     /* Drop */
+	     
+	   case 'e':
+	   case 'E':
+	     if (slrn_edit_file (Slrn_Editor_Post, file, 1) < 0) 
+	       {
+		  slrn_error ("User abort.");
+		  return -1;
+	       }
+	     break;
+
+#if SLRN_HAS_SLANG
+	   case 'F':
+	   case 'f':
+	     if (filter_hook != NULL)
+	       {
+		  SLang_push_string (file);
+		  if (SLang_Error == 0)
+		    SLexecute_function (filter_hook);
+		  if (SLang_Error) 
+		    filter_hook = NULL;
+	       }
+#endif
 	  }
      }
    
-   if (-1 == check_file_for_posting (file)) return -1;
-   
    slrn_message_now ("Posting ...");
    
-   if ((Slrn_Save_Posts_File != NULL) && (*Slrn_Save_Posts_File))
-     {
-	FILE *infp, *outfp;
-	time_t now;
-	char save_post_file[256];
-	
-	if (NULL == (infp = fopen (file, "r")))
-	  {
-	     slrn_error ("File not found: %s--- message not posted.", file);
-	     return -1;
-	  }
-	
-	if (NULL == (outfp = slrn_open_home_file (Slrn_Save_Posts_File, "at",
-						  save_post_file, 1)))
-	  {
-	     slrn_error ("Error saving to %s", save_post_file);
-	     slrn_fclose (infp);
-	     return -1;
-	  }
-	
-	time (&now);
-	fprintf (outfp, "From %s@%s %s", Slrn_User_Info.username, Slrn_User_Info.host, ctime(&now));
-	fprintf (outfp, "From: %s@%s (%s)\n", Slrn_User_Info.username, Slrn_User_Info.host, Slrn_User_Info.realname);
-	
-	while (fgets (line, sizeof(line) - 1, infp) != NULL)
-	  {
-	     if ((*line == 'F')
-		 && !strncmp ("From", line, 4)
-		 && ((unsigned char)line[4] <= ' '))
-	       {
-		  putc ('>', outfp);
-	       }
-	     
-	     fputs (line, outfp);
-	  }
-	fputs ("\n\n", outfp);	       /* separator */
-	slrn_fclose (infp);
-	slrn_fclose (outfp);
-     }
-   
+#if SLRN_HAS_GEN_MSGID
+   msgid = slrn_create_message_id ();
+#endif
+
+   if (-1 == slrn_save_file_to_mail_file (file, Slrn_Save_Posts_File, msgid))
+     return -1;
+
    slrn_chmap_fix_file (file);
-   
+
    if ((fp = fopen (file, "r")) == NULL)
      {
 	slrn_error ("File not found: %s--- message not posted.", file);
@@ -570,11 +779,31 @@ int slrn_post_file (char *file, char *to)
    if (Slrn_Use_Mime)
      slrn_mime_scan_file (fp);
 #endif
+#if SLRN_HAS_SLANG
+   if (NULL != SLang_get_function ("post_file_hook"))
+     {
+	fclose (fp);
+	(void) SLang_run_hooks ("post_file_hook", 1, file);
+	if (SLang_Error)
+	  {
+	     slrn_error ("post_file_hook returned error.  %s not posted.", file);
+	     return -1;
+	  }
+
+	if ((fp = fopen (file, "r")) == NULL)
+	  {
+	     slrn_error ("File not found: %s--- message not posted.", file);
+	     return -1;
+	  }
+	slrn_message_now ("Posting ...");
+     }
+#endif
 
    /* slrn_set_suspension (1); */
    status = Slrn_Post_Obj->po_start ();
    if (status != CONT_POST)
      {
+	/* FIXME!! Put the file in a known place, e.g., dead.letter */
 	if (status != -1)
 	  slrn_error ("Posting not allowed.");
 	return -1;
@@ -593,18 +822,35 @@ int slrn_post_file (char *file, char *to)
      {
 	len = strlen (linep);
 	if (len == 0) continue;
+
 	if (header)
 	  {
 	     unsigned char *b;
 	     char *white = linep;
-	     
+
+	     /* Check the references header.  Rumor has it that many
+	      * servers choke on long references lines.  In fact, to be
+	      * GNKSA compliant, references headers cannot be longer than
+	      * 998 characters.  Sigh.
+	      */
+
+	     if (((*white == 'R') || (*white == 'r'))
+		 && (0 == slrn_case_strncmp ((unsigned char *)"References: ",
+					     (unsigned char *)white,
+					     12)))
+	       {
+		  (void) post_references_header (white);
+		  continue;
+	       }
+
 	     while ((*white == ' ') || (*white == '\t')) white++;
+
 	     if (*white == '\n')
 	       {
 #if SLRN_HAS_GEN_MSGID
 		  if (has_messageid == 0)
 		    {
-		       Slrn_Last_Message_Id = msgid = slrn_create_message_id ();
+		       Slrn_Last_Message_Id = msgid;
 		       if (msgid != NULL)
 			 {
 			    Slrn_Post_Obj->po_printf ("Message-Id: <slrn%s@%s>\n", msgid, Slrn_User_Info.posting_host);
@@ -615,8 +861,14 @@ int slrn_post_file (char *file, char *to)
 		  if (Slrn_Use_Mime)
 		    slrn_mime_add_headers (0);   /* 0 --> Slrn_Post_Obj->po_puts */
 #endif
-		  Slrn_Post_Obj->po_printf ("X-Newsreader: slrn (%s %s)\n\n", Slrn_Version, 
+#if SLRN_HAS_USER_AGENT
+		  Slrn_Post_Obj->po_printf ("User-Agent: slrn/%s (%s)\n\n", 
+					    Slrn_Version, 
+					    SYSTEM_OS_NAME);
+#else
+		  Slrn_Post_Obj->po_printf ("X-Newsreader: slrn (%s %s)\n\n", Slrn_Version,
 			       SYSTEM_OS_NAME);
+#endif
 		  header = 0;
 #if SLRN_HAS_MIME
 		  if (Slrn_Use_Mime) fp = slrn_mime_encode (fp);
@@ -633,6 +885,7 @@ int slrn_post_file (char *file, char *to)
 		  if (*b && (*b != ',')) perform_cc = 1;
 		  continue;
 	       }
+
 	     if (is_empty_header (linep)) continue;
 #if SLRN_HAS_GEN_MSGID
 	     if (!slrn_case_strncmp ((unsigned char *)"Message-Id: ",
@@ -678,7 +931,7 @@ int slrn_post_file (char *file, char *to)
 	slrn_smg_refresh ();
 	sleep (2);
 	slrn_clear_message (); SLang_Error = 0;
-	rsp = slrn_get_response ("RrEeCc\007", "Select one: R-epost, E-dit, C-ancel");
+	rsp = slrn_get_response ("RrEeCc\007", "Select one: \001Repost, \001Edit, \001Cancel");
 	if (rsp == 7) return -1;
 	rsp |= 0x20;
 	if (rsp == 'c') return -1;
@@ -699,7 +952,7 @@ int slrn_post_file (char *file, char *to)
 int slrn_post (char *newsgroup, char *subj, char *dist)
 {
    FILE *fp;
-   char file[256];
+   char file[SLRN_MAX_PATH_LEN];
    unsigned int header_lines;
    int ret;
    
@@ -738,7 +991,7 @@ int slrn_post (char *newsgroup, char *subj, char *dist)
    
    if (slrn_edit_file (Slrn_Editor_Post, file, header_lines) >= 0)
      {
-	ret = slrn_post_file (file, NULL);
+	ret = slrn_post_file (file, NULL, 0);
      }
    else ret = -1;
    
@@ -763,7 +1016,11 @@ int slrn_add_custom_headers (FILE *fp, char *headers, int (*write_fun)(char *, F
    last_ch = 0;
    while ((ch = *s1) != 0)
      {
-	if (ch == '\n') n++;
+	if (ch == '\n') 
+	  {
+	     n++;
+	     while (s1[1] == '\n') s1++;   /* skip multiple newlines */
+	  }
 	last_ch = ch;
 	s1++;
      }
@@ -781,7 +1038,102 @@ int slrn_add_custom_headers (FILE *fp, char *headers, int (*write_fun)(char *, F
    return n;
 }
 
+static int get_postpone_dir (char *dirbuf)
+{
+   char *dir;
+   
+   if (Slrn_Postpone_Dir != NULL)
+     dir = Slrn_Postpone_Dir;
+   else 
+     dir = Slrn_Save_Directory;
+   
+   if ((dir == NULL)
+       || (-1 == slrn_make_home_dirname (dir, dirbuf))
+       || (2 != slrn_file_exists (dirbuf)))
+     {
+	slrn_error_now (2, "postpone_directory not specified or does not exist");
+	return -1;
+     }
+
+   return 0;
+}
+
+	     
+static int postpone_file (char *file)
+{
+#ifdef VMS
+   slrn_error ("Not implemented yet. Sorry");
+   return -1;
+#else
+   char dir [SLRN_MAX_PATH_LEN];
+   char dirfile [SLRN_MAX_PATH_LEN + 256];
+
+   if (-1 == get_postpone_dir (dir))
+     return -1;
+   
+   while (1)
+     {
+	int status;
+
+	if (-1 == slrn_dircat (dir, NULL, dirfile))
+	  return -1;
+
+	if (-1 == slrn_read_input ("Save to", NULL, dirfile, 1, 1))
+	  return -1;
+	
+	status = slrn_file_exists (dirfile);
+	if (status == 1)
+	  {
+	     status = slrn_get_yesno_cancel ("File exists, overwrite");
+	     if (status == 0)
+	       continue;
+	     if (status == 1)
+	       break;
+
+	     return -1;
+	  }
+	
+	if (status == 0)
+	  break;
+	if (status == 2)
+	  slrn_error_now (2, "You must specify a name");
+	else
+	  slrn_error_now (2, "Illegal filename. Try again");
+     }
+   
+   if (-1 == slrn_move_file (file, dirfile))
+     return -1;
+   
+   return 0;
+#endif
+}
+
+void slrn_post_postponed (void)
+{
+   char dir [SLRN_MAX_PATH_LEN];
+   char *file;
+
+   if (-1 == get_postpone_dir (dir))
+     return;
+   
+   file = slrn_browse_dir (dir);
+   if (file == NULL)
+     return;
+   
+   if (1 != slrn_file_exists (file))
+     {
+	slrn_error ("%s: not a regular file");
+	return;
+     }
+   
+   if (0 == slrn_post_file (file, NULL, 1))
+     slrn_delete_file (file);
+
+   slrn_free (file);
+}
+
    
 
-
-
+	     
+	  
+	  
