@@ -1,5 +1,24 @@
-#include "config.h"
+/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
+ *
+ * This file is part of slrn.
+ *
+ * Slrn is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ * 
+ * Slrn is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with Slrn; see the file COPYING.  If not, write to the Free
+ * Software Foundation, 59 Temple Place - Suite 330, 
+ * Boston, MA  02111-1307, USA.
+ */
 
+#include "config.h"
 #ifndef SLRNPULL_CODE
 # include "slrnfeat.h"
 #endif
@@ -239,6 +258,16 @@ int slrn_score_header (Slrn_Header_Type *h, char *newsgroup)
 	     
 	     if (s == NULL)
 	       {
+		  if (srt->not_flag)
+		    {
+		       /* Match */
+		       if (or_type)
+			 break;
+		       
+		       goto next_srt;
+		    }
+		  
+		  /* Match failed */
 		  if (or_type) goto next_srt;
 		  break;
 	       }
@@ -452,39 +481,30 @@ int slrn_open_score (char *group_name)
    while (gsc != NULL)
      {
 	Group_Score_Name_Type *gsnt;
-	SLRegexp_Type *re;
-	unsigned int gst_not_flag;
-	
+	int match;
+
 	gsnt = &gsc->gsnt;
-	gst_not_flag = gsc->gst_not_flag;
-	
-	while (gsnt != NULL)
+	match = 0;
+	while ((gsnt != NULL) && (match == 0))
 	  {
-	     int match;
-	     
-	     re = &gsnt->group_regexp;
-	
+	     SLRegexp_Type *re = &gsnt->group_regexp;
+
 	     match = ((re->min_length <= n)
 		      && (NULL != SLang_regexp_match ((unsigned char *) group_name, n, re)));
-	     
-	     /* Note: the following could be replaced by simply
-	      *  match = (match ^ gst_not_flag);
-	      * however, I may have more uses for this flag in the future.
-	      */
-	     match = ((match && (gst_not_flag == 0))
-		      || ((match == 0) && gst_not_flag));
-	       
-	     if (match)
-	       {
-		  if (-1 == chain_group_regexp (gsc->pst, &generic))
-		    {
-		       free_group_chain ();
-		       return -1;
-		    }
-		  break;
-	       }
 	     gsnt = gsnt->next;
 	  }
+
+	if (gsc->gst_not_flag)
+	  match = !match;
+
+	if (match)
+	  {
+	     if (-1 == chain_group_regexp (gsc->pst, &generic))
+	       {
+		  free_group_chain ();
+		  return -1;
+	       }
+	  }
 	gsc = gsc->next;
      }
    
@@ -583,6 +603,19 @@ static int compile_group_names (char *group, Group_Score_Type *gst)
 	if (comma != NULL) 
 	  {
 	     *comma++ = 0;
+	     
+	     while (1)
+	       {
+		  comma = slrn_skip_whitespace (comma);
+		  if (*comma == 0)
+		    {
+		       comma = NULL;
+		       break;
+		    }
+		  if (*comma != ',')
+		    break;
+		  comma++;
+	       }
 	  }
 	
 	(void) slrn_trim_string (group);
@@ -763,6 +796,7 @@ int slrn_read_score_file (char *file)
    unsigned char group[256];
    char line[1024];
    FILE *fp;
+   SLPreprocess_Type pt;
    int start_new_group = 1, not_flag, gnt_not_flag = 0;
    unsigned int linenum = 0;
    int score = 0;
@@ -789,11 +823,16 @@ int slrn_read_score_file (char *file)
    group[0] = '*';
    group[1] = 0;
    
+   (void) SLprep_open_prep (&pt);
    while (fgets (line, sizeof (line) - 1, fp))
      {
 	unsigned char *lp;
 	
 	linenum++;
+
+	if (0 == SLprep_line_ok (line, &pt))
+	  continue;
+
 	lp = (unsigned char *) slrn_skip_whitespace (line);
 	if ((*lp == '#') || (*lp == '%') || (*lp <= ' ')) continue;
 	
@@ -931,11 +970,13 @@ int slrn_read_score_file (char *file)
 	  }
      }
    
+   SLprep_close_prep (&pt);
    fclose (fp);
    Slrn_Apply_Score = 1;
    return 1;
-   
+
    error_return:
+   SLprep_close_prep (&pt);
    fclose (fp);
    return -1;
 }
