@@ -1,6 +1,23 @@
-/*  Copyright (c) 1996 John E. Davis (davis@space.mit.edu)
- *  All rights reserved.
+/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
+ *
+ * This file is part of slrn.
+ *
+ * Slrn is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ * 
+ * Slrn is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with Slrn; see the file COPYING.  If not, write to the Free
+ * Software Foundation, 59 Temple Place - Suite 330, 
+ * Boston, MA  02111-1307, USA.
  */
+
 #include "config.h"
 #include "slrnfeat.h"
 
@@ -167,6 +184,8 @@ static int inews_parse_args (char **argv, int argc)
 
 int slrn_init_objects (void)
 {
+   char *server;
+
 #if SLRN_HAS_NNTP_SUPPORT
    if (-1 == nntp_init_objects ())
      return -1;
@@ -183,6 +202,27 @@ int slrn_init_objects (void)
    if (-1 == pull_init_objects ())
      return -1;
 #endif
+   
+   switch (Slrn_Server_Id)
+     {
+      case SLRN_SERVER_ID_NNTP:
+	server = "NNTP";
+	break;
+	
+      case SLRN_SERVER_ID_SPOOL:
+	server = "SPOOL";
+	break;
+	
+      default:
+	server = NULL;
+     }
+   
+   if ((server != NULL) 
+       && (-1 == SLdefine_for_ifdef (server)))
+     {
+	slrn_exit_error ("Unable to add preprocessor name %s.", server);
+     }
+   
    return 0;
 }
 
@@ -193,6 +233,8 @@ int slrn_init_objects (void)
 
 int slrn_select_post_object (int id)
 {
+   char *post;
+
    switch (id)
      {
 #if SLRN_HAS_INEWS_SUPPORT
@@ -221,7 +263,11 @@ int slrn_select_post_object (int id)
 #endif
 
       default:
-	slrn_error ("Object %d is not a supported posting agent.", id);
+	post = slrn_map_object_id_to_name (1, id);
+	if (post == NULL)
+	  post = "UNKNOWN";
+
+	slrn_error ("%s is not a supported posting agent.", post);
      }
    
    return -1;
@@ -229,8 +275,11 @@ int slrn_select_post_object (int id)
 
 int slrn_select_server_object (int id)
 {
+#if SLRN_HAS_SPOOL_SUPPORT
    int ret;
-   
+#endif
+   char *server;
+
    switch (id)
      {
 #if SLRN_HAS_NNTP_SUPPORT
@@ -249,11 +298,16 @@ int slrn_select_server_object (int id)
 	if (Slrn_Use_Pull_Post) 
 	  Slrn_Post_Id = SLRN_POST_ID_PULL;
 # endif
-#endif
 	return ret;
+#endif
    
       default:
-	slrn_error ("server(%d) is not a supported server object.", id);
+	server = slrn_map_object_id_to_name (0, id);
+
+	if (server == NULL)
+	  server = "UNKNOWN";
+
+	slrn_error ("%s is not a supported server object.", server);
      }
    return -1;
 }
@@ -316,9 +370,98 @@ int slrn_parse_object_args (char *name, char **argv, int argc)
    return num_parsed;
 }
 
+typedef struct
+{
+   char *name;
+   int id;
+}
+Object_Name_Id_Type;
+
+static Object_Name_Id_Type Server_Objects_and_Ids [] = 
+{
+   {"nntp", SLRN_SERVER_ID_NNTP},
+   {"spool", SLRN_SERVER_ID_SPOOL},
+   {NULL}
+};
+
+static Object_Name_Id_Type Post_Objects_and_Ids [] = 
+{
+   {"nntp", SLRN_POST_ID_NNTP},
+   {"inews", SLRN_POST_ID_INEWS},
+   {"slrnpull", SLRN_POST_ID_PULL},
+   {NULL}
+};
+
+/* These functions should not attempt to write any error messages */
+static Object_Name_Id_Type *map_object_type (int type, int *def_id)
+{
+   Object_Name_Id_Type *obj;
+   
+   switch (type)
+     {
+      case 0:
+	obj = Server_Objects_and_Ids;
+	if (def_id != NULL)
+	  *def_id = SLRN_DEFAULT_SERVER_OBJ;
+	break;
+	
+      case 1:
+	obj = Post_Objects_and_Ids;
+	if (def_id != NULL)
+	  *def_id = SLRN_DEFAULT_POST_OBJ;
+	break;
+	
+      default:
+	obj = NULL;
+     }
+   
+   return obj;
+}
+
+char *slrn_map_object_id_to_name (int type, int id)
+{
+   Object_Name_Id_Type *obj;
+   
+   if (NULL == (obj = map_object_type (type, NULL)))
+     return NULL;
+   
+   while (obj->name != NULL)
+     {
+	if (obj->id == id)
+	  return obj->name;
+	obj++;
+     }
+   
+   return NULL;
+}
+
+int slrn_map_name_to_object_id (int type, char *name)
+{
+   Object_Name_Id_Type *obj;
+   int default_object_id;
+
+   if (NULL == (obj = map_object_type (type, &default_object_id)))
+     return -1;
+
+   if (name == NULL)
+     {
+	name = slrn_map_object_id_to_name (type, default_object_id);
+	if (name == NULL)
+	  return -1;
+     }
+   
+   while (obj->name != NULL)
+     {
+	if (0 == strcmp (name, obj->name))
+	  return obj->id;
+	obj++;
+     }
+   return -1;
+}
+
 #if SLRN_HAS_PULL_SUPPORT
 static Slrn_Post_Obj_Type Pull_Post_Obj;
-FILE *Pull_Fp;
+static FILE *Pull_Fp;
 unsigned int Pull_Num_Posted;
 char Pull_Post_Filename [SLRN_MAX_PATH_LEN + 1];
 char Pull_Post_Dir [SLRN_MAX_PATH_LEN + 1];
@@ -388,7 +531,7 @@ static int pull_end_post (void)
    
    if (Pull_Fp == NULL)
      return ERR_FAULT;
-   
+
    if (-1 == slrn_fclose (Pull_Fp))
      {
 	Pull_Fp = NULL;
@@ -396,6 +539,22 @@ static int pull_end_post (void)
      }
    Pull_Fp = NULL;
    
+
+#if SLRNPULL_USE_SETGID_POSTS
+    /*
+     * 1998/07/07 Sylvain Robitaille (syl@alcor.concordia.ca)
+     *            We have our out.going directory setgid to news, and we
+     *            want to be sure the file is readable and writable to
+     *            group news. This way, the news user will have
+     *            appropriate permissions for manipulating this file.
+     *            (NOTE: This has only been tested on Linux-2.0.xx)
+     */
+    if (-1 == chmod (Pull_Post_Filename, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP))
+      {
+	 return ERR_FAULT;
+      }
+#endif   
+ 
    while (1)
      {
 	if (-1 == pull_make_tempname (file, "X"))
