@@ -1,7 +1,24 @@
 /* -*- mode: C; mode: fold -*- */
-/*  Copyright (c) 1995, 1996 John E. Davis (davis@space.mit.edu)
- *  All rights reserved.
+/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
+ *
+ * This file is part of slrn.
+ *
+ * Slrn is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ * 
+ * Slrn is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with Slrn; see the file COPYING.  If not, write to the Free
+ * Software Foundation, 59 Temple Place - Suite 330, 
+ * Boston, MA  02111-1307, USA.
  */
+
 #include "config.h"
 #include "slrnfeat.h"
 /*{{{ Include files */
@@ -22,7 +39,7 @@
 #ifdef VMS
 # include "vms.h"
 #else
-# ifndef sun
+# if !defined(sun) && !defined(IBMPC_SYSTEM)
 #  include <sys/ioctl.h>
 # endif
 # ifdef HAVE_TERMIOS_H
@@ -40,6 +57,10 @@
 # endif
 #endif /* !VMS */
 
+#ifdef HAVE_SYS_WAIT_H
+# include <sys/wait.h>
+#endif
+
 #include <slang.h>
 #include "jdmacros.h"
 
@@ -56,6 +77,10 @@
 #include "util.h"
 #include "chmap.h"
 
+#if SLRN_USE_SLTCP
+# include "sltcp.h"
+#endif
+
 #if SLRN_HAS_GROUPLENS
 # include "grplens.h"
 #endif
@@ -67,6 +92,9 @@
 # include <os2.h>
 #endif
 
+#if defined(__NT__) || defined(__WIN32__)
+# include <windows.h>
+#endif
 
 /*}}}*/
 
@@ -78,7 +106,6 @@ int Slrn_TT_Initialized = 0;
  * do not use it.
  */
 int Slrn_Use_Mouse;
-char *Slrn_Version = SLRN_VERSION;
 
 int Slrn_Batch;
 int Slrn_Suspension_Ok;
@@ -87,22 +114,8 @@ char *Slrn_Newsrc_File = NULL;
 char *Slrn_Lib_Dir = NULL;
 Slrn_Mode_Type *Slrn_Current_Mode;
 
-#if !defined (SLRN_DEFAULT_SERVER_OBJ)
-# if SLRN_HAS_NNTP_SUPPORT
-#  define SLRN_DEFAULT_SERVER_OBJ SLRN_SERVER_ID_NNTP
-# else
-#  define SLRN_DEFAULT_SERVER_OBJ SLRN_SERVER_ID_SPOOL
-# endif
-#endif
-
-#if !defined (SLRN_DEFAULT_POST_OBJ)
-# if SLRN_HAS_NNTP_SUPPORT
-#  define SLRN_DEFAULT_POST_OBJ SLRN_POST_ID_NNTP
-# else
-#  define SLRN_DEFAULT_POST_OBJ SLRN_POST_ID_INEWS
-# endif
-#endif
-
+int Slrn_Default_Server_Obj = SLRN_DEFAULT_SERVER_OBJ;
+int Slrn_Default_Post_Obj = SLRN_DEFAULT_POST_OBJ;
 
 /*}}}*/
 /*{{{ Static Variables */
@@ -111,14 +124,16 @@ static int Can_Suspend;
 static volatile int Want_Suspension;
 static volatile int Want_Window_Size_Change;
 static void perform_suspend (int);
+static int Current_Mouse_Mode;
 
 /*}}}*/
 /*{{{ Static Function Declarations */
 
-static int main_loop (void);
 static int suspend_display_mode (int);
-static int resume_display_mode (int, int);
+static int resume_display_mode (int, int, int);
 static void init_suspend_signals (int);
+static void slrn_hangup (int);
+static void run_winch_functions (int, int);
 /*}}}*/
 
 /*{{{ Newsrc Locking Routines */
@@ -133,12 +148,12 @@ static void test_lock( char *file ) /*{{{*/
 	if (1 == fscanf (fp, "%d", &pid) )
 	  {
 	     if ((pid > 0)
-#ifndef __os2__
+#if !defined(IBMPC_SYSTEM)
 		 && (0 == kill (pid, 0))
 #endif
 		 )
 	       {
-#ifdef __os2__
+#if defined(IBMPC_SYSTEM)
 		  slrn_exit_error ("\
 slrn: pid %d is locking the newsrc file. If you're not running another\n\
       copy of slrn, delete the file %s", 
@@ -168,21 +183,24 @@ static int make_lock( char *file ) /*{{{*/
    if (fp == NULL) return -1;
    
    pid = getpid ();
-   fprintf (fp, "%d", pid);
-   slrn_fclose (fp);
-   return 0;
+   if (EOF == fprintf (fp, "%d", pid))
+     {
+	slrn_fclose (fp);
+	return -1;
+     }
+   return slrn_fclose (fp);
 }
 
 /*}}}*/
 
 static void lock_file (int how) /*{{{*/
 {
-   char file[1024];
+   char file[SLRN_MAX_PATH_LEN];
    char name[256];
    static int not_ok_to_unlock;
 #if SLRN_HAS_RNLOCK
    int rnlock = 0;
-   char file_rn[1024];
+   char file_rn[SLRN_MAX_PATH_LEN];
 #endif
    
    if (Slrn_Newsrc_File == NULL) return;
@@ -190,12 +208,12 @@ static void lock_file (int how) /*{{{*/
    
    not_ok_to_unlock = 1;
 
-#ifndef __os2__
-   sprintf (name, "%s-lock", Slrn_Newsrc_File);
+#ifdef SLRN_USE_OS2_FAT
+   slrn_os2_make_fat (name, Slrn_Newsrc_File, ".lck");
 #else
-   sprintf (name, "lk-%s", Slrn_Newsrc_File);
+   sprintf (name, "%s-lock", Slrn_Newsrc_File);
 #endif
-     
+
    slrn_make_home_filename (name, file);
    
 #if SLRN_HAS_RNLOCK
@@ -393,6 +411,8 @@ static void perform_suspend (int smg_suspend_flag) /*{{{*/
 #else
    
    int init;
+   int mouse_mode = Current_Mouse_Mode;
+
 # ifdef SLRN_POSIX_SIGNALS
    sigset_t mask;
 
@@ -426,10 +446,8 @@ static void perform_suspend (int smg_suspend_flag) /*{{{*/
    kill(getpid(),SIGSTOP);
 # endif
    
-   resume_display_mode (smg_suspend_flag, init);
-
-   SLtty_set_suspend_state (1);
-#endif
+   resume_display_mode (smg_suspend_flag, init, mouse_mode);
+#endif				       /* !defined(SIGSTOP) || !defined(REAL_UNIX_SYSTEM) */
 }
 
 /*}}}*/
@@ -447,8 +465,12 @@ static void check_for_suspension (void)
    f = SLsignal (SIGTSTP, SIG_DFL);
    (void) SLsignal (SIGTSTP, f);
    
+#if 0
    Can_Suspend = (f == SIG_DFL);
-     
+#else
+   Can_Suspend = (f != SIG_IGN);
+#endif
+   
 #else
    Can_Suspend = 0;
 #endif
@@ -458,20 +480,6 @@ static void check_for_suspension (void)
 
 /*{{{ Hangup Signals */
 
-static void slrn_hangup (int sig) /*{{{*/
-{
-   slrn_init_hangup_signals (0);
-   
-   if ((Slrn_Current_Mode != NULL)
-       && (Slrn_Current_Mode->hangup_fun != NULL))
-     (*Slrn_Current_Mode->hangup_fun) (sig);
-   else
-     slrn_write_newsrc ();
-   
-   slrn_quit (sig);
-}
-
-/*}}}*/
 
 void slrn_init_hangup_signals (int state) /*{{{*/
 {
@@ -505,99 +513,18 @@ static void sig_winch_handler (int sig)
 
 static void slrn_set_screen_size (int sig) /*{{{*/
 {
-   int r = 0, c = 0;
    int old_r, old_c;
-#ifdef VMS
-   int status, code;
-   unsigned short chan;
-   $DESCRIPTOR(dev_dsc, "SYS$INPUT:");
-#endif
-#ifdef __os2__
-   VIOMODEINFO vioModeInfo;
-#endif
-   
-#ifdef TIOCGWINSZ
-   struct winsize wind_struct;
-
-   if (Slrn_Batch)
-     return;
-
-   do
-     {
-	if ((ioctl(1,TIOCGWINSZ,&wind_struct) == 0)
-	    || (ioctl(0, TIOCGWINSZ, &wind_struct) == 0)
-	    || (ioctl(2, TIOCGWINSZ, &wind_struct) == 0))
-	  {
-	     c = (int) wind_struct.ws_col;
-	     r = (int) wind_struct.ws_row;
-	     break;
-	  }
-     }
-   while (errno == EINTR);
-   
-#else
-# ifdef VMS
-   status = sys$assign(&dev_dsc,&chan,0,0,0);
-   if (status & 1)
-     {
-	code = DVI$_DEVBUFSIZ;
-	status = lib$getdvi(&code, &chan,0, &c, 0,0);
-	if (!(status & 1))
-	  c = 80;
-	code = DVI$_TT_PAGE;
-	status = lib$getdvi(&code, &chan,0, &r, 0,0);
-	if (!(status & 1))
-	  r = 24;
-	sys$dassgn(chan);
-     }
-# else
-#  ifdef __os2__
-   vioModeInfo.cb = sizeof(vioModeInfo);
-   VioGetMode (&vioModeInfo, 0);
-   c = vioModeInfo.col;
-   r = vioModeInfo.row;   
-#  endif
-# endif
-#endif
-   
-   if (r <= 0)
-     {
-	char *s = getenv ("LINES");
-	if (s != NULL) r = atoi (s);
-     }
-   
-   if (c <= 0)
-     {
-	char *s = getenv ("COLUMNS");
-	if (s != NULL) c = atoi (s);
-     }
-   
-   if ((r <= 0) || (r > 200)) r = 24;
-   if ((c <= 0) || (c > 250)) c = 80;
    
    old_r = SLtt_Screen_Rows;
    old_c = SLtt_Screen_Cols;
    
-   SLtt_Screen_Rows = r;
-   SLtt_Screen_Cols = c;
-   
-   if (SLtt_Screen_Rows > 3)
-     Slrn_Group_Window.nrows = SLtt_Screen_Rows - 3;
-   else
-     Slrn_Group_Window.nrows = 1;
+   SLtt_get_screen_size ();
 
+   Slrn_Full_Screen_Update = 1;
    Want_Window_Size_Change = 0;
-   
-   if ((Slrn_Current_Mode != NULL)
-       && (Slrn_Current_Mode->sigwinch_fun != NULL))
-     {
-	(*Slrn_Current_Mode->sigwinch_fun) (old_r, old_c);
-#if SLRN_HAS_SLANG
-	if (SLang_Error == 0)
-	  SLang_run_hooks ("resize_screen_hook", NULL, NULL);
-#endif
-     }
-   
+
+   run_winch_functions (old_r, old_c);
+
    if (sig)
      {
 	SLsmg_reset_smg ();
@@ -672,6 +599,11 @@ static int init_tty (void) /*{{{*/
 #ifdef REAL_UNIX_SYSTEM
    SLang_getkey_intr_hook = slrn_handle_interrupts;
 #endif
+
+#ifdef REAL_UNIX_SYSTEM
+   if (Can_Suspend)
+     SLtty_set_suspend_state (1);
+#endif
    
    Slrn_TT_Initialized |= SLRN_TTY_INIT;
    
@@ -698,12 +630,12 @@ static int reset_tty (void) /*{{{*/
 
 /*}}}*/
 
-static int init_smg (int use_resume) /*{{{*/
+static int init_smg (int use_resume, int mouse_mode) /*{{{*/
 {
    if (Slrn_TT_Initialized & SLRN_SMG_INIT)
      return 0;
 
-   slrn_enable_mouse (1);
+   slrn_enable_mouse (mouse_mode);
 
    if (Slrn_TT_Initialized == 0)
      init_display_signals (1);
@@ -720,7 +652,7 @@ static int init_smg (int use_resume) /*{{{*/
 	Slrn_TT_Initialized |= SLRN_SMG_INIT;
 	     
 	/* We do not want the -> overlay cursor to affect the scroll. */
-#ifndef __os2__
+#if !defined(IBMPC_SYSTEM)
 	SLsmg_Scroll_Hash_Border = 5;
 #endif
 	slrn_redraw ();
@@ -775,7 +707,7 @@ static int suspend_display_mode (int smg_suspend_flag) /*{{{*/
 
 /*}}}*/
 
-static int resume_display_mode (int smg_suspend_flag, int mode) /*{{{*/
+static int resume_display_mode (int smg_suspend_flag, int mode, int mouse_mode) /*{{{*/
 {
    SLsig_block_signals ();
    
@@ -783,7 +715,7 @@ static int resume_display_mode (int smg_suspend_flag, int mode) /*{{{*/
      init_tty ();
    
    if (mode & SLRN_SMG_INIT)
-     init_smg (smg_suspend_flag);
+     init_smg (smg_suspend_flag, mouse_mode);
    
    SLsig_unblock_signals ();
    return 0;
@@ -803,7 +735,7 @@ void slrn_set_display_state (int state) /*{{{*/
      reset_tty ();
    
    if (state & SLRN_SMG_INIT)
-     init_smg (0);
+     init_smg (0, 1);
    else
      reset_smg (0);
    
@@ -814,13 +746,15 @@ void slrn_set_display_state (int state) /*{{{*/
 
 void slrn_enable_mouse (int mode) /*{{{*/
 {
-#ifndef __os2__
+   if (Current_Mouse_Mode == mode)
+     return;
+
    if (Slrn_Use_Mouse)
      {
 	if (-1 == SLtt_set_mouse_mode (mode, (Slrn_Use_Mouse < 0)))
 	  Slrn_Use_Mouse = 0;
      }
-#endif
+   Current_Mouse_Mode = mode;
 }
 
 /*}}}*/
@@ -858,7 +792,7 @@ int slrn_get_new_news (int no_new_groups, int create_flag) /*{{{*/
 
 /*}}}*/
 
-void slrn_quit (int retcode) /*{{{*/
+static void perform_cleanup (void)
 {
    if (Slrn_Server_Obj != NULL) 
      Slrn_Server_Obj->sv_close ();
@@ -869,16 +803,6 @@ void slrn_quit (int retcode) /*{{{*/
 #if SLRN_HAS_GROUPLENS
    slrn_close_grouplens ();
 #endif
-   if (retcode) fprintf (stderr, "slrn: quiting on signal %d.\n", retcode);
-   exit (retcode);
-}
-
-/*}}}*/
-
-static void perform_cleanup (void)
-{
-   if (Slrn_Groups_Dirty)
-     slrn_write_newsrc ();
       
    if (Slrn_Server_Obj != NULL)
      Slrn_Server_Obj->sv_close ();
@@ -887,12 +811,27 @@ static void perform_cleanup (void)
    slrn_close_grouplens ();
 #endif
 
+#if SLRN_HAS_SLANG
+   (void) slrn_reset_slang ();
+#endif
+
+#if SLRN_USE_SLTCP
+   (void) sltcp_close_sltcp ();
+#endif
    lock_file (0);
 }
 
-void slrn_exit_error (char *fmt, ...) /*{{{*/
+void slrn_quit (int retcode) /*{{{*/
+{
+   perform_cleanup ();
+   if (retcode) fprintf (stderr, "slrn: quiting on signal %d.\n", retcode);
+   exit (retcode);
+}
+
+/*}}}*/
+
+void slrn_va_exit_error (char *fmt, va_list ap)
 {
-   va_list ap;
    static int trying_to_exit;
 
    if (trying_to_exit == 0)
@@ -903,11 +842,9 @@ void slrn_exit_error (char *fmt, ...) /*{{{*/
 	if (fmt != NULL)
 	  {
 	     fprintf (stderr, "slrn fatal error:\n");
-	     
-	     va_start (ap, fmt);
 	     vfprintf (stderr, fmt, ap);
-	     va_end(ap);
 	  }   
+	if (Slrn_Groups_Dirty) slrn_write_newsrc ();
 	perform_cleanup ();
      }
 
@@ -915,6 +852,15 @@ void slrn_exit_error (char *fmt, ...) /*{{{*/
    exit (1);
 }
 
+void slrn_exit_error (char *fmt, ...) /*{{{*/
+{
+   va_list ap;
+   
+   va_start (ap, fmt);
+   slrn_va_exit_error (fmt, ap);
+   va_end(ap);
+}
+
 /*}}}*/
 
 static void usage (char *extra) /*{{{*/
@@ -938,7 +884,7 @@ Usage: slrn [--inews ...] [--nntp ...] [--spool ...] OPTIONS\n\
 --help          Print this usage.\n\
 --version       Show version and supported features\n\
 \n\
-For additional info use one of forms:\n\
+For additional info use one of the forms:\n\
     slrn --inews --help\n\
     slrn --nntp --help\n\
     slrn --spool --help\n\
@@ -979,15 +925,23 @@ static void version (void) /*{{{*/
 # ifdef __os2__
    os = "OS/2";
 # else
-#  ifdef __unix__
-   os = "Unix";
+#  ifdef AMIGA
+   os = "Amiga";
 #  else
+#   ifdef __unix__
+   os = "Unix";
+#   else
+#    if defined(__NT__) || defined(__WIN32__)
+   os = "Win32";
+#    else
    os = "Unknown";
+#    endif
+#   endif
 #  endif
 # endif
 #endif
-     
-#if defined(__DATE__) && defined(__TIME__)
+   
+# if defined(__DATE__) && defined(__TIME__)
    fprintf (stdout, "Slrn Version: %s (%s %s)\n", Slrn_Version, __DATE__, __TIME__);
 #else
    fprintf (stdout, "Slrn Version: %s\n", Slrn_Version);
@@ -998,7 +952,7 @@ static void version (void) /*{{{*/
 	fprintf (stdout, "\t** Note: This program was compiled against version %s.\n",
 		 make_slang_version (SLANG_VERSION));
      }
-   fprintf (stdout, "OS Type: %s\n", os);
+   fprintf (stdout, "OS-Type: %s\n", os);
 
    fprintf (stdout, "\nslrn compiled with support for:");
 
@@ -1042,38 +996,16 @@ static void version (void) /*{{{*/
 #if SLRN_HAS_MSGID_CACHE
    fputs ("\n\tMsg-id Cache", stdout);
 #endif
-   fputs ("\n\nDefault server object: ", stdout);
-   switch (SLRN_DEFAULT_SERVER_OBJ)
-     {
-      case SLRN_SERVER_ID_NNTP:
-	fputs ("nntp", stdout);
-	break;
 
-      case SLRN_SERVER_ID_SPOOL:
-	fputs ("spool", stdout);
-	break;
-     }
+   fprintf (stdout, "\n\nDefault server object: %s\n",
+	    slrn_map_object_id_to_name (0, SLRN_DEFAULT_SERVER_OBJ));
    
-   fputs ("\nDefault posting mechanism: ", stdout);
-   switch (SLRN_DEFAULT_POST_OBJ)
-     {
-      case SLRN_POST_ID_NNTP:
-	fputs ("nntp", stdout);
-	break;
+   fprintf (stdout, "Default posting mechanism: %s\n",
+	    slrn_map_object_id_to_name (1, SLRN_DEFAULT_POST_OBJ));
 
-      case SLRN_POST_ID_INEWS:
-	fputs ("inews", stdout);
-#if SLRN_FORCE_INEWS
-	fputs (" (fixed)", stdout);
+#if SLRN_HAS_CHARACTER_MAP
+   slrn_chmap_show_supported ();
 #endif
-	break;
-
-      case SLRN_POST_ID_PULL:
-	fputs ("slrnpull", stdout);
-	break;
-     }
-   
-   fputc ('\n', stdout);
    exit (0);
 }
 
@@ -1086,12 +1018,7 @@ static int parse_object_args (char *obj, char **argv, int argc) /*{{{*/
    if (obj == NULL)
      {
 	zero_ok = 0;
-#if SLRN_DEFAULT_SERVER_OBJ == SLRN_SERVER_ID_SPOOL
-	obj = "spool";
-#endif
-#if SLRN_DEFAULT_SERVER_OBJ == SLRN_SERVER_ID_NNTP
-	obj = "nntp";
-#endif
+	obj = slrn_map_object_id_to_name (0, SLRN_DEFAULT_SERVER_OBJ);
      }
    
    num_parsed = slrn_parse_object_args (obj, argv, argc);
@@ -1126,10 +1053,10 @@ static void read_score_file (void)
 }
 
 
-int main (int argc, char **argv) /*{{{*/
+static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 {
    char *hlp_file;
-   int i;
+   unsigned int i;
    int create_flag = 0;
    int no_new_groups = 0;
    int no_score_file = 0;
@@ -1138,7 +1065,7 @@ int main (int argc, char **argv) /*{{{*/
    int dsc_flag = 0;
    int use_active = 0;
    FILE *fp;
-   char file[512];
+   char file [SLRN_MAX_PATH_LEN];
    char *init_file = SLRN_USER_SLRNRC_FILENAME;
 
    check_for_suspension ();
@@ -1146,13 +1073,24 @@ int main (int argc, char **argv) /*{{{*/
    (void) umask (077);
 #endif
    
-#ifdef __os2__
+   /* These are already available if the interpreter is available */
+#if !SLRN_HAS_SLANG
+# ifdef __os2__
    SLdefine_for_ifdef ("OS2");
-#else
-# ifdef VMS
-   SLdefine_for_ifdef ("VMS");
 # else
+#  ifdef VMS
+   SLdefine_for_ifdef ("VMS");
+#  else
+#   ifdef __NT__
+   SLdefine_for_ifdef ("NT");
+#   else
+#    ifdef __WIN32__
+   SLdefine_for_ifdef ("WIN32");
+#    else
    SLdefine_for_ifdef ("UNIX");
+#    endif
+#   endif
+#  endif
 # endif
 #endif
 
@@ -1163,44 +1101,49 @@ int main (int argc, char **argv) /*{{{*/
      Slrn_Unsubscribe_New_Groups = 1;
 #endif
    
-   for (i = 1; i < argc; i++)
+   for (i = 1; i < (unsigned int) argc; i++)
      {
-	if (!strcmp ("--spool", argv[i])
-	    || !strcmp ("--nntp", argv[i])
-	    || !strcmp ("--inews", argv[i]))
+	char *argv_i = argv[i];
+	
+	if ((argv_i[0] == '-') && (argv_i[1] == '-'))
 	  {
-	     i += parse_object_args (argv[i] + 2, argv + (i + 1), argc - (i + 1));
-	  }
-	else if (!strcmp ("--help", argv[i])) usage (NULL);
+	     int status;
+
+	     argv_i += 2;
+	     
+	     status = slrn_parse_object_args (argv_i, NULL, 0);
+	     if (status != -1)
+	       i += parse_object_args (argv_i, argv + (i + 1), argc - (i + 1));
 #if 0
-	else if (!strcmp ("--batch", argv[i])) Slrn_Batch = 1;
+	     else if (!strcmp ("batch", argv_i)) Slrn_Batch = 1;
 #endif
-	else if (!strcmp ("-create", argv[i])) create_flag = 1;
-	else if (!strcmp ("-C", argv[i])) use_color = 1;
-	else if (!strcmp ("-C-", argv[i])) use_color = -1;
-	else if (!strcmp ("-a", argv[i])) use_active = 1;
-	else if (!strcmp ("-n", argv[i])) no_new_groups = 1;
-	else if (!strcmp ("-d", argv[i])) dsc_flag = 1;
-	else if (!strcmp ("-m", argv[i])) use_mouse = 1;
-	else if (!strcmp ("-k", argv[i])) no_score_file = 1;
-	else if (!strcmp ("-k0", argv[i]))
-	  Slrn_Perform_Scoring &= ~SLRN_EXPENSIVE_SCORING;
-	else if (!strcmp ("--version", argv[i]))
-	  {
-	     version ();
+	     else if (!strcmp ("create", argv_i)) create_flag = 1;
+	     else if (!strcmp ("version", argv_i))
+	       version ();
+	     else usage (NULL);
 	  }
-	else if (!strncmp ("-D", argv[i], 2) && (argv[i][2] != 0))
+	else if (!strcmp ("-create", argv_i)) create_flag = 1;
+	else if (!strcmp ("-C", argv_i)) use_color = 1;
+	else if (!strcmp ("-C-", argv_i)) use_color = -1;
+	else if (!strcmp ("-a", argv_i)) use_active = 1;
+	else if (!strcmp ("-n", argv_i)) no_new_groups = 1;
+	else if (!strcmp ("-d", argv_i)) dsc_flag = 1;
+	else if (!strcmp ("-m", argv_i)) use_mouse = 1;
+	else if (!strcmp ("-k", argv_i)) no_score_file = 1;
+	else if (!strcmp ("-k0", argv_i))
+	  Slrn_Perform_Scoring &= ~SLRN_EXPENSIVE_SCORING;
+	else if (!strncmp ("-D", argv_i, 2) && (argv_i[2] != 0))
 	  {
-	     if (SLdefine_for_ifdef (argv[i] + 2) == 0)
+	     if (-1 == SLdefine_for_ifdef (argv_i + 2))
 	       {
 		  slrn_exit_error ("Unable to add preprocessor name %s.",
-				   argv[i] + 2);
+				   argv_i + 2);
 	       }
 	  }
-	else if (i + 1 < argc)
+	else if (i + 1 < (unsigned int) argc)
 	  {
-	     if (!strcmp ("-f", argv[i])) Slrn_Newsrc_File = argv[++i];
-	     else if (!strcmp ("-i", argv[i])) init_file = argv[++i];
+	     if (!strcmp ("-f", argv_i)) Slrn_Newsrc_File = argv[++i];
+	     else if (!strcmp ("-i", argv_i)) init_file = argv[++i];
 	     else 
 	       {
 		  i += parse_object_args (NULL, argv + i, argc - i);
@@ -1215,15 +1158,6 @@ int main (int argc, char **argv) /*{{{*/
      }
    
 
-   if (-1 == slrn_init_objects ())
-     {
-	slrn_exit_error ("Error configuring server objects.");
-     }
-
-
-   if (Slrn_Server_Id == 0) Slrn_Server_Id = SLRN_DEFAULT_SERVER_OBJ;
-   if (Slrn_Post_Id == 0) Slrn_Post_Id = SLRN_DEFAULT_POST_OBJ;
-
 #if defined(__DATE__) && defined(__TIME__)
    fprintf (stdout, "slrn %s (%s %s)\n", Slrn_Version, __DATE__, __TIME__);
 #else
@@ -1242,9 +1176,32 @@ int main (int argc, char **argv) /*{{{*/
 	else if (use_color == -1) SLtt_Use_Ansi_Colors = 0;
      }
 
+#if SLRN_USE_SLTCP
+   /* This is necessary to ensure that gethostname works on Win32 */
+   if (-1 == sltcp_open_sltcp ())
+     slrn_exit_error ("Error initializing SLtcp interface");
+#endif
+
+#if SLRN_HAS_SLANG
+   if (-1 == slrn_init_slang ())
+     fprintf (stderr, "Error initializing S-Lang interpreter.\n");
+#endif
+
    slrn_startup_initialize ();
+
+   /* The next function call will also define slang preprocessing tokens 
+    * for the appropriate objects.  For that reason, it is called after 
+    * startup initialize.
+    */
+   if (-1 == slrn_init_objects ())
+     {
+	slrn_exit_error ("Error configuring server objects.");
+     }
+
    slrn_get_user_info ();
-   
+
+   slrn_init_hangup_signals (1);
+
    if (Slrn_Lib_Dir == NULL)
      {
 	Slrn_Lib_Dir = getenv ("SLRN_LIB_DIR");
@@ -1257,22 +1214,21 @@ int main (int argc, char **argv) /*{{{*/
    sprintf (file, "%s/%s", Slrn_Lib_Dir, "slrn.rc");
 #endif
 
-#if SLRN_HAS_SLANG
-   if (-1 == slrn_init_slang ())
-     {
-	fprintf (stderr, "Error initializing S-Lang interpreter.\n");
-     }
-#endif
-
-	
-   slrn_init_hangup_signals (1);
-
+   /* Make sure terminal is initialized before setting colors.  The 
+    * SLtt_get_terminfo call above fixed that.
+    */
    slrn_read_startup_file (file);      /* global file for all users */
    slrn_read_startup_file (init_file);
 
+   if (Slrn_Server_Id == 0) Slrn_Server_Id = Slrn_Default_Server_Obj;
+   if (Slrn_Post_Id == 0) Slrn_Post_Id = Slrn_Default_Post_Obj;
+
 #if SLRN_HAS_CHARACTER_MAP
    if (-1 == slrn_set_charset (Slrn_Charset))
-     slrn_exit_error ("Failed to select character set.");
+     {
+	slrn_chmap_show_supported ();
+	slrn_exit_error ("Failed to select character set.");
+     }
 #endif
    
 #ifdef SIGINT
@@ -1283,11 +1239,10 @@ int main (int argc, char **argv) /*{{{*/
    if ((-1 == slrn_select_server_object (Slrn_Server_Id))
        || (-1 == slrn_select_post_object (Slrn_Post_Id)))
      {
-	slrn_exit_error ("Unable to select object.");
+	slrn_exit_error ("Unable to select server/post object.");
      }
 
-   
-#ifndef __os2__
+#if !defined(IBMPC_SYSTEM)
    /* Allow blink characters if in mono */
    if (SLtt_Use_Ansi_Colors == 0)
      SLtt_Blink_Mode = 1;
@@ -1323,7 +1278,7 @@ cannot be constructed.  Try setting the USER environment variable.\r\n");
    
    if ((Slrn_Newsrc_File == NULL)
        && ((Slrn_Newsrc_File = slrn_map_file_to_host (Slrn_Server_Obj->sv_name)) == NULL))
-#if defined(VMS) || defined(__os2__)
+#if defined(VMS) || defined(IBMPC_SYSTEM)
      Slrn_Newsrc_File = "jnews.rc";
 #else
    Slrn_Newsrc_File = ".jnewsrc";
@@ -1340,7 +1295,7 @@ cannot be constructed.  Try setting the USER environment variable.\r\n");
 	  }
 	slrn_get_group_descriptions ();
 	Slrn_Server_Obj->sv_close ();
-	return 0;
+	exit (0);
      }
    
    if (create_flag == 0)
@@ -1359,19 +1314,22 @@ If you want to create %s, add command line options:\r\n\
 	       }
 	  }
 	slrn_fclose (fp);
-	
-	lock_file (1);
      }
+   
+   lock_file (1);
 
 #if SLRN_HAS_SLANG
-   (void) SLang_run_hooks ("startup_hook", NULL, NULL);
+   (void) SLang_run_hooks ("startup_hook", 0);
 #endif
    
    
 #if SLRN_HAS_GROUPLENS
+   if (Slrn_Server_Id != SLRN_SERVER_ID_NNTP)
+     Slrn_Use_Group_Lens = 0;
+
    if (Slrn_Use_Group_Lens && (Slrn_Batch == 0))
      {
-	fprintf (stdout, "Initializing GroupLens\n");
+	slrn_message ("Initializing GroupLens");
 	if (-1 == slrn_init_grouplens ())
 	  {
 	     fprintf (stderr, "GroupLens disabled.\n");
@@ -1389,28 +1347,118 @@ If you want to create %s, add command line options:\r\n\
    
    slrn_set_display_state (SLRN_SMG_INIT | SLRN_TTY_INIT);
 
-#if defined(__unix__) && !defined(__os2__)
+#if defined(__unix__) && !defined(IBMPC_SYSTEM)
    if (Slrn_Autobaud) SLtt_Baud_Rate = SLang_TT_Baud_Rate;
 #endif
 
-   main_loop ();
-   
-   slrn_quit (0);
    return 0;
 }
 
 /*}}}*/
 
-/*{{{ Main Loop and Key Processing Functions */
 
-void slrn_switch_to_mode (Slrn_Mode_Type *mode)
+
+/*{{{ Main Loop and Key Processing Functions */
+#define MAX_MODE_STACK_LEN 15
+/* Allow one extra because the top one is the current mode */
+static Slrn_Mode_Type *Mode_Stack [MAX_MODE_STACK_LEN + 1];
+static unsigned int Mode_Stack_Depth;
+
+void slrn_push_mode (Slrn_Mode_Type *mode)
 {
-   Slrn_Current_Mode = mode;
+   if (Mode_Stack_Depth == MAX_MODE_STACK_LEN)
+     slrn_exit_error ("Internal Error: Mode_Stack overflow");
+
+   Mode_Stack[Mode_Stack_Depth] = Slrn_Current_Mode;
+   Mode_Stack_Depth++;
+
+   Mode_Stack[Mode_Stack_Depth] = Slrn_Current_Mode = mode;
+   
    Slrn_Full_Screen_Update = 1;
    if ((mode != NULL) && (mode->enter_mode_hook != NULL))
      (*mode->enter_mode_hook) ();
 }
 
+void slrn_pop_mode (void)
+{
+   if (Mode_Stack_Depth == 0)
+     slrn_exit_error ("Internal Error: Mode_Stack underflow");
+   
+   Mode_Stack [Mode_Stack_Depth] = NULL;   /* null current mode */
+   Mode_Stack_Depth--;
+   Slrn_Current_Mode = Mode_Stack[Mode_Stack_Depth];
+
+   Slrn_Full_Screen_Update = 1;
+   if ((Slrn_Current_Mode != NULL) && (Slrn_Current_Mode->enter_mode_hook != NULL))
+     (*Slrn_Current_Mode->enter_mode_hook) ();
+}
+
+
+void slrn_update_screen (void)
+{
+   Slrn_Mode_Type *mode;
+   unsigned int i, imax;
+
+   if (Slrn_Batch) return;
+   
+   slrn_push_suspension (0);
+   imax = Mode_Stack_Depth + 1;	       /* include current mode */
+   for (i = 0; i < imax; i++)
+     {
+	Slrn_Full_Screen_Update = 1;
+	mode = Mode_Stack [i];
+	if ((mode != NULL)
+	    && (mode->redraw_fun != NULL))
+	  (*mode->redraw_fun) ();
+     }
+   slrn_smg_refresh ();
+   slrn_pop_suspension ();
+}
+
+static void run_winch_functions (int old_r, int old_c)
+{
+   Slrn_Mode_Type *mode;
+   unsigned int i, imax;
+
+   imax = Mode_Stack_Depth + 1;	       /* include current */
+   for (i = 0; i < imax; i++)
+     {
+	mode = Mode_Stack [i];
+	if ((mode != NULL)
+	    && (mode->sigwinch_fun != NULL))
+	  (*mode->sigwinch_fun) (old_r, old_c);
+     }
+
+#if SLRN_HAS_SLANG
+   if (SLang_Error == 0)
+     SLang_run_hooks ("resize_screen_hook", 0);
+#endif
+}
+
+
+static void slrn_hangup (int sig) /*{{{*/
+{
+   Slrn_Mode_Type *mode;
+   unsigned int i;
+
+   slrn_init_hangup_signals (0);
+   
+   i = Mode_Stack_Depth + 1;	       /* include current */
+   while (i != 0)
+     {
+	i--;
+	mode = Mode_Stack [i];
+	if ((mode != NULL)
+	    && (mode->hangup_fun != NULL))
+	  (*mode->hangup_fun) (sig);
+     }
+   
+   slrn_write_newsrc ();
+   slrn_quit (sig);
+}
+
+/*}}}*/
+
 void slrn_call_command (char *cmd) /*{{{*/
 {
    SLKeymap_Function_Type *list;
@@ -1490,7 +1538,7 @@ static int slrn_getkey (void)
 }
 
    
-static void do_key (SLKeyMap_List_Type *map) /*{{{*/
+void slrn_do_keymap_key (SLKeyMap_List_Type *map) /*{{{*/
 {
    SLang_Key_Type *key;
    static SLKeyMap_List_Type *last_map;
@@ -1562,7 +1610,7 @@ void slrn_digit_arg (void) /*{{{*/
 {
    char buf[20];
    unsigned char key;
-   int i;
+   unsigned int i;
    
    i = 0;
    buf[i++] = (char) SLang_Last_Key_Char;
@@ -1574,7 +1622,11 @@ void slrn_digit_arg (void) /*{{{*/
 	buf[i] = 0;
 	key = (unsigned char) slrn_getkey ();
 	if ((key < '0') || (key > '9')) break;
-	buf[i++] = (char) key;
+	if (i + 1 < sizeof (buf))
+	  {
+	     buf[i] = (char) key;
+	     i++;
+	  }
      }
 
    SLang_Key_TimeOut_Flag = 0;
@@ -1583,7 +1635,7 @@ void slrn_digit_arg (void) /*{{{*/
    SLang_ungetkey (key);
    if ((Slrn_Current_Mode != NULL) 
        && (Slrn_Current_Mode->keymap != NULL))
-     do_key (Slrn_Current_Mode->keymap);
+     slrn_do_keymap_key (Slrn_Current_Mode->keymap);
    
    Slrn_Prefix_Arg_Ptr = NULL;
 }
@@ -1597,22 +1649,21 @@ void slrn_repeat_last_key (void) /*{{{*/
 
 /*}}}*/
 
-static int main_loop (void) /*{{{*/
+int main (int argc, char **argv) /*{{{*/
 {
-   if (-1 == slrn_select_group_mode ())
-     return -1;
+   if (-1 == main_init_and_parse_args (argc, argv))
+     return 1;
    
-#ifdef REAL_UNIX_SYSTEM
-   if (Can_Suspend)
-     SLtty_set_suspend_state (1);
-#endif
+   if (-1 == slrn_select_group_mode ())
+     return 1;
+
    slrn_push_suspension (1);
    
 #if SLRN_HAS_SLANG
-   (void) SLang_run_hooks ("group_mode_startup_hook", NULL, NULL);
+   (void) SLang_run_hooks ("group_mode_startup_hook", 0);
 #endif
 
-   if (Slrn_Batch) return -1;
+   if (Slrn_Batch) return 1;
    
    while (Slrn_Current_Mode != NULL)
      {
@@ -1626,14 +1677,30 @@ static int main_loop (void) /*{{{*/
 
 	if (SLang_Error || !SLang_input_pending(0))
 	  {
-	     slrn_update_screen (1);
+	     slrn_update_screen ();
 	  }
 
-	do_key (Slrn_Current_Mode->keymap);
+	slrn_do_keymap_key (Slrn_Current_Mode->keymap);
      }
-   return -1;
+
+   return 1;
 }
 
 /*}}}*/
 
 /*}}}*/
+
+int slrn_posix_system (char *cmd, int reset) /*{{{*/
+{
+   int ret;
+   int init_mode = Slrn_TT_Initialized;
+
+   if (reset) slrn_set_display_state (0);
+   ret = SLsystem (cmd);
+   if (reset) slrn_set_display_state (init_mode);
+   Slrn_Full_Screen_Update = 1;
+   return ret;
+}
+
+/*}}}*/
+
