@@ -1,4 +1,24 @@
 /* -*- mode: C; mode: fold -*- */
+/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
+ *
+ * This file is part of slrn.
+ *
+ * Slrn is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ * 
+ * Slrn is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with Slrn; see the file COPYING.  If not, write to the Free
+ * Software Foundation, 59 Temple Place - Suite 330, 
+ * Boston, MA  02111-1307, USA.
+ */
+
 
 #include "config.h"
 
@@ -28,12 +48,12 @@
 
 #include <ctype.h>
 
-/* This should be ported to older systems. */
-#include <dirent.h>
-
+#ifndef S_ISREG
+# define S_ISREG(mode)  (((mode) & (_S_IFMT)) == (_S_IFREG))
+#endif
+ 
 #include <slang.h>
 #include "jdmacros.h"
-
 /*}}}*/
 
 /*{{{ Local Includes */
@@ -43,6 +63,8 @@
 #include "sltcp.h"
 #include "nntplib.h"
 #include "nntpcodes.h"
+#include "slrndir.h"
+#include "version.h"
 
 #include "score.c"
 #include "xover.c"
@@ -67,6 +89,12 @@
 # define SLRNPULL_OUTGOING_DIR	"out.going"
 #endif
 
+#if SLRNPULL_USE_SETGID_POSTS
+# define OUTGOING_DIR_MODE	(0777 | 03000)
+#else
+# define OUTGOING_DIR_MODE	(0777 | 01000)
+#endif
+
 #ifndef SLRNPULL_SCORE_FILE
 # define SLRNPULL_SCORE_FILE	"score"
 #endif
@@ -83,6 +111,20 @@
 # define SLRNPULL_OUTGOING_BAD_DIR	"rejects"
 #endif
 
+#ifndef SLRNPULL_OUTGOING_DIR_MODE
+# define SLRNPULL_OUTGOING_DIR_MODE	(0777 | 01000)
+#endif
+
+static int Exit_Code;
+#define SLRN_EXIT_UNKNOWN		1
+#define SLRN_EXIT_BAD_USAGE		2
+#define SLRN_EXIT_CONNECTION_FAILED	3
+#define SLRN_EXIT_CONNECTION_LOST	4
+#define SLRN_EXIT_SIGNALED		5
+#define SLRN_EXIT_MALLOC_FAILED		10
+
+#define SLRN_EXIT_FILEIO		20
+
 char *SlrnPull_Dir = SLRNPULL_ROOT_DIR;
 char *SlrnPull_Spool_News_Dir;
 char *Group_Min_Max_File;	       /* relative to group dir */
@@ -106,7 +148,38 @@ static int handle_interrupts (void);
 typedef struct _Active_Group_Type /*{{{*/
 {
    unsigned int flags;
-   unsigned int min, max;
+   
+   /* Unfortunately, three different sets of article ranges are required.  
+    * Ideally, only one would be required but this does not seem to be 
+    * possible.  This is because the articles in the spool directory 
+    * created by slrnpull do not expire when the articles on the server
+    * expire.  In addition, slrnpull removes duplicate articles so that
+    * the articles present on the server may not actually be present in the 
+    * spool directory.  
+    * 
+    * The main problem is that it appears that some servers will reuse 
+    * article numbers from articles that have been cancelled.  The spool 
+    * directory created by slrnpull does not know about cancel messages,
+    * which means that the same article number can refer to two different
+    * articles.  The ultimate solution would be for slrnpull to use its own
+    * numbering scheme that is independent of the server's.  This would mean
+    * caching all message-ids and stripping Xref headers from the articles
+    * that are received from the server.  At first sight it would appear that
+    * new Xref headers would have to be generated for the .overview files but
+    * since slrnpull removes duplicates, this should not be necessary.  A
+    * major advantage of this approach is that one could merge multiple feeds.
+    * A rough estimate of the size of the message-id cache is 100 * 500 * 80,
+    * or 4,000,000 bytes assuming 100 newsgroups with 500 articles per group
+    * and a message id of 80 characters.  Of course that number would be
+    * smaller if Pine were eliminated.
+    */
+   unsigned int min, max;	       /* range of articles that slrnpull has 
+					already dealt with */
+   unsigned int active_min, active_max;/* article numbers that are in spool dir */
+   unsigned int server_min;	       /* artcle numbers that server reports */
+   unsigned int server_max;
+   
+   
    unsigned int max_to_get;	       /* if non-zero, get only this many */
    unsigned int expire_days;	       /* if zero, no expiration */
 #define MAX_GROUP_NAME_LEN 80
@@ -125,8 +198,8 @@ static Active_Group_Type *Active_Groups_Tail;
 
 /*}}}*/
 
-static FILE *MLog_Fp = stdout;
-static FILE *ELog_Fp = stderr;
+static FILE *MLog_Fp;
+static FILE *ELog_Fp;
 
 static void write_timestamp (FILE *fp) /*{{{*/
 {
@@ -144,20 +217,28 @@ static void write_timestamp (FILE *fp) /*{{{*/
 
 /*}}}*/
 
-static void va_log (FILE *fp, char *pre, char *fmt, va_list ap) /*{{{*/
+static void write_log (FILE *fp, char *pre, char *buf)
 {
    write_timestamp (fp);
    if (pre != NULL) fputs (pre, fp);
-   vfprintf (fp, fmt, ap);
+   fputs (buf, fp);
    fputc ('\n', fp);
-   
    fflush (fp);
+}
+
+static void va_log (FILE *fp, char *pre, char *fmt, va_list ap) /*{{{*/
+{
+   char buf[2048];   
+
+   vsprintf (buf, fmt, ap);
    
-   if (Stdout_Is_TTY && ((fp != stdout) && (fp != stderr)))
-     {
-	if (fp == MLog_Fp) fp = stdout; else fp = stderr;
-	va_log (fp, pre, fmt, ap);
-     }
+   write_log (fp, pre, buf);
+   
+   if ((Stdout_Is_TTY == 0) || (fp == stdout) || (fp == stderr))
+     return;
+   if (fp == MLog_Fp) fp = stdout; else fp = stderr;
+
+   write_log (fp, pre, buf);
 }
 
 /*}}}*/
@@ -239,9 +320,15 @@ static Active_Group_Type *add_group_type (char *name) /*{{{*/
 
 /*}}}*/
 
+#if defined(__MINGW32__)
+# define MKDIR(x,y) mkdir(x)
+#else
+# define MKDIR(x,y) mkdir(x,y)
+#endif
+
 static int do_mkdir (char *dir, int err) /*{{{*/
 {
-   if (0 == mkdir (dir, 0777))
+   if (0 == MKDIR (dir, 0777))
      {
 	log_message ("Created dir %s.", dir);
 	return 0;
@@ -304,8 +391,8 @@ static int read_group_min_max_file (Active_Group_Type *g) /*{{{*/
 	return -1;
      }
    
-   g->min = min;
-   g->max = max;
+   g->active_min = g->min = min;
+   g->server_max = g->active_max = g->max = max;
    
    return 1;
 }
@@ -351,7 +438,7 @@ static int create_group_directory (Active_Group_Type *g) /*{{{*/
    d = g->dirname;
    while (*d != 0)
      {
-	if (*d == '.') *d = '/';
+	if (*d == '.') *d = SLRN_PATH_SLASH_CHAR;
 	d++;
      }
    
@@ -372,7 +459,7 @@ static int create_group_directory (Active_Group_Type *g) /*{{{*/
    len = strlen (SlrnPull_Spool_News_Dir);
    
    strcpy (dirbuf, SlrnPull_Spool_News_Dir);
-   dirbuf [len] = '/';
+   dirbuf [len] = SLRN_PATH_SLASH_CHAR;
    len++;
    
    dir = dirbuf + len;
@@ -388,7 +475,7 @@ static int create_group_directory (Active_Group_Type *g) /*{{{*/
    do
      {
 	ch = *d;
-	if ((ch == '/') || (ch == 0))
+	if ((ch == SLRN_PATH_SLASH_CHAR) || (ch == 0))
 	  {
 	     *d = 0;
 	     if (-1 == do_mkdir (dirbuf, 1))
@@ -576,36 +663,38 @@ static int read_active_groups (void) /*{{{*/
 
 /*}}}*/
 
-static char *root_dircat (char *name) /*{{{*/
+static char *do_dircat (char *dir, char *name)
 {
    char *f;
 
-   if (*name == '/')
+   if (slrn_is_absolute_path (name))
      f = slrn_strmalloc (name, 0);
    else
-     f = slrn_spool_dircat (SlrnPull_Dir, name, 0);
+     f = slrn_spool_dircat (dir, name, 0);
 
    if (f == NULL)
-     slrn_exit_error ("Out of memory.");
+     {
+	Exit_Code = SLRN_EXIT_MALLOC_FAILED;
+	slrn_exit_error ("Out of memory.");
+     }
+
+#if defined(__os2__) || defined(__NT__)
+   slrn_os2_convert_path (f);
+#endif
 
    return f;
 }
 
+static char *root_dircat (char *name) /*{{{*/
+{
+   return do_dircat (SlrnPull_Dir, name);
+}
+
 /*}}}*/
 
 static char *data_dircat (char *name) /*{{{*/
 {
-   char *f;
-
-   if (*name == '/')
-     f = slrn_strmalloc (name, 0);
-   else
-     f = slrn_spool_dircat (Data_Dir, name, 0);
-
-   if (f == NULL)
-     slrn_exit_error ("Out of memory.");
-
-   return f;
+   return do_dircat (Data_Dir, name);
 }
 
 /*}}}*/
@@ -624,8 +713,11 @@ static int make_filenames (void) /*{{{*/
    
    Outgoing_Bad_Dir = slrn_spool_dircat (Outgoing_Dir, SLRNPULL_OUTGOING_BAD_DIR, 0);
    if (Outgoing_Bad_Dir == NULL)
-     slrn_exit_error ("Out of memory.");
-	
+     {
+	Exit_Code = SLRN_EXIT_MALLOC_FAILED;
+	slrn_exit_error ("Out of memory.");
+     }
+
    if (-1 == do_mkdir (SlrnPull_Spool_News_Dir, 1))
      return -1;
    
@@ -757,8 +849,10 @@ static int *list_server_numbers (NNTP_Type *s, Active_Group_Type *g, unsigned in
 	if (NULL != is_msgid_cached (b1, name, num, 0))
 	  {
 	     if (g->min > g->max) g->min = num;
-	     g->max = num;
-	
+	     g->max = max;	       /* was: g->max = num */
+	     
+	     if (g->server_max < (unsigned int) num) g->server_max = num;
+	     
 	     Num_Duplicates++;
 	     continue;
 	  }
@@ -782,13 +876,34 @@ static int *list_server_numbers (NNTP_Type *s, Active_Group_Type *g, unsigned in
      }
 
    log_message ("%s: Retrieving articles %d-%d.", g->name, min, max);
-
+   if (num_numbers) g->server_max = max;
+   /* Otherwise, there were no unique articles in that range. */
+   
    if (status == -1)
      {
 	slrn_free ((char *) numbers);
 	return NULL;
      }
+
+   /* If there were no articles in the requested range, update g->max now
+    * so that we do not have to try this range next time.
+    * 
+    * Unfortunately, this does not seem possible since the articles in the
+    * requested range may have been cancelled and the server may reuse the
+    * article numbers (yuk).  Note also that g->max has already been updated
+    * for duplicate articles.
+    */
+   if (numbers == NULL)
+     {
+#if 0
+	g->max = max;
+#else
+	log_message ("%s: No articles in specified range.", g->name);
+#endif
+     }
    
+	     
+
    *nump = num_numbers;
    return numbers;
 }
@@ -929,9 +1044,8 @@ static int write_head_and_body (Active_Group_Type *g, int n, /*{{{*/
    if (-1 == write_xover_line (xov_fp, xov))
      return -1;
 
-   if (g->min > g->max) g->min = n;
-   g->max = n;
-   
+   if (g->min > g->max) g->active_min = g->min = n;
+   g->active_max = g->max = n;
    
    return 0;
 }
@@ -1055,7 +1169,6 @@ static int fetch_head (NNTP_Type *s, int n, char **headers, Slrn_XOver_Type *xov
 
 /*}}}*/
 
-
 static int get_heads (NNTP_Type *s, int *numbers, char **heads, /*{{{*/
 		      Slrn_XOver_Type *xovs, unsigned int num)
 {
@@ -1167,7 +1280,8 @@ static int get_articles (NNTP_Type *s, Active_Group_Type *g, int *numbers, unsig
 
 /*}}}*/
 
-static int get_group_articles (NNTP_Type *s, Active_Group_Type *g, int min, int max) /*{{{*/
+static int get_group_articles (NNTP_Type *s, Active_Group_Type *g, 
+			       int server_min, int server_max) /*{{{*/
 {
    unsigned int gmin, gmax;
    int *numbers;
@@ -1180,13 +1294,13 @@ static int get_group_articles (NNTP_Type *s, Active_Group_Type *g, int min, int
    gmin = g->min;
    gmax = g->max;
    
-   if (((min > max) || (max < 0))
-       || (((unsigned int)max <= gmax) && (gmin <= gmax)))
+   if (((server_min > server_max) || (server_max < 0))
+       || (((unsigned int)server_max <= gmax) && (gmin <= gmax)))
      {
 	log_message ("%s: no new articles available.", g->name);
 	return 0;
      }
-   
+
    Num_Duplicates = 0;
    numbers = list_server_numbers (s, g, &num_numbers);
    if (Num_Duplicates)
@@ -1201,7 +1315,8 @@ static int get_group_articles (NNTP_Type *s, Active_Group_Type *g, int min, int
    
    if (i == num_numbers)
      {
-	log_message ("%s: No articles available.", g->name);
+	g->max = g->server_max;
+	log_message ("%s: No new articles available.", g->name);
 	slrn_free ((char *) numbers);
 	return 0;
      }
@@ -1242,8 +1357,9 @@ static int get_group_articles (NNTP_Type *s, Active_Group_Type *g, int min, int
 	Num_Articles_Received += j;
      }
    
+   g->max = g->server_max;
+   
    (void) slrn_close_score ();
-
    slrn_free ((char *) numbers);
    return 0;
 }
@@ -1265,11 +1381,18 @@ static int pull_news (NNTP_Type *s) /*{{{*/
 	status = nntp_select_group (s, g->name, &min, &max);
 	if (status != OK_GROUP)
 	  {
-	     log_error ("Error selecting group %s.", g->name);
+	     log_error ("Error selecting group %s.  Code = %d", g->name, status);
+
+	     if (status == -1)
+	       break;
+
 	     g = g->next;
 	     continue;
 	  }
 
+	if (g->server_max > (unsigned int) max)
+	  g->server_max = max;
+
 	Current_Newsgroup = g->name;
 	
 	(void) get_group_articles (s, g, min, max);
@@ -1367,15 +1490,15 @@ static int post_file (NNTP_Type *s, char *file) /*{{{*/
 static int make_outgoing_dir (char *dir) /*{{{*/
 {
    log_error ("%s directory does not exist.  Creating it...", dir);
-   if (-1 == mkdir (dir, 0700))
+   if (-1 == MKDIR (dir, 0700))
      {
 	log_error ("Unable to create %s.", dir);
 	return -1;
      }
 	
-   if (-1 == chmod (dir, 0777 | 01000))
-     log_error ("chmod 01777 failed on %s.", dir);
-   
+   if (-1 == chmod (dir, OUTGOING_DIR_MODE))
+     log_error ("chmod 0%o failed on %s.", OUTGOING_DIR_MODE, dir);
+
    return 0;
 }
 
@@ -1383,12 +1506,12 @@ static int make_outgoing_dir (char *dir) /*{{{*/
 
 static int post_outgoing (NNTP_Type *s) /*{{{*/
 {
-   DIR *dp;
-   struct dirent *df;
+   Slrn_Dir_Type *dp;
+   Slrn_Dirent_Type *df;
    int n;
    char file [SLRN_MAX_PATH_LEN + 1];
    
-   dp = opendir (Outgoing_Dir);
+   dp = slrn_open_dir (Outgoing_Dir);
    if (dp == NULL)
      return make_outgoing_dir (Outgoing_Dir);
 
@@ -1403,11 +1526,11 @@ static int post_outgoing (NNTP_Type *s) /*{{{*/
      }
 
    n = 0;
-   while (NULL != (df = readdir (dp)))
+   while (NULL != (df = slrn_read_dir (dp)))
      {
 	char *name;
 	
-	name = df->d_name;
+	name = df->name;
 
 	if (*name != 'X')
 	  continue;
@@ -1427,7 +1550,7 @@ static int post_outgoing (NNTP_Type *s) /*{{{*/
 	  }
      }
    
-   closedir (dp);
+   slrn_close_dir (dp);
    return n;
 }
 
@@ -1450,7 +1573,8 @@ static int write_active (void) /*{{{*/
    
    while (g != NULL)
      {
-	if (EOF == fprintf (fp, "%s %u %u y\n", g->name, g->max, g->min))
+	if (EOF == fprintf (fp, "%s %u %u y\n", g->name, 
+			    g->active_max, g->active_min))
 	  {
 	     fclose (fp);
 	     goto write_error;
@@ -1483,20 +1607,48 @@ static int write_active (void) /*{{{*/
 static NNTP_Type *Pull_Server;
 static time_t Actual_Start_Time;
 
+static void connection_lost_hook (NNTP_Type *s)
+{
+   log_error ("Connection to %s lost. Performing shutdown.", s->host);
+   (void) write_active ();
+   Exit_Code = SLRN_EXIT_CONNECTION_LOST;
+   slrn_exit_error (NULL);
+}
+
+   
 static int open_servers (char *host) /*{{{*/
 {
+   if (-1 == sltcp_open_sltcp ())
+     {
+	fprintf (stderr, "Error opening sltcp interface.\n");
+	return -1;
+     }
+
    time (&Actual_Start_Time);
    Pull_Server = nntp_open_server (host, -1);
    if (Pull_Server == NULL)
-     return -1;
-   
+     {
+	sltcp_close_sltcp ();
+	return -1;
+     }
+
+   /* Since multple commands are sent, reconnection is not good idea since
+    * the context is fuzzy.
+    */
+
+   /* Pull_Server->flags |= NNTP_RECONNECT_OK; */
+
+   NNTP_Connection_Lost_Hook = connection_lost_hook;
+
    Pull_Server->tcp->bytes_in = Pull_Server->tcp->bytes_out = 0;
    time (&Start_Time);
    
    /* Probe for XHDR now because DNEWS does not handle probing later properly. */
    if (-1 == nntp_has_cmd (Pull_Server, "XHDR"))
-     return -1;
-
+     {
+	sltcp_close_sltcp ();
+	return -1;
+     }
    return 0;
 }
 
@@ -1522,6 +1674,8 @@ static void close_servers (void) /*{{{*/
    unsigned long bytes_in = 0;
    unsigned long bytes_out = 0;
 
+   (void) sltcp_close_sltcp ();
+
    if (Pull_Server != NULL)
      {
 	if (Pull_Server->tcp != NULL) 
@@ -1578,13 +1732,31 @@ static void close_log_files (void) /*{{{*/
 
 static void usage (char *pgm) /*{{{*/
 {
-   log_error ("%s usage: %s [-h HOSTNAME] [-d SPOOLDIR] [--expire] [--post] [--new-groups]\n", pgm, pgm);
+   log_error ("%s usage: %s [options]\n\
+ Options:\n\
+  -h HOSTNAME       name of NNTP server\n\
+  -d SPOOLDIR       Spool Directory associated with server\n\
+  --expire          Perform expiration but do not pull news\n\
+  --post-only       Post news but do not pull new news\n\
+  --no-post         Do not post news\n\
+  --new-groups      Get new groups\n\
+  --version         Show version number\n\
+",
+	      pgm, pgm);
    close_log_files ();
-   exit (1);
+   exit (SLRN_EXIT_BAD_USAGE);
 }
 
 /*}}}*/
 
+static void show_version (char *pgm)
+{
+   log_error ("%s version: %s\n", pgm, Slrn_Version);
+   close_log_files ();
+   exit (0);
+}
+
+
 static int read_score_file (void) /*{{{*/
 {
    char file [SLRN_MAX_PATH_LEN + 1];
@@ -1615,8 +1787,11 @@ int main (int argc, char **argv) /*{{{*/
    
    Stdout_Is_TTY = isatty (fileno(stdout));
    expire_mode = 0;
-   post_mode = 0;
-   
+   post_mode = -1;
+
+   MLog_Fp = stdout;
+   ELog_Fp = stderr;
+
    dir = getenv ("SLRNPULL_ROOT");
    
    while (argc > 0)
@@ -1639,10 +1814,15 @@ int main (int argc, char **argv) /*{{{*/
 	  }
 	else if (!strcmp (arg, "--expire"))
 	  expire_mode = 1;
-	else if (!strcmp (arg, "--post"))
+	else if ((!strcmp (arg, "--post-only"))
+		 || (!strcmp (arg, "--post")))
 	  post_mode = 1;
+	else if (!strcmp (arg, "--no-post"))
+	  post_mode = 0;
 	else if (!strcmp (arg, "--new-groups"))
 	  check_new_groups = 1;
+	else if (!strcmp (arg, "--version"))
+	  show_version (pgm);
 	else usage (pgm);
      }
 
@@ -1663,6 +1843,8 @@ int main (int argc, char **argv) /*{{{*/
      log_message ("slrnpull started.");
    
    SLang_init_case_tables ();
+   
+   Exit_Code = SLRN_EXIT_FILEIO;
 
    if (-1 == make_filenames ())
      slrn_exit_error (NULL);
@@ -1685,22 +1867,30 @@ int main (int argc, char **argv) /*{{{*/
    if (-1 == read_score_file ())
      slrn_exit_error (NULL);
      
+   Exit_Code = SLRN_EXIT_UNKNOWN;
+
    if (-1 == open_servers (host))
-     slrn_exit_error ("Unable to initialize server.");
+     {
+	Exit_Code = SLRN_EXIT_CONNECTION_FAILED;
+	slrn_exit_error ("Unable to initialize server.");
+     }
 
    SLTCP_Interrupt_Hook = handle_interrupts;
    
-   post_outgoing (Pull_Server);
+   if (post_mode) post_outgoing (Pull_Server);
    
    if (check_new_groups)
      (void) get_new_groups (Pull_Server);
        
-   if (post_mode == 0)
+   if (post_mode != 1)
      {
 	init_signals ();
 	pull_news (Pull_Server);
 	if (-1 == write_active ())
-	  slrn_exit_error (NULL);
+	  {
+	     Exit_Code = SLRN_EXIT_FILEIO;
+	     slrn_exit_error (NULL);
+	  }
      }
 
    close_servers ();
@@ -1871,17 +2061,18 @@ void slrn_exit_error (char *fmt, ...) /*{{{*/
    close_servers ();
    close_log_files ();
    
-   exit (1);
+   exit (Exit_Code);
 }
 
 /*}}}*/
 
 
-static Terminate_Slrn_Pull_Requested;
+static int Terminate_Slrn_Pull_Requested;
 static void sigint_handler (int sig) /*{{{*/
 {
    (void) sig;
 
+   Exit_Code = SLRN_EXIT_SIGNALED;
    SLKeyBoard_Quit = 1;
    Terminate_Slrn_Pull_Requested = 1;
 }
@@ -1891,9 +2082,13 @@ static void sigint_handler (int sig) /*{{{*/
 static void init_signals (void) /*{{{*/
 {
    SLsignal_intr (SIGINT, sigint_handler);
+#ifdef SIGHUP
    SLsignal_intr (SIGHUP, sigint_handler);
+#endif
    SLsignal_intr (SIGTERM, sigint_handler);
+#ifdef SIGPIPE
    SLsignal_intr (SIGPIPE, SIG_IGN);
+#endif
 }
 
 /*}}}*/
@@ -1904,6 +2099,7 @@ static int handle_interrupts (void) /*{{{*/
      {
 	log_error ("Performing shutdown.");
 	write_active ();
+	Exit_Code = SLRN_EXIT_SIGNALED;
 	slrn_exit_error ("Slrn exiting on signal.");
      }
    
@@ -1913,9 +2109,10 @@ static int handle_interrupts (void) /*{{{*/
 /*}}}*/
 
 
-void slrn_error_now (char *fmt, ...) /*{{{*/
+void slrn_error_now (unsigned int unused, char *fmt, ...) /*{{{*/
 {
    va_list ap;
+   (void) unused;
    va_start(ap, fmt);
    va_log_error (fmt, ap);
    va_end (ap);
@@ -2040,10 +2237,16 @@ static int create_overview_for_dir (Active_Group_Type *g, unsigned int *nums, un
 	
 	(*qsort_fun) ((char *) nums, n_nums, sizeof (unsigned int), sort_int_cmp);
 	
-	g->min = nums[0];
+	g->active_min = g->min = nums[0];
+	g->active_max = nums [n_nums - 1];
      }
-   else g->min = g->max + 1;
-
+   else 
+     {
+	g->active_min = g->min = g->max + 1;
+	g->active_max = g->active_min - 1;
+     }
+   
+   
    for (i = 0; i < n_nums; i++)
      {
 	char *header;
@@ -2095,8 +2298,8 @@ static int create_overview_for_dir (Active_Group_Type *g, unsigned int *nums, un
 
 static int expire_group (Active_Group_Type *g) /*{{{*/
 {
-   DIR *dp;
-   struct dirent *df;
+   Slrn_Dir_Type *dp;
+   Slrn_Dirent_Type *df;
    char dir [SLRN_MAX_PATH_LEN + 1];
    char file [SLRN_MAX_PATH_LEN + 1];
    unsigned int *ok_names;
@@ -2112,7 +2315,7 @@ static int expire_group (Active_Group_Type *g) /*{{{*/
    if (-1 == slrn_dircat (SlrnPull_Spool_News_Dir, g->dirname, dir))
      return -1;
 
-   dp = opendir (dir);
+   dp = slrn_open_dir (dir);
    if (dp == NULL)
      {
 	log_error ("opendir %s failed.", dir);
@@ -2122,11 +2325,11 @@ static int expire_group (Active_Group_Type *g) /*{{{*/
    ok_names = NULL;
    max_num_ok_names = num_ok_names = 0;
    
-   while (NULL != (df = readdir (dp)))
+   while (NULL != (df = slrn_read_dir (dp)))
      {
 	char *name, *p;
 	
-	name = df->d_name;
+	name = df->name;
 	
 	/* Look for names composed of digits.  Skip others. */
 	p = name;
@@ -2143,7 +2346,7 @@ static int expire_group (Active_Group_Type *g) /*{{{*/
 	     if (ok_names == NULL)
 	       {
 		  log_error ("malloc error. Unable to expire group %s.", g->name);
-		  closedir (dp);
+		  slrn_close_dir (dp);
 		  return -1;
 	       }
 	  }
@@ -2152,7 +2355,7 @@ static int expire_group (Active_Group_Type *g) /*{{{*/
 	num_ok_names++;
      }
    
-   closedir (dp);
+   slrn_close_dir (dp);
 
    min_not_expired = 0xFFFFFFFF;
    time (&expire_time);
@@ -2231,6 +2434,14 @@ static int do_expire (void) /*{{{*/
 
    while (g != NULL)
      {
+	if (Terminate_Slrn_Pull_Requested)
+	  {
+	     log_error ("Termination requested.  Shutting down.");
+	     (void) write_active ();
+	     Exit_Code = SLRN_EXIT_SIGNALED;
+	     slrn_exit_error (NULL);
+	  }
+	
 	(void) expire_group (g);
 	g = g->next;
      }
