@@ -1,3 +1,23 @@
+/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
+ *
+ * This file is part of slrn.
+ *
+ * Slrn is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ * 
+ * Slrn is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with Slrn; see the file COPYING.  If not, write to the Free
+ * Software Foundation, 59 Temple Place - Suite 330, 
+ * Boston, MA  02111-1307, USA.
+ */
+
 /* Local spool support for slrn added by Olly Betts <olly@mantis.co.uk> */
 #include "config.h"
 #include "slrnfeat.h"
@@ -26,6 +46,7 @@
 
 #include "misc.h"
 #include "slrn.h"
+#include "slrndir.h"
 
 #ifndef SLRN_SPOOL_ROOT
 # define SLRN_SPOOL_ROOT "/var/spool/news" /* a common place for the newsspool */
@@ -43,24 +64,6 @@
 #include <sys/stat.h>
 #include <assert.h>
 
-#if HAVE_DIRENT_H
-# include <dirent.h>
-# define NAMLEN(dirent) strlen((dirent)->d_name)
-#else
-# define dirent direct
-# define NAMLEN(dirent) (dirent)->d_namlen
-# define NEED_D_NAMLEN
-# if HAVE_SYS_NDIR_H
-#  include <sys/ndir.h>
-# endif
-# if HAVE_SYS_DIR_H
-#  include <sys/dir.h>
-# endif
-# if HAVE_NDIR_H
-#  include <ndir.h>
-# endif
-#endif
-
 #include <limits.h>
 
 static int spool_put_server_cmd (char *, char *, unsigned int );
@@ -77,6 +80,7 @@ static int spool_list_newsgroups (void);
 static int spool_list_active (void);
 static char *spool_read_xover (char *, unsigned int);
 static char *spool_read_xpat (char *, unsigned int);
+static int spool_article_num_exists (int);
 
 static Slrn_Server_Obj_Type Spool_Server_Obj;
 
@@ -236,7 +240,9 @@ static int spool_nntp_xover (int min, int max)
 		    && (ch != EOF))
 	       ; /* do nothing */
 	  }
-
+#ifndef SEEK_SET
+# define SEEK_SET	0
+#endif
 	fseek (Spool_fh_local, fp, SEEK_SET); /* reset to start of line */
      }
 
@@ -251,7 +257,7 @@ static char *spool_read_xover (char *the_buf, unsigned int len)
 {
    char *p;
    long pos;
-   
+
 #if DEBUG_SPOOL
    spool_debug ("spool_read_xover");
 #endif
@@ -265,37 +271,54 @@ static char *spool_read_xover (char *the_buf, unsigned int len)
 	return NULL;
      }
 
-   pos = ftell (Spool_fh_nov);
-   
-   p = fgets (the_buf, len, Spool_fh_nov);
-   if (p != NULL)
+   while (1)
      {
 	unsigned int buflen;
 
+	pos = ftell (Spool_fh_nov);
+   
+	if (NULL == (p = fgets (the_buf, len, Spool_fh_nov)))
+	  {
+	     Spool_Doing_XOver = 0;
+	     return NULL;
+	  }
+	
 	buflen = strlen (the_buf);
 	if (buflen && (the_buf[buflen - 1] == '\n'))
 	  the_buf [buflen - 1] = 0;
-     }
    
-   /* check if we've reached the end of the requested range */
-   if (p != NULL)
-     {
+	/* check if we've reached the end of the requested range */
 	Spool_XOver_Next = atoi (p);
 	if (Spool_XOver_Next > Spool_XOver_Max)
 	  {
 	     fseek (Spool_fh_nov, pos, SEEK_SET);
-	     p = NULL;
+	     Spool_Doing_XOver = 0;
+	     return NULL;
 	  }
-	else Spool_XOver_Next++;
+	
+	if (Slrn_Spool_Check_Up_On_Nov == 0)
+	  break;
+	
+	/* check that the article file actually exists */
+	/* if not, this nov entry is defunct, so ignore it */
+#if DEBUG_SPOOL
+	spool_debug ("Nov entry %ld", Spool_XOver_Next);
+#endif
+	
+	if (0 == spool_article_num_exists (Spool_XOver_Next))
+	  {
+#if DEBUG_SPOOL
+	     spool_debug (" - OK");
+#endif
+	     break;
+	  }
+	
+#if DEBUG_SPOOL
+	spool_debug (" skipped!");
+#endif
      }
 
-#if DEBUG_SPOOL
-   spool_debug (p);
-#endif
-   
-   if (p == NULL) 
-     Spool_Doing_XOver = 0;
-   
+   Spool_XOver_Next++;
    return p;
 }
 
@@ -667,21 +690,18 @@ static int spool_put_server_cmd (char *line, char *buf, unsigned int len)
 /*}}}*/
 
 
-static int spool_read_minmax_from_dp (DIR *dp, int *min, int *max)
+static int spool_read_minmax_from_dp (Slrn_Dir_Type *dp, int *min, int *max)
 {
-   struct dirent *ep;
+   Slrn_Dirent_Type *ep;
    char *p;
    long l;
    long hi = 0;
    long lo = LONG_MAX;
 
    /* Scan through all the files, checking the ones with numbers for names */
-   while ((ep = readdir(dp)) != NULL)
+   while ((ep = slrn_read_dir(dp)) != NULL)
      {
-	p = ep->d_name;
-#ifdef NEED_D_NAMLEN
-	p[ep->d_namlen] = 0;
-#endif
+	p = ep->name;
 	if (!isdigit(*p)) continue;
 
 	if (0 == spool_is_name_all_digits (p))
@@ -708,18 +728,20 @@ static int spool_read_minmax_from_dp (DIR *dp, int *min, int *max)
 
 /* Get the lowest and highest article numbers by the simple method
  * or looking at the files in the directory.
- * Returns 1 on success, 0 on failure
+ * Returns 0 on success, -1 on failure
  */
-static int spool_read_minmax_from_dir( int *min, int *max, char *dir )
+static int spool_read_minmax_from_dir (int *min, int *max, char *dir)
 {
-   /* This is adapted from some code in INN's ng.c */
-   DIR *dp;
+   Slrn_Dir_Type *dp;
 
    if (dir == NULL) dir = ".";
    
    /* I suspect this is very unlikely to fail */
-   if ((dp = opendir(dir)) == NULL)
-     return 0;
+   if ((dp = slrn_open_dir (dir)) == NULL)
+     {
+	slrn_error ("Unable to open directory %s", dir);
+	return -1;
+     }
 
    if (-1 == spool_read_minmax_from_dp (dp, min, max))
      {
@@ -727,13 +749,13 @@ static int spool_read_minmax_from_dir( int *min, int *max, char *dir )
 	*max = 0;
      }
 
-   (void) closedir(dp);
-   return 1;
+   (void) slrn_close_dir(dp);
+   return 0;
 }
 
 #if SPOOL_ACTIVE_FOR_ART_RANGE
 /* Get the lowest and highest article numbers from the active file
- * Returns 1 on success, 0 on failure
+ * Returns 0 on success, -1 on failure
  * (failure => active file didn't open, or the group wasn't in it)
  */
 static int spool_read_minmax_from_active( char *name, int *min, int *max )
@@ -743,7 +765,7 @@ static int spool_read_minmax_from_active( char *name, int *min, int *max )
 
    spool_fclose_local();
    Spool_fh_local = fopen(Slrn_Active_File,"r");
-   if (Spool_fh_local == NULL) return 0;
+   if (Spool_fh_local == NULL) return -1;
 
    len = strlen(name);
    buf[len] = 0;		       /* init this for test below */
@@ -756,25 +778,25 @@ static int spool_read_minmax_from_active( char *name, int *min, int *max )
 	  {
 	     spool_fclose_local ();
 	     if (2 != sscanf (buf + len + 1, "%d%d", max, min))
-	       return 0;
+	       return -1;
 
 	     Spool_Max_Artnum = *max;
 # if DEBUG_SPOOL
 	     spool_debug ("from active:%s %d %d",
 			  name,*min,*max);
 # endif
-	     return 1;
+	     return 0;
 	  }
 	buf[len] = 0;
      }
    spool_fclose_local();
 
-   return 0;
+   return -1;
 }
 #endif
 
 /* Get the lowest and highest article numbers from the overview file
- * Returns 1 on success, 0 on failure
+ * Returns 0 on success, -1 on failure
  */
 static int spool_read_minmax_from_overview (char *name, int *min, int *max)
 {
@@ -795,7 +817,7 @@ static int spool_read_minmax_from_overview (char *name, int *min, int *max)
 				    && (1 == fscanf (Spool_fh_nov,"%d", min)));
 
    if (0 == Spool_Server_Obj.sv_has_xover)
-     return 0;
+     return -1;
 
    /* find start of last line */
    fseek (Spool_fh_nov, 0, SEEK_END); /* !HACK! check return value from this and all the rest... */
@@ -861,7 +883,7 @@ static int spool_read_minmax_from_overview (char *name, int *min, int *max)
 #if DEBUG_SPOOL
    spool_debug ("%s %d %d",name,*min,*max);
 #endif
-   return 1;
+   return 0;
 }
 
 static int spool_select_group (char *name, int *min, int *max)
@@ -887,9 +909,9 @@ static int spool_select_group (char *name, int *min, int *max)
 
    Spool_fh_nov = spool_open_nov_file ();
 
-   if (!spool_read_minmax_from_overview (name, min, max)
+   if ((-1 == spool_read_minmax_from_overview (name, min, max))
 #if SPOOL_ACTIVE_FOR_ART_RANGE
-       && !spool_read_minmax_from_active (name, min, max)
+       && (-1 == spool_read_minmax_from_active (name, min, max)))
 #endif
        )
      {
@@ -900,7 +922,7 @@ static int spool_select_group (char *name, int *min, int *max)
 	if (chdir (Spool_Group))
 	  return ERR_NOGROUP;
 #endif
-	if (!spool_read_minmax_from_dir (min, max, Spool_Group))
+	if (-1 == spool_read_minmax_from_dir (min, max, Spool_Group))
 	  return -1;
 	
      }
@@ -1215,7 +1237,7 @@ static int spool_list_active (void)
 typedef struct _Spool_DirTree_Type
 {
    struct _Spool_DirTree_Type *parent;
-   DIR *dp;
+   Slrn_Dir_Type *dp;
    int len;
    long lo, hi;
 }
@@ -1230,19 +1252,19 @@ static int Spool_Is_LeafDir;
 static void spool_fake_active_in (char *dir)
 {
    char *p;
-   DIR *dp;
+   Slrn_Dir_Type *dp;
    Spool_DirTree_Type *tmp;
 
    p = Spool_Buf + strlen(Spool_Buf);
 
    if (dir != NULL)
      {
-	*p = '/';
+	*p = SLRN_PATH_SLASH_CHAR;
 	strcpy (p + 1, dir);
      }
 
    if ((2 != slrn_file_exists (Spool_Buf))
-       || (NULL == (dp = opendir (Spool_Buf))))
+       || (NULL == (dp = slrn_open_dir (Spool_Buf))))
      {
 	*p = 0;
 	return;
@@ -1275,7 +1297,7 @@ static void spool_fake_active_out (void)
    Spool_DirTree_Type *tmp;
    int i;
 
-   (void)closedir (Spool_Head->dp);
+   (void)slrn_close_dir (Spool_Head->dp);
    Spool_Is_LeafDir = 0;
 
    Spool_Buf [strlen(Spool_Buf) - Spool_Head->len - 1] = '\0';
@@ -1301,7 +1323,7 @@ static int spool_fake_active (char *path)
 
 static char *spool_fakeactive_read_line(char *line, int len)
 {
-   struct dirent *ep;
+   Slrn_Dirent_Type *ep;
    char *p;
    long l;
 
@@ -1317,12 +1339,10 @@ static char *spool_fakeactive_read_line(char *line, int len)
      }
 
    /* Scan through all the files, checking the ones with numbers for names */
-   while ((ep = readdir(Spool_Head->dp)) != NULL)
+   while ((ep = slrn_read_dir(Spool_Head->dp)) != NULL)
      {
-	p = ep->d_name;
-#ifdef NEED_D_NAMLEN
-	p[ep->d_namlen] = 0;
-#endif
+	p = ep->name;
+
 	if ((0 == spool_is_name_all_digits (p))
 	    || ((l = atol (p)) == 0))
 	  {
@@ -1364,6 +1384,10 @@ static char *spool_fakeactive_read_line(char *line, int len)
    return line;
 }
 
+static void spool_reset (void)
+{
+   spool_fclose_local ();
+}
 
 char *Slrn_Inn_Root;
 char *Slrn_Spool_Root;
@@ -1372,12 +1396,14 @@ char *Slrn_Nov_File;
 char *Slrn_Active_File;
 char *Slrn_ActiveTimes_File;
 char *Slrn_Newsgroups_File;
+int Slrn_Spool_Check_Up_On_Nov;
 
 static int spool_init_objects (void)
 {
    Spool_Server_Obj.sv_select_group = spool_select_group;
    Spool_Server_Obj.sv_read_line = spool_read_line;
    Spool_Server_Obj.sv_close = spool_close_server;
+   Spool_Server_Obj.sv_reset = spool_reset;
    Spool_Server_Obj.sv_initialize = spool_initialize_server;
    Spool_Server_Obj.sv_select_article = spool_select_article;
    Spool_Server_Obj.sv_put_server_cmd = spool_put_server_cmd;
@@ -1400,7 +1426,15 @@ static int spool_init_objects (void)
    Slrn_Active_File = slrn_safe_strmalloc (SLRN_SPOOL_ACTIVE);
    Slrn_ActiveTimes_File = slrn_safe_strmalloc (SLRN_SPOOL_ACTIVETIMES);
    Slrn_Newsgroups_File = slrn_safe_strmalloc (SLRN_SPOOL_NEWSGROUPS);
-   
+#if defined(IBMPC_SYSTEM)
+   slrn_os2_convert_path (Slrn_Inn_Root);
+   slrn_os2_convert_path (Slrn_Spool_Root);
+   slrn_os2_convert_path (Slrn_Nov_Root);
+   slrn_os2_convert_path (Slrn_Nov_File);
+   slrn_os2_convert_path (Slrn_Active_File);
+   slrn_os2_convert_path (Slrn_ActiveTimes_File);
+   slrn_os2_convert_path (Slrn_Newsgroups_File);
+#endif   
    return 0;
 }
 
@@ -1409,7 +1443,9 @@ static char *spool_root_dircat (char *file)
 {
    char *f;
 
-   if (*file == '/') return file;
+   if (slrn_is_absolute_path (file))
+     return file;
+
    f = slrn_spool_dircat (Slrn_Inn_Root, file, 0);
    SLFREE (file);
    return f;
