@@ -1,7 +1,24 @@
 /* -*- mode: C; mode: fold; -*- */
-/*  Copyright (c) 1995, 1996 John E. Davis (davis@space.mit.edu)
- *  All rights reserved.
+/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
+ *
+ * This file is part of slrn.
+ *
+ * Slrn is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ * 
+ * Slrn is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with Slrn; see the file COPYING.  If not, write to the Free
+ * Software Foundation, 59 Temple Place - Suite 330, 
+ * Boston, MA  02111-1307, USA.
  */
+
 /* Read startup .slrnrc file */
 
 #include "config.h"
@@ -39,6 +56,7 @@
 #endif
 #include "server.h"
 #include "chmap.h"
+#include "print.h"
 
 #ifdef VMS
 # include "vms.h"
@@ -59,6 +77,8 @@ static int set_variable_fun (int, SLcmd_Cmd_Table_Type *);
 static int nnrp_fun (int, SLcmd_Cmd_Table_Type *);
 static int grouplens_fun (int, SLcmd_Cmd_Table_Type *);
 static int interpret_fun (int, SLcmd_Cmd_Table_Type *);
+static int include_file_fun (int, SLcmd_Cmd_Table_Type *);
+static int set_header_format_fun (int, SLcmd_Cmd_Table_Type *);
 
 /*}}}*/
 /*{{{ Static Global Variables */
@@ -67,6 +87,9 @@ static int This_Line_Num;	       /* current line number in startup file */
 static char *This_File;
 static char *This_Line;		       /* line being parsed */
 
+char *Server_Object;
+char *Post_Object;
+
 static SLcmd_Cmd_Table_Type Slrn_Cmd_Table;
 static SLcmd_Cmd_Type Slrn_Startup_File_Cmds[] = /*{{{*/
 {
@@ -76,7 +99,6 @@ static SLcmd_Cmd_Type Slrn_Startup_File_Cmds[] = /*{{{*/
      {color_fun, "color", "SSS"},
      {mono_fun, "mono", "SSsss"},
      {set_variable_fun, "set", "SG"},
-     {user_data_fun, "username", "S"},
      {user_data_fun, "hostname", "S"},
      {nnrp_fun, "nnrpaccess", "SSS" },
 #define SLRN_MAX_QUOTE_REGEXP 5
@@ -84,9 +106,12 @@ static SLcmd_Cmd_Type Slrn_Startup_File_Cmds[] = /*{{{*/
      {autobaud_fun, "autobaud", ""},
      {grouplens_fun, "grouplens_add", "S"},
      {interpret_fun, "interpret", "S"},
+     {include_file_fun, "include", "S"},
+     {set_header_format_fun, "header_display_format", "IS"},
      {user_data_fun, "scorefile", "S"},
    
    /* The following are considered obsolete */
+     {user_data_fun, "username", "S"},
      {user_data_fun, "replyto", "S"},
      {user_data_fun, "organization", "S"},
      {user_data_fun, "signature", "S"},
@@ -279,6 +304,12 @@ static int grouplens_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 
 /*}}}*/
 
+static int set_header_format_fun (int argc, SLcmd_Cmd_Table_Type *table) 
+{
+   (void) argc;
+   return slrn_set_header_format (table->int_args[1], table->string_args[2]);
+}
+
 /*{{{ Setting/Getting Variable Functions */
 
 typedef struct /*{{{*/
@@ -297,6 +328,8 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
      {"use_color", &SLtt_Use_Ansi_Colors},
      {"ignore_signature", &Slrn_Sig_Is_End_Of_Article},
      {"reject_long_lines", &Slrn_Reject_Long_Lines},
+     {"generate_message_id", &Slrn_Generate_Message_Id},
+     {"display_cursor_bar", &Slrn_Display_Cursor_Bar},
 #if SLRN_HAS_NNTP_SUPPORT
      {"query_reconnect", &Slrn_Query_Reconnect},
 #else
@@ -309,8 +342,8 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
 #endif
      {"use_xgtitle", &Slrn_Use_Xgtitle},
      {"show_article", &Slrn_Startup_With_Article},
-     {"author_display", &Slrn_Show_Author},
-     {"display_author_realname", &Slrn_Show_Author_Realname},
+     {"author_display", NULL},
+     {"display_author_realname", NULL},
      {"show_descriptions", &Slrn_Group_Display_Descriptions},
      {"no_backups", &Slrn_No_Backups},
      {"beep", &SLtt_Ignore_Beep},
@@ -331,7 +364,7 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
      {"min_high_score", &Slrn_High_Score_Min},
      {"max_low_score", &Slrn_Low_Score_Max},
      {"kill_score", &Slrn_Kill_Score_Max},
-#ifndef __os2__
+#if !defined(IBMPC_SYSTEM)
      {"use_blink", &SLtt_Blink_Mode},
 #endif
      {"wrap_flags", &Slrn_Wrap_Mode},
@@ -373,6 +406,11 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
      {"use_tilde", &Slrn_Use_Tildes},
 #else
      {"use_tilde", NULL},
+#endif
+#if SLRN_HAS_SPOOL_SUPPORT
+     {"spool_check_up_on_nov", &Slrn_Spool_Check_Up_On_Nov},
+#else
+     {"spool_check_up_on_nov", NULL},
 #endif
      {NULL, NULL}
 };
@@ -395,13 +433,13 @@ static Set_String_Type String_Things_To_Set [] = /*{{{*/
      {"group_help_line", &Slrn_Group_Help_Line},
      {"quote_string", &Slrn_Quote_String},
      {"realname", &Slrn_User_Info.realname},
+     {"username", &Slrn_User_Info.username},
      {"replyto", &Slrn_User_Info.replyto},
      {"organization", &Slrn_User_Info.org},
-     {"followup", &Slrn_User_Info.followup_string},
+     {"followup", &Slrn_User_Info.followup_string},   /* obsolete */
+     {"followup_string", &Slrn_User_Info.followup_string},
+     {"reply_string", &Slrn_User_Info.reply_string},
      {"cc_followup_string", &Slrn_Courtesy_CC_Message},
-#if SLRN_HAS_DECODE
-     {"decode_directory", &Slrn_Decode_Directory},
-#endif
      {"editor_command", &Slrn_Editor},
      {"post_editor_command", &Slrn_Editor_Post},
      {"score_editor_command", &Slrn_Editor_Score},
@@ -409,7 +447,9 @@ static Set_String_Type String_Things_To_Set [] = /*{{{*/
      {"non_Xbrowser", &Slrn_NonX_Browser},
      {"Xbrowser", &Slrn_X_Browser},
      {"save_posts", &Slrn_Save_Posts_File},
+     {"save_replies", &Slrn_Save_Replies_File},
      {"save_directory", &Slrn_Save_Directory},
+     {"postpone_directory", &Slrn_Postpone_Dir},
      {"signature", &Slrn_User_Info.signature},
      {"custom_headers", &Slrn_Post_Custom_Headers},
      {"followup_custom_headers", &Slrn_Followup_Custom_Headers},
@@ -508,7 +548,9 @@ static Set_String_Type String_Things_To_Set [] = /*{{{*/
 	  NULL
 #endif
      },
-
+     {"server_object", &Server_Object},
+     {"post_object", &Post_Object},
+     {"printer_name", &Slrn_Printer_Name},
      {NULL, NULL}
 };
 
@@ -658,48 +700,48 @@ static Color_Handle_Type Color_Handles[] = /*{{{*/
      {"grouplens_display",GROUPLENS_DISPLAY_COLOR,"blue","white", 0},
      {"tilde",		SLRN_TILDE_COLOR,"green",	"white", SLTT_BOLD_MASK},
      {"header_name",SLRN_HEADER_KEYWORD_COLOR,"green", "white", SLTT_BOLD_MASK},
+     {"response_char",RESPONSE_CHAR_COLOR,"green", "white", SLTT_BOLD_MASK},
+     {"box", BOX_COLOR,"black", "white", 0},
+     {"frame", FRAME_COLOR,"yellow", "blue", SLTT_REV_MASK},
+     {"selection", SELECT_COLOR,"yellow", "blue", SLTT_BOLD_MASK},
+   
      {NULL, -1, NULL, NULL, 0}
 };
 
 /*}}}*/
 
+int slrn_set_object_color (char *name, char *fg, char *bg)
+{
+   Color_Handle_Type *ct = Color_Handles;
+
+   while (ct->name != NULL)
+     {
+	if (!strcmp (ct->name, name))
+	  {
+	     SLtt_set_color (ct->value, name, fg, bg);
+	     return 0;
+	  }
+	ct++;
+     }
+   
+   slrn_error ("%s is not a color object", name);
+   return -1;
+}
+
 static int color_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 {
    char *what = table->string_args[1];
    char *fg = table->string_args[2];
    char *bg = table->string_args[3];
-   Color_Handle_Type *ct = Color_Handles;
    
    (void) argc;   
-   
-   while (ct->name != NULL)
-     {
-	if (!strcmp (ct->name, what))
-	  {
-	     SLtt_set_color (ct->value, what, fg, bg);
-	     return 0;
-	  }
-	ct++;
-     }
-   exit_unknown_object ();
+   if (-1 == slrn_set_object_color (what, fg, bg))
+     exit_unknown_object ();
    return 0;
 }
 
 /*}}}*/
 
-#if SLANG_VERSION < 9932
-# ifdef __os2__
-void SLtt_set_mono (int a, char *b, SLtt_Char_Type c) /*{{{*/
-{
-   (void) a;
-   (void) b;
-   (void) c;
-}
-
-/*}}}*/
-# endif
-#endif
-
 static int mono_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 {
    char *what = table->string_args[1];
@@ -764,12 +806,12 @@ User_Info_Variable_Type;
 
 static User_Info_Variable_Type User_Info_Variables[] = /*{{{*/
 {
-     {"username", &Slrn_User_Info.username, 0},
      {"hostname", (char **)Slrn_User_Info.host, MAX_HOST_NAME_LEN},
      {"scorefile", &Slrn_Score_File, 0},
-   
-#define USER_INFO_OBSOLETE_STARTS_HERE 3
+
+#define USER_INFO_OBSOLETE_STARTS_HERE 2
    /* The following are obsolete and will not be supported in the future. */
+     {"username", &Slrn_User_Info.username, 0},
      {"realname", &Slrn_User_Info.realname, 0},
      {"replyto", &Slrn_User_Info.replyto, 0},
      {"organization", &Slrn_User_Info.org, 0},
@@ -976,7 +1018,7 @@ int slrn_get_authorization (char *host, char **name, char **pass) /*{{{*/
    if (*s->username == 0)
      {
 	*buf = 0;
-	if (-1 == slrn_read_input ("Username", NULL, buf, 1))
+	if (-1 == slrn_read_input ("Username", NULL, buf, 1, 0))
 	  return -1;
 	SLFREE (s->username);
 	s->username = slrn_safe_strmalloc (buf);
@@ -985,7 +1027,7 @@ int slrn_get_authorization (char *host, char **name, char **pass) /*{{{*/
    if (*s->password == 0)
      {
 	*buf = 0;
-	if (-1 == slrn_read_input_no_echo ("Password", NULL, buf, 1))
+	if (-1 == slrn_read_input_no_echo ("Password", NULL, buf, 1, 0))
 	  return -1;
 	SLFREE (s->password);
 	s->password = slrn_safe_strmalloc (buf);
@@ -1012,7 +1054,6 @@ static void slrn_init_modes (void) /*{{{*/
 void slrn_startup_initialize (void) /*{{{*/
 {
    Color_Handle_Type *h;
-   char *server;
    
    slrn_init_modes ();
    SLang_init_case_tables ();
@@ -1027,41 +1068,29 @@ void slrn_startup_initialize (void) /*{{{*/
 	h++;
      }
 
-#ifndef __os2__
+#if !defined(IBMPC_SYSTEM)
    /* We are not using the blink characters (unless in mono) */
    SLtt_Blink_Mode = 0;
 #endif
    
-   switch (Slrn_Server_Id)
-     {
-      case SLRN_SERVER_ID_NNTP:
-	server = "NNTP";
-	break;
-	
-      case SLRN_SERVER_ID_SPOOL:
-	server = "SPOOL";
-	break;
-	
-      default:
-	server = NULL;
-     }
-   
-   if ((server != NULL) 
-       && (0 == SLdefine_for_ifdef (server)))
-     {
-	slrn_exit_error ("Unable to add preprocessor name %s.", server);
-     }
+   (void) slrn_set_header_format (0, "%F%-5S%G%-5l:[%12r]%t%s");
+   (void) slrn_set_header_format (1, "%F%G%-5l:[%12r]%t%s");
+   (void) slrn_set_header_format (2, "%F%-5l:%t%s");
+   (void) slrn_set_header_format (3, "%F%-5S%-5l:%t%50s %r");
 }
 
 
 /*}}}*/
 
-void slrn_read_startup_file (char *name) /*{{{*/
+int slrn_read_startup_file (char *name) /*{{{*/
 {
    FILE *fp;
    char file [SLRN_MAX_PATH_LEN];
-   char line [256];
+   char line [512];
    SLPreprocess_Type pt;
+   int save_this_line_num;
+   char *save_this_file;
+   char *save_this_line;
 
    if (-1 == slrn_init_readline ())
      {
@@ -1076,16 +1105,20 @@ void slrn_read_startup_file (char *name) /*{{{*/
    
    
    fp = slrn_open_home_file (name, "r", file, 0);
-   if (fp == NULL) return;
+   if (fp == NULL) return -1;
    
    slrn_message ("Reading startup file %s.", file);
+   
+   save_this_file = This_File;
+   save_this_line = This_Line;
+   save_this_line_num = This_Line_Num;
 
    This_File = file;
    This_Line = line;
+   This_Line_Num = 0;
    
    Slrn_Cmd_Table.table = Slrn_Startup_File_Cmds;
    
-   This_Line_Num = 0;
    while (NULL != fgets (line, sizeof(line) - 1, fp))
      {
 	This_Line_Num++;
@@ -1097,10 +1130,39 @@ void slrn_read_startup_file (char *name) /*{{{*/
    slrn_fclose (fp);
    
    SLprep_close_prep (&pt);
+   
+   if ((Server_Object != NULL)
+       && (-1 == (Slrn_Default_Server_Obj = slrn_map_name_to_object_id (0, Server_Object))))
+     slrn_exit_error ("Server object '%s' is not supported.", Server_Object);
+   
+   if ((Post_Object != NULL) 
+       && (-1 == (Slrn_Default_Post_Obj = slrn_map_name_to_object_id (1, Post_Object))))
+     slrn_exit_error ("Post object '%s' is not supported.", Post_Object);
+   
+   This_File = save_this_file;
+   This_Line = save_this_line;
+   This_Line_Num = save_this_line_num;
+   return 0;
 }
 
 /*}}}*/
 
+static int include_file_fun (int argc, SLcmd_Cmd_Table_Type *table)
+{
+   char *file;
+   
+   (void) argc;
+   
+   file = table->string_args [1];
+
+   if (-1 == slrn_read_startup_file (file))
+     slrn_exit_error ("%s: line %d:\n%sFile not opened: %s",
+		      This_File, This_Line_Num, This_Line, file);
+
+   return 0;
+}
+
+     
 static int interpret_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 {
 #if SLRN_HAS_SLANG
