@@ -1,8 +1,26 @@
-/*  Copyright (c) 1995 John E. Davis (davis@space.mit.edu)
- *  All rights reserved.
+/* -*- mode: C; mode: fold; -*- */
+/* Copyright (c) 1998 John E. Davis (davis@space.mit.edu)
+ *
+ * This file is part of slrn.
+ *
+ * Slrn is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ * 
+ * Slrn is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with Slrn; see the file COPYING.  If not, write to the Free
+ * Software Foundation, 59 Temple Place - Suite 330, 
+ * Boston, MA  02111-1307, USA.
  */
+
 #include "config.h"
-#include "features.h"
+#include "slrnfeat.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -19,95 +37,87 @@
 # include <stdlib.h>
 #endif
 
+#include <slang.h>
+
 #define MAX_ARTICLE_LINE_LEN 4096
+#include "uudecode.h"
+#include "util.h"
+#include "ttymsg.h"
+#include "slrndir.h"
 
 #ifdef STANDALONE
-# define SLRN_ERROR(x) fprintf(stderr, (x))
-# define SLRN_MESSAGE(x) fprintf(stdout, (x))
-# define SLRN_ERROR_1(x,y) fprintf(stderr, (x), (y))
-# define SLRN_MESSAGE_1(x,y) fprintf(stdout, (x), (y))
-# define SLRN_MESSAGE_2(x,y,z) fprintf(stdout, (x), (y), (z))
-# define SLRN_POPEN popen
-# define SLRN_PCLOSE pclose
-# define slrn_fclose slrn_fclose_standalone
-# define UPPER_CASE(x) ((x) >= 'a') && ((x) <= 'z') ? (x) - 32 : (x)
-# define LOWER_CASE(x) ((x) >= 'A') && ((x) <= 'Z') ? (x) + 32 : (x)
-static int slrn_fclose_standalone (FILE *fp)
+# include <stdarg.h>
+
+# if SLRN_HAS_PIPING
+#  define SLRN_POPEN uudecode_popen
+#  define SLRN_PCLOSE uudecode_pclose
+# endif
+# define slrn_message_now slrn_tty_message
+# define slrn_message slrn_tty_message
+
+static FILE *uudecode_popen (char *cmd, char *mode) /*{{{*/
 {
-   if (0 == fclose (fp)) return 0;
-   SLRN_ERROR ("Error closing file.  File system full?");
-   return -1;
+# if SLRN_HAS_PIPING
+   return popen (cmd, mode);
+# else
+   slrn_error ("Piping not implemented on this system.");
+   return NULL;
+# endif
 }
-   
+
+/*}}}*/
+
+static int uudecode_pclose (FILE *fp) /*{{{*/
+{
+# if SLRN_HAS_PIPING
+   return pclose (fp);
+# else
+   return -1;
+# endif
+}
+
+/*}}}*/
+
 #else
+# include <slang.h>
+# include "jdmacros.h"
 # include "slrn.h"
 # include "misc.h"
 # include "uudecode.h"
-# define SLRN_ERROR slrn_error
-# define SLRN_ERROR_1 slrn_error
-# define SLRN_MESSAGE(x) slrn_message(x); slrn_refresh ()
-# define SLRN_MESSAGE_1(x,y) slrn_message((x), (y)); slrn_smg_refresh ()
-# define SLRN_MESSAGE_2(x,y,z) slrn_message((x), (y), (z)); slrn_smg_refresh ()
 # define SLRN_POPEN slrn_popen
 # define SLRN_PCLOSE slrn_pclose
 char *Slrn_Decode_Directory;
 #endif
 
+#ifdef VMS
+# include "vms.h"
+#endif
 
-
-
-/* Note!!!  These routines assume a flat address space !! */
-int slrn_case_strncmp (unsigned char *a, register unsigned char *b, register unsigned int n)
+static int uudecode_fclose (FILE *fp, FILE *pipe_fp) /*{{{*/
 {
-   register unsigned char cha, chb, *bmax;
-   register int diff = a - b;
+   if ((fp == pipe_fp) || (fp == NULL))
+     return 0;
    
-   bmax = b + n;
-   while (b < bmax)
-     {
-	cha = UPPER_CASE(b[diff]);
-	chb = UPPER_CASE(*b);
-	if (cha != chb)
-	  {
-	     return (int) cha - (int) chb;
-	  }
-	else if (chb == 0) return 0;
-	b++;
-     }
-   return 0;
+#ifdef STANDALONE
+   if (0 == fclose (fp)) return 0;
+   slrn_error ("Error closing file.  File system full?");
+   return -1;
+#else
+   return slrn_fclose (fp);
+#endif
 }
 
-int slrn_case_strcmp (unsigned char *a, register unsigned char *b)
-{
-   register unsigned char cha, chb;
-   register int diff = a - b;
-   
-   while (1)
-     {
-	cha = UPPER_CASE(b[diff]);
-	chb = UPPER_CASE(*b);
-	if (cha != chb)
-	  {
-	     return (int) cha - (int) chb;
-	  }
-	else if (chb == 0) break;
-	b++;
-     }
-   return 0;
-}
+/*}}}*/
 
-
-static int unpack_as_shell_archive (FILE *fp, char *buf, int size)
+static int unpack_as_shell_archive (FILE *fp, char *buf, int size) /*{{{*/
 {
-#if SLRN_HAS_PIPING
-   
    FILE *pp;
    
    size--;
    pp = SLRN_POPEN ("/bin/sh", "w");
    if (pp == NULL)
      {
-	SLRN_ERROR ("Unable to open /bin/sh\n");
+	slrn_error ("Unable to open /bin/sh\n");
 	return -1;
      }
    
@@ -120,38 +130,42 @@ static int unpack_as_shell_archive (FILE *fp, char *buf, int size)
    
    if (-1 == SLRN_PCLOSE (pp))
      {
-	SLRN_ERROR ("Error encountered while processing shell archive.\n");
+	slrn_error ("Error encountered while processing shell archive.\n");
 	return -1;
      }
    return 0;
-#else
-   slrn_error ("Piping not implemented on this system.");
-   return -1;
-#endif				       /* HAS_PIPING */
 }
 
+/*}}}*/
 
-
-static FILE *open_output_file (char *name, char *type)
+static FILE *open_output_file (char *name, char *type, int mode, FILE *use_this) /*{{{*/
 {
    FILE *fp;
    
+   if (use_this != NULL)
+     return use_this;
+   
    fp = fopen (name, "wb");
 	
    if (fp == NULL)
      {
-	SLRN_ERROR_1("Unable to create %s\n", name);
+	slrn_error("Unable to create %s\n", name);
 	return NULL;
      }
    
-   SLRN_MESSAGE_2("creating %s (%s)\n", name, type);
+   slrn_message_now ("creating %s (%s)\n", name, type);
+   
+   if (mode != -1) 
+     chmod (name, mode);
    
    return fp;
 }
 
+/*}}}*/
+
 static unsigned char Base64_Table [256];
    
-static void initialize_base64 (void)
+static void initialize_base64 (void) /*{{{*/
 {
    int i;
    
@@ -164,11 +178,13 @@ static void initialize_base64 (void)
    Base64_Table['/'] = 63;
 }
 
+/*}}}*/
+
 /* returns 0 if entire line was decoded, or 1if line appears to be padded, or
  * -1 if line looks bad.
  */
 /* Calling routine guarantees at least 4 characters in line and multiple of 4 */
-static int base64_decode_line (char *line, FILE *fp)
+static int base64_decode_line (char *line, FILE *fp) /*{{{*/
 {
    unsigned char ch;
    unsigned char ch1;
@@ -237,6 +253,8 @@ static int base64_decode_line (char *line, FILE *fp)
    return 0;
 }
 
+/*}}}*/
+
 	       
 	
 static char Base64_Filename[256];
@@ -245,7 +263,7 @@ static int Base64_Unknown_Number;
 /* Returns 1 if padded line which indicates end of encoded file, 0 if
  * decoded something or -1 if nothing.
  */
-static int decode_base64 (FILE *fp, FILE *fpout, char *line, unsigned int buflen)
+static int decode_base64 (FILE *fp, FILE *fpout, char *line, unsigned int buflen) /*{{{*/
 {
    int decoding = 0;
    unsigned int len;
@@ -288,13 +306,17 @@ static int decode_base64 (FILE *fp, FILE *fpout, char *line, unsigned int buflen
    return 0;
 }
 
-static char *skip_whitespace (char *p)
+/*}}}*/
+
+static char *skip_whitespace (char *p) /*{{{*/
 {
    while ((*p == ' ') || (*p == '\t') || (*p == '\n')) p++;
    return p;
 }
 
-static char *skip_header_string (char *p, char *name)
+/*}}}*/
+
+static char *skip_header_string (char *p, char *name) /*{{{*/
 {
    unsigned int len;
    
@@ -307,8 +329,10 @@ static char *skip_header_string (char *p, char *name)
    return p + len;
 }
 
+/*}}}*/
 
-static char *skip_beyond_name_eqs (char *p, char *name)
+
+static char *skip_beyond_name_eqs (char *p, char *name) /*{{{*/
 {
    int ch;
    unsigned int len;
@@ -339,7 +363,9 @@ static char *skip_beyond_name_eqs (char *p, char *name)
      }
 }
 
-static int parse_name_eqs_int (char *p, char *name, int *val)
+/*}}}*/
+
+static int parse_name_eqs_int (char *p, char *name, int *val) /*{{{*/
 {
    int ch;
    int ival;
@@ -359,7 +385,9 @@ static int parse_name_eqs_int (char *p, char *name, int *val)
    return 0;
 }
 
-static int parse_name_eqs_string (char *p, char *name, char *str, unsigned int len)
+/*}}}*/
+
+static int parse_name_eqs_string (char *p, char *name, char *str, unsigned int len) /*{{{*/
 {
    char ch;
    char *pmax;
@@ -392,14 +420,18 @@ static int parse_name_eqs_string (char *p, char *name, char *str, unsigned int l
    return 0;
 }
 
+/*}}}*/
+
    
 				   
-static void parse_content_disposition (char *p)
+static void parse_content_disposition (char *p) /*{{{*/
 {
    (void) parse_name_eqs_string (p, "filename", Base64_Filename, sizeof (Base64_Filename));
 }
 
-static int is_encoding_base64 (char *p)
+/*}}}*/
+
+static int is_encoding_base64 (char *p) /*{{{*/
 {
    p = skip_whitespace (p);
    if (slrn_case_strncmp ((unsigned char *)p, (unsigned char *)"base64", 6))
@@ -407,6 +439,8 @@ static int is_encoding_base64 (char *p)
    return 1;
 }
 
+/*}}}*/
+
 
    
 /* Looking for message/partial; id="bla-bla"; number=10; total=20
@@ -446,16 +480,23 @@ static int is_encoding_base64 (char *p)
  *  Note that part numbering begins with 1, not 0.
  */
 
-typedef struct 
+typedef struct /*{{{*/
 {
    int number;		       /* >= 1 */
    int total;		       /* not required!! */
    char id[256];		       /* required */
 }
+
+/*}}}*/
+
 Mime_Message_Partial_Type;
 
-static int parse_content_type (char *p, Mime_Message_Partial_Type *mpt)
+static int parse_content_type (char *p, Mime_Message_Partial_Type *mpt) /*{{{*/
 {
+#if 1
+   (void) parse_name_eqs_string (p, "name", 
+				 Base64_Filename, sizeof (Base64_Filename));
+#endif
    if ((NULL == (p = skip_header_string (p, "message")))
        || (NULL == (p = skip_header_string (p, "/")))
        || (NULL == (p = skip_header_string (p, "partial")))
@@ -477,7 +518,9 @@ static int parse_content_type (char *p, Mime_Message_Partial_Type *mpt)
    return 0;
 }
 
-static void read_in_whole_header_line (char *line, FILE *fp)
+/*}}}*/
+
+static void read_in_whole_header_line (char *line, FILE *fp) /*{{{*/
 {
    unsigned int len;
    unsigned int space;
@@ -506,8 +549,9 @@ static void read_in_whole_header_line (char *line, FILE *fp)
      }
 }
 
+/*}}}*/
    
-static int check_and_decode_base64 (FILE *fp)
+static int check_and_decode_base64 (FILE *fp, FILE *pipe_fp) /*{{{*/
 {
    char *p;
    int found_base64_signature = 0;
@@ -537,7 +581,7 @@ static int check_and_decode_base64 (FILE *fp)
 		    }
 		  
 		  if (fpout == NULL)
-		    fpout = open_output_file (Base64_Filename, "base64");
+		    fpout = open_output_file (Base64_Filename, "base64", -1, pipe_fp);
 		  
 		  if (fpout == NULL) return -1;
 		  
@@ -552,7 +596,7 @@ static int check_and_decode_base64 (FILE *fp)
 		      || (part != current_pmt->number)
 		      || (ret == 1))
 		    {
-		       fclose (fpout);
+		       uudecode_fclose (fpout, pipe_fp);
 		       fpout = NULL;
 		       found_base64_signature = 0;
 		       current_pmt = NULL;
@@ -581,7 +625,7 @@ static int check_and_decode_base64 (FILE *fp)
 		    {
 		       if (fpout != NULL) 
 			 {
-			    fclose (fpout);
+			    uudecode_fclose (fpout, pipe_fp);
 			    fpout = NULL;
 			    current_pmt = NULL;
 			    *Base64_Filename = 0;
@@ -616,7 +660,7 @@ static int check_and_decode_base64 (FILE *fp)
 	       {
 		  if (fpout != NULL)
 		    {
-		       fclose (fpout);
+		       uudecode_fclose (fpout, pipe_fp);
 		       fpout = NULL;
 		       *Base64_Filename = 0;
 		       found_base64_signature = 0;
@@ -629,12 +673,13 @@ static int check_and_decode_base64 (FILE *fp)
 	  }
      }
    
-   if (fpout != NULL) fclose (fpout);
+   uudecode_fclose (fpout, pipe_fp);
    return 0;
 }
 
+/*}}}*/
 
-static int parse_uuencode_begin_line (unsigned char *b, char **file, int *mode)
+static int parse_uuencode_begin_line (unsigned char *b, char **file, int *mode) /*{{{*/
 {
    int m = 0;
    unsigned char *bmax;
@@ -660,7 +705,10 @@ static int parse_uuencode_begin_line (unsigned char *b, char **file, int *mode)
    return 0;
 }
 
-static int check_and_uudecode (FILE *fp)
+/*}}}*/
+
+static int check_and_uudecode (FILE *fp, FILE *pipe_fp, 
+			       int no_slop) /*{{{*/
 {
    FILE *outfp = NULL;
    char buf[MAX_ARTICLE_LINE_LEN];
@@ -669,7 +717,7 @@ static int check_and_uudecode (FILE *fp)
      {
 	char *file;
 	int mode = 0;
-	
+
 	/* We are looking for 'begin ddd filename' */
 	if (strncmp (buf, "begin ", 6))
 	  {
@@ -688,18 +736,13 @@ static int check_and_uudecode (FILE *fp)
 	if (-1 == parse_uuencode_begin_line ((unsigned char *)buf + 6, &file, &mode))
 	  continue;
 
-	outfp = open_output_file (file, "uuencoded");
-	
-	if (outfp == NULL)
-	  {
-	     return -1;
-	  }
-	chmod (file, mode);
-	
+	if (NULL == (outfp = open_output_file (file, "uuencoded", mode, pipe_fp)))
+	  return -1;
+
 	/* Now read parts of the file in. */
 	while (fgets (buf, sizeof(buf) - 1, fp) != NULL)
 	  {
-	     unsigned int len;
+	     unsigned int len, write_len;
 	     unsigned int buflen;
 	     unsigned char out[60], *outp, *outmax, *b, *bmax;
 	     
@@ -708,13 +751,13 @@ static int check_and_uudecode (FILE *fp)
 		  /* we may be on the last line before the end.  Lets check */
 		  if (NULL == fgets (buf, sizeof(buf) - 1, fp))
 		    {
-		       SLRN_ERROR ("Unexpected end of file.\n");
+		       slrn_error ("Unexpected end of file.\n");
 		       return -1;
 		    }
 		  
 		  if (!strcmp (buf, "end\n"))
 		    {
-		       slrn_fclose (outfp);
+		       uudecode_fclose (outfp, pipe_fp);
 		       outfp = NULL;
 		       break;
 		    }
@@ -722,22 +765,27 @@ static int check_and_uudecode (FILE *fp)
 	     
 	     /* Now perform some sanity checking */
 	     len = *buf;
-	     
-	     if ((len <= ' ') 
+
+	     no_slop = 0;
+	     if ((len <= ' ')
+		 || (len >= (60 + ' '))
 		 || ((buflen = strlen (buf)) > 62) || (buflen < 2)
-		 || ((buflen - 2) % 4) != 0)
+		 || (no_slop
+		     && (((buflen - 2) % 4) != 0)))
 	       {
+#if 0
+		  fprintf (stderr, "%s", buf);
+#endif
 		  if (strncmp (buf, "begin ", 6)
-		      || (-1 == parse_uuencode_begin_line ((unsigned char *)buf + 6, 
+		      || (-1 == parse_uuencode_begin_line ((unsigned char *)buf + 6,
 							   &file, &mode)))
 		    continue;
-		  
-		  if (outfp == NULL) slrn_fclose (outfp);
-		  outfp = open_output_file (file, "uuencoded");
-		  if (outfp == NULL)
-		    {
-		       return -1;
-		    }
+
+		  uudecode_fclose (outfp, pipe_fp);
+
+		  if (NULL == (outfp = open_output_file (file, "uuencoded", mode, pipe_fp)))
+		    return -1;
+		    
 		  continue;
 	       }
 	     
@@ -756,9 +804,21 @@ static int check_and_uudecode (FILE *fp)
 	       }
 	     else
 	       {
-		  if (buflen != ((len + 2) / 3) * 4) continue;
+		  unsigned int buflen1 = ((len + 2) / 3) * 4; 
+		  if (buflen != buflen1)
+		    {
+		       if (no_slop) continue;
+#if 0
+#ifdef STANDALONE
+		       slrn_message_now ("uuencode line appears corrupt: \"%s\"", 
+					 buf);
+#endif
+#endif
+		       continue;
+		    }
 	       }
 	     
+	     
 	     b = (unsigned char *) buf + 1;
 	     bmax = b + buflen;
 	     
@@ -793,10 +853,11 @@ static int check_and_uudecode (FILE *fp)
 		  b += 4;
 	       }
 	     
-	     if (len != fwrite ((char *) out, 1, len, outfp))
+	     if (len != (write_len = fwrite ((char *) out, 1, len, outfp)))
 	       {
-		  SLRN_ERROR ("write to file failed.\n");
-		  slrn_fclose (outfp);
+		  slrn_error ("write to file failed (%u/%u bytes)\n", 
+			      write_len, len);
+		  uudecode_fclose (outfp, pipe_fp);
 		  return -1;
 	       }
 	  }
@@ -807,96 +868,189 @@ static int check_and_uudecode (FILE *fp)
    
    if (outfp != NULL) 
      {
-	slrn_fclose (outfp);
+	uudecode_fclose (outfp, pipe_fp);
 	outfp = NULL;
      }
    return 0;
 }
 
+/*}}}*/
 
-int uudecode_file (char *file, char *newdir, int base64_only)
+static int decode_best_guess (FILE *fp, FILE *pipe_fp, int no_slop) /*{{{*/
+{
+   int ch;
+   int nl_seen = 1;
+   char *mime_ptr;
+   
+   while (EOF != (ch = getc (fp)))
+     {
+	if (ch == '\n')
+	  {
+	     nl_seen = 1;
+	     continue;
+	  }
+	
+	if (nl_seen == 0)
+	  continue;
+	
+	if (ch == 'b')
+	  {
+	     ungetc (ch, fp);
+	     return check_and_uudecode (fp, pipe_fp, no_slop);
+	  }
+	
+	nl_seen = 0;
+	
+	mime_ptr = "mime-version:";
+	while (1)
+	  {
+	     if ((ch | 0x20) != *mime_ptr) 
+	       break;
+	     
+	     mime_ptr++;
+	     if (*mime_ptr == ':')
+	       {
+		  return check_and_decode_base64 (fp, pipe_fp);
+	       }
+	     
+	     ch = getc (fp);
+	     
+	     if (ch == EOF) break;
+	     if (ch == '\n')
+	       {
+		  nl_seen = 1;
+		  break;
+	       }
+	  }
+     }
+   return 0;
+}
+
+/*}}}*/
+
+int slrn_uudecode_file (char *file, char *newdir, /*{{{*/
+			int base64_only,
+			FILE *pipe_fp)
 {
    FILE *fp;
-#ifdef __os2__
-   char olddir_buf[256];
-#endif
+#ifndef STANDALONE
+   char olddir_buf[SLRN_MAX_PATH_LEN + 1];
+   char newdir_buf[SLRN_MAX_PATH_LEN + 1];
+#endif	
    int ret;
-     
+   int no_slop = 0;		       /* Set to 1 for up-to-spec files */
+
+#ifndef STANDALONE
    if (newdir == NULL)
      {
-#ifndef STANDALONE
-	char newdir_buf[256];
-#ifdef __os2__
-	if (getwd(olddir_buf) == 0) 
-	  {
-	     SLRN_ERROR_1 ("Unable to save old directory name: %s\n", olddir_buf);
-	     return -1;
-	  }
-#endif
-	if (Slrn_Decode_Directory == NULL)
-	  Slrn_Decode_Directory = "News";
 	newdir = Slrn_Decode_Directory;
+	if (newdir == NULL)
+	  newdir = "News";
+
 	slrn_make_home_dirname (newdir, newdir_buf);
 	newdir = newdir_buf;
-#endif
      }
+#endif
+
+#ifndef STANDALONE
+   if (NULL == slrn_getcwd (olddir_buf, sizeof(olddir_buf)))
+     return -1;
+#endif
    
    if ((newdir != NULL)
-       && (-1 == chdir(newdir)))
-     {
-	SLRN_ERROR_1 ("Unable to chdir to %s.\n", newdir);
-	return -1;
-     }
+       && (-1 == slrn_chdir(newdir)))
+     return -1;
    
    if (file == NULL) fp = stdin;
    else fp = fopen (file, "r");
    
    if (fp == NULL)
      {
-	SLRN_ERROR_1("Unable to open %s.\n", file);
+	slrn_error("Unable to open %s.\n", file);
+#ifndef STANDALONE
+	(void) slrn_chdir (olddir_buf);
+#endif
 	return -1;
      }
 
-   if (base64_only == 0) 
+   if (base64_only == -1)
+     ret = decode_best_guess (fp, pipe_fp, no_slop);
+   else if (base64_only == 0) 
      {
-	ret = check_and_uudecode (fp);
+	ret = check_and_uudecode (fp, pipe_fp, no_slop);
 	if ((ret == 0) && (fp != stdin))
 	  {
 	     rewind (fp);
-	     ret = check_and_decode_base64 (fp);
+	     ret = check_and_decode_base64 (fp, pipe_fp);
 	  }
      }
-   else ret = check_and_decode_base64 (fp);
+   else ret = check_and_decode_base64 (fp, pipe_fp);
    
    if (fp != stdin)
-     slrn_fclose (fp);
-
-#ifdef __os2__
-   if (-1 == chdir(olddir_buf))
+     fclose (fp);
+   
+#ifdef STANDALONE
+   if (pipe_fp == NULL) slrn_message ("No more files found.\n");
+#endif
+   
+#ifndef STANDALONE
+   if (-1 == slrn_chdir (olddir_buf))
      {
-	SLRN_ERROR_1 ("Unable to chdir back to %s.\n", olddir_buf);
-	return -1;
+	slrn_error ("Unable to chdir back to %s.\n", olddir_buf);
+	ret = -1;
      }
 #endif
 
-#ifdef STANDALONE
-   SLRN_MESSAGE ("No more files found.\n");
-#endif
    return ret;
 }
 
+/*}}}*/
+
 #ifdef STANDALONE
-static void usage (void)
+static void usage (void) /*{{{*/
 {
-   fprintf (stderr, "Usage: uudecode [-64] [filename ...]\n");
+   fprintf (stderr, "Usage: uudecode [-64] [--stdout] [--guess] [filename ...]\n");
 }
 
-int main (int argc, char **argv)
+/*}}}*/
+
+void slrn_error (char *fmt, ...)
+{
+   va_list ap;
+   
+   if (fmt != NULL)
+     {
+	va_start (ap, fmt);
+	vfprintf (stderr, fmt, ap);
+	va_end (ap);
+     }
+   fputc ('\n', stderr);
+}
+
+void slrn_exit_error (char *fmt, ...)
+{
+   va_list ap;
+   
+   if (fmt != NULL)
+     {
+	va_start (ap, fmt);
+	vfprintf (stderr, fmt, ap);
+	va_end (ap);
+     }
+
+   fputc ('\n', stderr);
+   exit (1);
+}
+
+int main (int argc, char **argv) /*{{{*/
 {
    int i;
    int base64_only = 0;
+   FILE *pipe_fp = NULL;
 
-   if (argc > 1)
+   SLang_init_case_tables ();
+
+   while (argc > 1)
      {
 	if (!strcmp ("--help", argv[1]))
 	  {
@@ -910,13 +1064,26 @@ int main (int argc, char **argv)
 	     argc--;
 	     argv++;
 	  }
+	else if (!strcmp ("--stdout", argv[1]))
+	  {
+	     pipe_fp = stdout;
+	     argc--;
+	     argv++;
+	  }
+	else if (!strcmp ("--guess", argv[1]))
+	  {
+	     base64_only = -1;
+	     argc--;
+	     argv++;
+	  }
+	else break;
      }
    
    if (argc <= 1)
      {
-	if (!isatty (fileno(stdin)))
+	if (0 == isatty (fileno(stdin)))
 	  {
-	     return uudecode_file (NULL, NULL, base64_only);
+	     return slrn_uudecode_file (NULL, NULL, base64_only, pipe_fp);
 	  }
 	usage ();
 	return 1;
@@ -924,10 +1091,13 @@ int main (int argc, char **argv)
    
    for (i = 1; i < argc; i++)
      {
-	uudecode_file (argv[i], NULL, base64_only);
+	slrn_uudecode_file (argv[i], NULL, base64_only, pipe_fp);
      }
    return 0;
 }
+
+/*}}}*/
+
 #endif
 
 
