@@ -13,7 +13,7 @@
  */
 
 #include "config.h"
-#include "features.h"
+#include "slrnfeat.h"
 
 #ifdef VMS
 
@@ -26,26 +26,15 @@
 #include <string.h>
 #include <stdlib.h>
 #include <file.h>
-#ifdef __GNUC__
-#include <sys$routines.h>
-#else
-#include <starlet.h>
-#endif
 
-char *
-getlogin (void)
+#include "vms.h"
+
+char *slrn_vms_getlogin (void)
 {
-   char *p;
-   
-   if ((p = getenv ("USER")) == NULL) 
-     {
-	return ( (char *) 0);
-     }
-   
-   return (p);
+   return getenv ("USER");
 }
 
-struct dsc$descriptor *c$dsc(char *c$_str)
+static struct dsc$descriptor *c$dsc(char *c$_str)
 {
    static struct dsc$descriptor c$_tmpdesc;
    
@@ -56,7 +45,7 @@ struct dsc$descriptor *c$dsc(char *c$_str)
    return(&c$_tmpdesc);
 }
 
-char *get_uaf_fullname()
+char *slrn_vms_get_uaf_fullname (void)
 {
    static char uaf_owner[40];
    char loc_username[13];
@@ -96,7 +85,7 @@ char *get_uaf_fullname()
 }
 
 /* Converts "TOD_MCQUILLIN" to "Tod McQuillin" */
-char *fix_fullname(char *p)
+char *slrn_vms_fix_fullname(char *p)
 {
    int cc = 0;
    char *q = p;
@@ -130,24 +119,27 @@ char *fix_fullname(char *p)
    return (p);
 }
 
+#if __VMS_VER < 70000000
 FILE *
 popen (
        char *command,
        char *mode)
 {
-   return ((FILE *) 0);
+   return NULL;
 }
 
 
-void
+int
 pclose (FILE *pipe)
 {
-   return;
+   return 1;
 }
+#endif
 
 void tzset(void)
 {
    return;
 }
+#endif
 
 #endif /* VMS */
